@@ -22,7 +22,7 @@
 #include "arm64_printer.h"
 #include "dynarec_arm64_private.h"
 #include "dynarec_arm64_functions.h"
-#include "dynarec_arm64_helper.h"
+#include "../dynarec_helper.h"
 
 uintptr_t dynarec64_AVX_66_0F(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int ninst, vex_t vex, int* ok, int* need_epilog)
 {
@@ -273,7 +273,7 @@ uintptr_t dynarec64_AVX_66_0F(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip,
             GETGX(v0, 0);
             GETEXSD(q0, 0, 0);
             FCMPD(v0, q0);
-            FCOMI(x1, x2);
+            FCOMI(x1, x2, 0, v0, q0, 0);    //disable precise cmp
             break;
 
         case 0x50:
@@ -295,6 +295,29 @@ uintptr_t dynarec64_AVX_66_0F(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip,
                 BFIx(gd, x2, 3, 1);
             }
             break;
+        case 0x51:
+            INST_NAME("VSQRTPD Gx, Ex");
+            nextop = F8;
+            if(!box64_dynarec_fastnan) {
+                d0 = fpu_get_scratch(dyn, ninst);
+                d1 = fpu_get_scratch(dyn, ninst);
+            }
+            for(int l=0; l<1+vex.l; ++l) {
+                if(!l) { GETGX_empty_EX(v0, v1, 0); } else { GETGY_empty_EY(v0, v1); }
+                if(!box64_dynarec_fastnan) {
+                    // check if any input value was NAN
+                    VFCMEQQD(d0, v1, v1);    // 0 if NAN, 1 if not NAN
+                    VFSQRTQD(v0, v1);
+                    VFCMEQQD(d1, v0, v0);    // 0 => out is NAN
+                    VBICQ(d1, d0, d1);      // forget it in any input was a NAN already
+                    VSHLQ_64(d1, d1, 63);   // only keep the sign bit
+                    VORRQ(v0, v0, d1);      // NAN -> -NAN
+                } else {
+                    VFSQRTQD(v0, v1);
+                }
+            }
+            if(!vex.l) YMM0(gd);
+            break;
 
         case 0x54:
             INST_NAME("VANDPD Gx, Vx, Ex");
@@ -389,7 +412,7 @@ uintptr_t dynarec64_AVX_66_0F(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip,
             nextop = F8;
             GETEX(v1, 0, 0);
             GETGX_empty(v0);
-            if(box64_dynarec_fastround) {
+            if(box64_dynarec_fastround==2) {
                 FCVTXN(v0, v1);
             } else {
                 u8 = sse_setround(dyn, ninst, x1, x2, x4);
@@ -398,7 +421,7 @@ uintptr_t dynarec64_AVX_66_0F(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip,
             }
             if(vex.l) {
                 GETEY(v1);
-                if(box64_dynarec_fastround) {
+                if(box64_dynarec_fastround==2) {
                     FCVTXN2(v0, v1);
                 } else {
                     FCVTN2(v0, v1);
@@ -1136,7 +1159,7 @@ uintptr_t dynarec64_AVX_66_0F(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip,
             GETGX(v0, 0);
             if(rex.w) {
                 if(MODREG) {
-                    ed = xRAX + (nextop&7) + (rex.b<<3);
+                    ed = TO_NAT((nextop & 0x07) + (rex.b << 3));
                     VMOVQDto(ed, v0, 0);
                 } else {
                     addr = geted(dyn, addr, ninst, nextop, &ed, x1, &fixedaddress, &unscaled, 0xfff<<3, 7, rex, NULL, 0, 0);
@@ -1145,7 +1168,7 @@ uintptr_t dynarec64_AVX_66_0F(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip,
                 }
             } else {
                 if(MODREG) {
-                    ed = xRAX + (nextop&7) + (rex.b<<3);
+                    ed = TO_NAT((nextop & 0x07) + (rex.b << 3));
                     VMOVSto(ed, v0, 0);
                 } else {
                     addr = geted(dyn, addr, ninst, nextop, &ed, x1, &fixedaddress, &unscaled, 0xfff<<2, 3, rex, NULL, 0, 0);
@@ -1216,7 +1239,7 @@ uintptr_t dynarec64_AVX_66_0F(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip,
             if(v0!=v2) VMOVQ(v0, v2);
             if(MODREG) {
                 u8 = (F8)&7;
-                ed = xRAX+(nextop&7)+(rex.b<<3);
+                ed = TO_NAT((nextop & 7) + (rex.b << 3));
                 VMOVQHfrom(v0, u8, ed);
             } else {
                 SMREAD();
