@@ -19,9 +19,9 @@
 #include "arm64_printer.h"
 #include "dynarec_arm64_private.h"
 #include "dynarec_arm64_functions.h"
-#include "dynarec_arm64_helper.h"
+#include "../dynarec_helper.h"
 
-// emit SHL32 instruction, from s1 , shift s2, store result in s1 using s3 and s4 as scratch. s3 can be same as s2
+// emit SHL32 instruction, from s1 , shift s2, store result in s1 using s3 and s4 as scratch. s3 can be same as s2. s2 must be non-0
 void emit_shl32(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, int s2, int s3, int s4)
 {
     MAYUSE(s2);
@@ -35,15 +35,6 @@ void emit_shl32(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, int s2, int s3
     } else IFX(X_ALL) {
         SET_DFNONE(s4);
     }
-    IFX(X_OF) {
-        CMPSxw_U12(s2, 0);
-        Bcond(cNE, +8+((dyn->insts[ninst].x64.gen_flags&X_PEND)?4:0));
-        BFCw(xFlags, F_OF, 1);
-        IFX(X_PEND) {
-            STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
-        }
-        B_NEXT(cEQ);
-    }
     IFX(X_CF | X_OF) {
         MOV32w(s4, rex.w?64:32);
         SUBxw_REG(s4, s4, s2);
@@ -54,21 +45,28 @@ void emit_shl32(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, int s2, int s3
     IFX(X_PEND) {
         STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
     }
-    IFX(X_ZF) {
-        TSTxw_REG(s1, s1);
-        CSETw(s4, cEQ);
-        BFIw(xFlags, s4, F_ZF, 1);
-    }
-    IFX(X_SF) {
-        LSRxw(s4, s1, (rex.w)?63:31);
-        BFIx(xFlags, s4, F_SF, 1);
-    }
     IFX(X_OF) {
         CMPSxw_U12(s2, 1);   // if s2==1
-            IFX(X_SF) {} else {LSRxw(s4, s1, (rex.w)?63:31);}
-            EORxw_REG(s4, s4, xFlags);  // CF is set if OF is asked
-            CSELw(s4, s4, wZR, cEQ);
-            BFIw(xFlags, s4, F_OF, 1);
+            LSRxw(s4, s1, (rex.w)?63:31);
+            EORxw_REG(s3, s4, xFlags);  // CF is set if OF is asked
+            CSELw(s3, s3, wZR, cEQ);
+            BFIw(xFlags, s3, F_OF, 1);
+    }
+    int need_tst = 0;
+    IFX(X_ZF) need_tst = 1;
+    IFXNATIVE(X_SF, NF_SF) need_tst = 1;
+    if(need_tst) TSTxw_REG(s1, s1);
+    IFX(X_SF) {
+        IFNATIVE(NF_SF) {} else {
+            IFX(X_OF) {} else {LSRxw(s4, s1, (rex.w)?63:31);}
+            BFIw(xFlags, s4, F_SF, 1);
+        }
+    }
+    IFX(X_ZF) {
+        IFNATIVE(NF_EQ) {} else {
+            CSETw(s4, cEQ);
+            BFIw(xFlags, s4, F_ZF, 1);
+        }
     }
     if(box64_dynarec_test)
         IFX(X_AF) {
@@ -92,15 +90,6 @@ void emit_shl32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, i
     } else IFX(X_ALL) {
         SET_DFNONE(s4);
     }
-    if(c==0) {
-        IFX(X_OF) {
-            BFCw(xFlags, F_OF, 1);
-        }
-        IFX(X_PEND) {
-            STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
-        }
-        return;
-    }
     IFX(X_CF|X_OF) {
         LSRxw(s3, s1, (rex.w?64:32)-c);
         BFIxw(xFlags, s3, F_CF, 1);
@@ -110,14 +99,21 @@ void emit_shl32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, i
     IFX(X_PEND) {
         STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
     }
+    int need_tst = 0;
+    IFX(X_ZF) need_tst = 1;
+    IFXNATIVE(X_SF, NF_SF) need_tst = 1;
+    if(need_tst) TSTxw_REG(s1, s1);
     IFX(X_ZF) {
-        TSTxw_REG(s1, s1);
-        CSETw(s4, cEQ);
-        BFIw(xFlags, s4, F_ZF, 1);
+        IFNATIVE(NF_EQ) {} else {
+            CSETw(s4, cEQ);
+            BFIw(xFlags, s4, F_ZF, 1);
+        }
     }
     IFX(X_SF) {
-        LSRxw(s4, s1, (rex.w)?63:31);
-        BFIx(xFlags, s4, F_SF, 1);
+        IFNATIVE(NF_SF) {} else {
+            LSRxw(s4, s1, (rex.w)?63:31);
+            BFIw(xFlags, s4, F_SF, 1);
+        }
     }
     IFX(X_OF) {
         if(c==1) {
@@ -171,14 +167,21 @@ void emit_shr32(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, int s2, int s3
     IFX(X_PEND) {
         STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
     }
+    int need_tst = 0;
+    IFX(X_ZF) need_tst = 1;
+    IFXNATIVE(X_SF, NF_SF) need_tst = 1;
+    if(need_tst) TSTxw_REG(s1, s1);
     IFX(X_ZF) {
-        TSTxw_REG(s1, s1);
-        CSETw(s4, cEQ);
-        BFIw(xFlags, s4, F_ZF, 1);
+        IFNATIVE(NF_EQ) {} else {
+            CSETw(s4, cEQ);
+            BFIw(xFlags, s4, F_ZF, 1);
+        }
     }
     IFX(X_SF) {
-        LSRxw(s4, s1, (rex.w)?63:31);
-        BFIx(xFlags, s4, F_SF, 1);
+        IFNATIVE(NF_SF) {} else {
+            LSRxw(s4, s1, (rex.w)?63:31);
+            BFIx(xFlags, s4, F_SF, 1);
+        }
     }
     if(box64_dynarec_test)
         IFX(X_AF) {
@@ -220,14 +223,72 @@ void emit_shr32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, i
     IFX(X_PEND) {
         STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
     }
+    int need_tst = 0;
+    IFX(X_ZF) need_tst = 1;
+    IFXNATIVE(X_SF, NF_SF) need_tst = 1;
+    if(need_tst) TSTxw_REG(s1, s1);
     IFX(X_ZF) {
-        TSTxw_REG(s1, s1);
-        CSETw(s4, cEQ);
-        BFIw(xFlags, s4, F_ZF, 1);
+        IFNATIVE(NF_EQ) {} else {
+            CSETw(s4, cEQ);
+            BFIw(xFlags, s4, F_ZF, 1);
+        }
     }
     IFX(X_SF) {
-        // no sign if c>0
-        BFCw(xFlags, F_SF, 1);
+        IFNATIVE(NF_SF) {} else {
+            // no sign if c>0
+            BFCw(xFlags, F_SF, 1);
+        }
+    }
+    if(box64_dynarec_test)
+        IFX(X_AF) {
+            BFCw(xFlags, F_AF, 1);
+        }
+    IFX(X_PF) {
+        emit_pf(dyn, ninst, s1, s4);
+    }
+}
+
+// emit SAR32 instruction, from s1 , s2, store result in s1 using s3 and s4 as scratch, s2 can be same as s3
+void emit_sar32(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, int s2, int s3, int s4)
+{
+    MAYUSE(s2);
+    int64_t j64;
+    MAYUSE(j64);
+
+    IFX(X_PEND) {
+        STRxw_U12(s1, xEmu, offsetof(x64emu_t, op1));
+        STRxw_U12(s2, xEmu, offsetof(x64emu_t, op2));
+        SET_DF(s4, rex.w?d_sar64:d_sar32);
+    } else IFX(X_ALL) {
+        SET_DFNONE(s4);
+    }
+    IFX(X_CF) {
+        SUBxw_U12(s3, s2, 1);
+        ASRxw_REG(s3, s1, s3);
+        BFIw(xFlags, s3, 0, 1);
+    }
+    IFX(X_OF) {
+        BFCw(xFlags, F_OF, 1);
+    }
+    ASRxw_REG(s1, s1, s2);
+    IFX(X_PEND) {
+        STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
+    }
+    int need_tst = 0;
+    IFX(X_ZF) need_tst = 1;
+    IFXNATIVE(X_SF, NF_SF) need_tst = 1;
+    if(need_tst) TSTxw_REG(s1, s1);
+    IFX(X_ZF) {
+        IFNATIVE(NF_EQ) {} else {
+            CSETw(s4, cEQ);
+            BFIw(xFlags, s4, F_ZF, 1);
+        }
+    }
+    IFX(X_SF) {
+        IFNATIVE(NF_SF) {} else {
+            LSRxw(s4, s1, (rex.w)?63:31);
+            BFIx(xFlags, s4, F_SF, 1);
+        }
     }
     if(box64_dynarec_test)
         IFX(X_AF) {
@@ -259,17 +320,24 @@ void emit_sar32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, i
     IFX(X_PEND) {
         STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
     }
+    int need_tst = 0;
+    IFX(X_ZF) need_tst = 1;
+    IFXNATIVE(X_SF, NF_SF) need_tst = 1;
+    if(need_tst) TSTxw_REG(s1, s1);
     IFX(X_ZF) {
-        TSTxw_REG(s1, s1);
-        CSETw(s4, cEQ);
-        BFIw(xFlags, s4, F_ZF, 1);
+        IFNATIVE(NF_EQ) {} else {
+            CSETw(s4, cEQ);
+            BFIw(xFlags, s4, F_ZF, 1);
+        }
     }
     IFX(X_SF) {
-        LSRxw(s4, s1, (rex.w)?63:31);
-        BFIx(xFlags, s4, F_SF, 1);
+        IFNATIVE(NF_SF) {} else {
+            LSRxw(s4, s1, (rex.w)?63:31);
+            BFIx(xFlags, s4, F_SF, 1);
+        }
     }
     IFX(X_OF)
-        if(c==1) {
+        if(c==1 || box64_dynarec_test) {
             BFCw(xFlags, F_OF, 1);
     }
     if(box64_dynarec_test)
@@ -305,14 +373,14 @@ void emit_shl8(dynarec_arm_t* dyn, int ninst, int s1, int s2, int s3, int s4)
     IFX(X_PEND) {
         STRB_U12(s1, xEmu, offsetof(x64emu_t, res));
     }
-    COMP_ZFSF(s1, 8)
     IFX(X_OF) {
         CMPSw_U12(s2, 1);   // if s2==1
-            IFX2(X_SF, && !arm64_flagm) {} else {LSRw(s3, s1, 7);}
+            LSRw(s3, s1, 7);
             EORw_REG(s4, s3, xFlags);  // CF is set if OF is asked
             CSELw(s4, s4, wZR, cEQ);
             BFIw(xFlags, s4, F_OF, 1);
     }
+    COMP_ZFSF(s1, 8)
     if(box64_dynarec_test)
         IFX(X_AF) {
             BFCw(xFlags, F_AF, 1);
@@ -347,7 +415,7 @@ void emit_shl8c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int s
         COMP_ZFSF(s1, 8)
         IFX(X_OF) {
             if(c==1) {
-                IFX2(X_SF, && !arm64_flagm) {} else {LSRw(s3, s1, 7);}
+                IFX2(X_SF, && !arm64_flagm) {} else {LSRw(s3, s1, 7);}  //use COMP_ZFSF operation
                 EORw_REG(s4, s3, xFlags);  // CF is set if OF is asked
                 BFIw(xFlags, s4, F_OF, 1);
             } else {
@@ -361,29 +429,35 @@ void emit_shl8c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int s
             emit_pf(dyn, ninst, s1, s4);
         }
     } else {
+        IFXNATIVE(X_ZF|X_SF|X_CF, NF_EQ|NF_SF|NF_CF) {
+            SUBSw_REG(s1, s1, s1);
+        } else {
+            MOVw_REG(s1, xZR);
+        }
+        IFX(X_PEND) {
+            STRB_U12(s1, xEmu, offsetof(x64emu_t, res));
+        }
         IFX(X_CF) {
-            LSLw(s3, s1, c-1);
-            BFXILw(xFlags, s3, 7, 1);   // insert F_CF from s3[7:1]
+            // sub X, X will generate a carry flags
+            IFNATIVE(NF_CF) {GEN_INVERTED_CARRY();} else {BFCw(xFlags, F_CF, 1);}
         }
-        MOVw_REG(s1, xZR);
         IFX(X_OF) {
             BFCw(xFlags, F_OF, 1);
         }
         IFX(X_SF) {
-            BFCw(xFlags, F_SF, 1);
+            IFNATIVE(NF_SF) {} else BFCw(xFlags, F_SF, 1);
         }
         if(box64_dynarec_test) IFX(X_AF) {
             BFCw(xFlags, F_AF, 1);
         }
-        IFX(X_PF | X_ZF) {
-            MOV32w(s3, 1);
-            IFX(X_ZF) {
-                BFIw(xFlags, s3, F_ZF, 1);
-            }
-            IFX(X_PF) {
-                BFIw(xFlags, s3, F_PF, 1);
+        IFX(X_ZF) {
+            IFNATIVE(NF_EQ) {} else {
+                ORRw_mask(xFlags, xFlags, 26, 0);   //1<<F_ZF
             }
         }
+        IFX(X_PF) {
+            ORRw_mask(xFlags, xFlags, 30, 0);   //1<<F_PF
+        }
     }
 }
 
@@ -487,8 +561,8 @@ void emit_sar8(dynarec_arm_t* dyn, int ninst, int s1, int s2, int s3, int s4)
     }
     COMP_ZFSF(s1, 8)
     IFX(X_OF) {
-        CMPSw_U12(s2, 1);
-        Bcond(cNE, 4+4);
+        SUBw_U12(s4, s2, 1);
+        CBNZw(s4, 4+4);
             BFCw(xFlags, F_OF, 1);
     }
     if(box64_dynarec_test)
@@ -513,8 +587,7 @@ void emit_sar8c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int s
         SET_DFNONE(s4);
     }
     if(c<8) IFX(X_CF) {
-        ASRw(s3, s1, c-1);
-        BFIw(xFlags, s3, 0, 1);
+        BFXILw(xFlags, s1, c-1, 1);
     }
     ASRw(s1, s1, c);
     IFX(X_PEND) {
@@ -530,13 +603,18 @@ void emit_sar8c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int s
             emit_pf(dyn, ninst, s1, s4);
         }
     } else {
-        LSRw(s3, s1, 7);
-        BFIw(xFlags, s3, F_CF, 1);
-        BFIw(wFlags, s3, F_SF, 1);
-        EORw_mask(s3, s3, 0, 0);    //1
-        BFIw(xFlags, s3, F_ZF, 1);
-        MOV32w(s3, 1);
-        BFIw(xFlags, s3, F_PF, 1);
+        IFX(X_CF|X_ZF|X_SF|X_PF) {
+            LSRw(s3, s1, 7);
+            IFX(X_CF){ BFIw(xFlags, s3, F_CF, 1); }
+            IFX(X_SF){ BFIw(wFlags, s3, F_SF, 1); }
+            IFX(X_ZF) {
+                EORw_mask(s3, s3, 0, 0);    //1
+                BFIw(xFlags, s3, F_ZF, 1);
+            }
+            IFX(X_PF) {
+                ORRw_mask(xFlags, xFlags, 30, 0);   //1<<F_PF
+            }
+        }
     }
     if(box64_dynarec_test)
         IFX(X_AF) {
@@ -564,14 +642,14 @@ void emit_shl16(dynarec_arm_t* dyn, int ninst, int s1, int s2, int s3, int s4)
     IFX(X_PEND) {
         STRH_U12(s1, xEmu, offsetof(x64emu_t, res));
     }
-    COMP_ZFSF(s1, 16)
     IFX(X_OF) {
         CMPSw_U12(s2, 1);   // if s2==1
-            IFX2(X_SF, && !arm64_flagm) {} else {LSRw(s3, s1, 15);}
+            LSRw(s3, s1, 15);
             EORw_REG(s4, s3, xFlags);  // CF is set if OF is asked
             CSELw(s4, s4, wZR, cEQ);
             BFIw(xFlags, s4, F_OF, 1);
     }
+    COMP_ZFSF(s1, 16)
     if(box64_dynarec_test)
         IFX(X_AF) {
             BFCw(xFlags, F_AF, 1);
@@ -606,7 +684,7 @@ void emit_shl16c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int
         COMP_ZFSF(s1, 16)
         IFX(X_OF) {
             if(c==1) {
-                IFX2(X_SF, && !arm64_flagm) {} else {LSRw(s3, s1, 15);}
+                IFX2(X_SF, && !arm64_flagm) {} else {LSRw(s3, s1, 15);} // use COMP_ZFSF operation
                 EORw_REG(s4, s3, xFlags);  // CF is set if OF is asked
                 BFIw(xFlags, s4, F_OF, 1);
             } else {
@@ -619,32 +697,40 @@ void emit_shl16c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int
         IFX(X_PF) {
             if(c>7) {
                 // the 0xff area will be 0, so PF is known
-                MOV32w(s3, 1);
-                BFIw(xFlags, s3, F_PF, 1);
+                ORRw_mask(xFlags, xFlags, 30, 0);   //1<<F_PF
             } else
                 emit_pf(dyn, ninst, s1, s4);
         }
     } else {
-        IFX(X_CF) {
-            LSLw(s3, s1, c-1);
-            BFXILw(xFlags, s3, 15, 1);   // insert F_CF from s3[15:1]
+        IFXNATIVE(X_ZF|X_SF|X_CF, NF_EQ|NF_SF|NF_CF) {
+            SUBSw_REG(s1, s1, s1);
+        } else {
+            MOVw_REG(s1, xZR);
+        }
+        IFX(X_PEND) {
+            STRH_U12(s1, xEmu, offsetof(x64emu_t, res));
         }
-        MOVw_REG(s1, xZR);
         IFX(X_OF) {
             BFCw(xFlags, F_OF, 1);
         }
-        IFX(X_SF) {
-            BFCw(xFlags, F_SF, 1);
+        IFX(X_CF) {
+            // sub X, X will generate a carry flags
+            IFNATIVE(NF_CF) {GEN_INVERTED_CARRY();} else {BFCw(xFlags, F_CF, 1);}
         }
-        IFX(X_PF | X_ZF) {
-            MOV32w(s3, 1);
-            IFX(X_ZF) {
-                BFIw(xFlags, s3, F_ZF, 1);
-            }
-            IFX(X_PF) {
-                BFIw(xFlags, s3, F_PF, 1);
+        IFX(X_SF) {
+            IFNATIVE(NF_SF) {} else BFCw(xFlags, F_SF, 1);
+        }
+        if(box64_dynarec_test) IFX(X_AF) {
+            BFCw(xFlags, F_AF, 1);
+        }
+        IFX(X_ZF) {
+            IFNATIVE(NF_EQ) {} else {
+                ORRw_mask(xFlags, xFlags, 26, 0);   //1<<F_ZF
             }
         }
+        IFX(X_PF) {
+            ORRw_mask(xFlags, xFlags, 30, 0);   //1<<F_PF
+        }
     }
 }
 
@@ -664,8 +750,8 @@ void emit_shr16(dynarec_arm_t* dyn, int ninst, int s1, int s2, int s3, int s4)
         BFIw(xFlags, s4, 0, 1);
     }
     IFX(X_OF) {
-        CMPSw_U12(s2, 1);   // if s2==1
-            Bcond(cNE, 4+2*4);
+        SUBw_U12(s4, s2, 1);   // if s2==1
+        CBNZw(s4, 4+2*4);
             LSRw(s4, s1, 15);
             BFIw(xFlags, s4, F_OF, 1);
     }
@@ -743,12 +829,12 @@ void emit_sar16(dynarec_arm_t* dyn, int ninst, int s1, int s2, int s3, int s4)
     IFX(X_PEND) {
         STRH_U12(s1, xEmu, offsetof(x64emu_t, res));
     }
-    COMP_ZFSF(s1, 16)
     IFX(X_OF) {
-        CMPSw_U12(s2, 1);
-        Bcond(cNE, 4+4);
+        SUBw_U12(s4, s2, 1);
+        CBNZw(s4, 4+4);
             BFCw(xFlags, F_OF, 1);
     }
+    COMP_ZFSF(s1, 16)
     if(box64_dynarec_test)
         IFX(X_AF) {
             BFCw(xFlags, F_AF, 1);
@@ -798,29 +884,15 @@ void emit_rol32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, i
 
     if (!c) return;
 
-    IFX(X_PEND) {
-        MOV32w(s3, c);
-        STRxw_U12(s3, xEmu, offsetof(x64emu_t, op2));
-        SET_DF(s4, rex.w?d_rol64:d_rol32);
-    } else IFX(X_ALL) {
-        SET_DFNONE(s4);
-    }
-    if(!c) {
-        IFX(X_PEND) {
-            STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
-        }
-        return;
-    }
+    SET_DFNONE(s4);
+
     RORxw(s1, s1, (rex.w?64:32)-c);
-    IFX(X_PEND) {
-        STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
-    }
     IFX(X_CF) {
         BFIw(xFlags, s1, F_CF, 1);
     }
     IFX(X_OF) {
         if(c==1) {
-            ADDxw_REG_LSR(s3, s1, s1, rex.w?63:31);
+            EORxw_REG_LSR(s3, s1, s1, rex.w?63:31);
             BFIw(xFlags, s3, F_OF, 1);
         }
     }
@@ -833,23 +905,9 @@ void emit_ror32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, i
 
     if (!c) return;
 
-    IFX(X_PEND) {
-        MOV32w(s3, c);
-        STRxw_U12(s3, xEmu, offsetof(x64emu_t, op2));
-        SET_DF(s4, rex.w?d_ror64:d_ror32);
-    } else IFX(X_ALL) {
-        SET_DFNONE(s4);
-    }
-    if(!c) {
-        IFX(X_PEND) {
-            STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
-        }
-        return;
-    }
+    SET_DFNONE(s4);
+
     RORxw(s1, s1, c);
-    IFX(X_PEND) {
-        STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
-    }
     IFX(X_CF) {
         BFXILxw(xFlags, s1, rex.w?63:31, 1);
     }
@@ -869,21 +927,13 @@ void emit_rol8c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int s
 
     if (!c) return;
 
-    IFX(X_PEND) {
-        MOV32w(s3, c);
-        STRB_U12(s3, xEmu, offsetof(x64emu_t, op2));
-        SET_DF(s4, d_rol8);
-    } else IFX(X_ALL) {
-        SET_DFNONE(s4);
-    }
+    SET_DFNONE(s4);
+
     if(c&7) {
         int rc = 8-(c&7);
         ORRw_REG_LSL(s1, s1, s1, 8);
         LSRw(s1, s1, rc);
     }
-    IFX(X_PEND) {
-        STRB_U12(s1, xEmu, offsetof(x64emu_t, res));
-    }
     IFX(X_CF) {
         BFIw(xFlags, s1, F_CF, 1);
     }
@@ -902,20 +952,12 @@ void emit_ror8c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int s
 
     if (!c) return;
 
-    IFX(X_PEND) {
-        MOV32w(s3, c);
-        STRB_U12(s3, xEmu, offsetof(x64emu_t, op2));
-        SET_DF(s4, d_ror8);
-    } else IFX(X_ALL) {
-        SET_DFNONE(s4);
-    }
+    SET_DFNONE(s4);
+
     if(c&7) {
         ORRw_REG_LSL(s1, s1, s1, 8);
         LSRw(s1, s1, c&7);
     }
-    IFX(X_PEND) {
-        STRB_U12(s1, xEmu, offsetof(x64emu_t, res));
-    }
     IFX(X_CF) {
         BFXILw(xFlags, s1, 7, 1);
     }
@@ -935,21 +977,13 @@ void emit_rol16c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int
 
     if (!c) return;
 
-    IFX(X_PEND) {
-        MOV32w(s3, c);
-        STRH_U12(s3, xEmu, offsetof(x64emu_t, op2));
-        SET_DF(s4, d_rol16);
-    } else IFX(X_ALL) {
-        SET_DFNONE(s4);
-    }
+    SET_DFNONE(s4);
+
     if(c&15) {
         int rc = 16-(c&15);
         ORRw_REG_LSL(s1, s1, s1, 16);
         LSRw(s1, s1, rc);
     }
-    IFX(X_PEND) {
-        STRH_U12(s1, xEmu, offsetof(x64emu_t, res));
-    }
     IFX(X_CF) {
         BFIw(xFlags, s1, F_CF, 1);
     }
@@ -968,20 +1002,12 @@ void emit_ror16c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int
 
     if (!c) return;
 
-    IFX(X_PEND) {
-        MOV32w(s3, c);
-        STRH_U12(s3, xEmu, offsetof(x64emu_t, op2));
-        SET_DF(s4, d_ror16);
-    } else IFX(X_ALL) {
-        SET_DFNONE(s4);
-    }
+    SET_DFNONE(s4);
+
     if(c&15) {
         ORRw_REG_LSL(s1, s1, s1, 16);
         LSRw(s1, s1, c&15);
     }
-    IFX(X_PEND) {
-        STRH_U12(s1, xEmu, offsetof(x64emu_t, res));
-    }
     IFX(X_CF) {
         BFXILw(xFlags, s1, 15, 1);
     }
@@ -1014,9 +1040,6 @@ void emit_rcl8c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int s
     }
     ORRw_REG_LSL(s1, s1, s1, 9);    // insert s1 again
     LSRw_IMM(s1, s1, 9-c); // do the rcl
-    IFX(X_PEND) {
-        STRB_U12(s1, xEmu, offsetof(x64emu_t, res));
-    }
     IFX(X_OF|X_CF) {
         IFX(X_CF) {
             BFIw(xFlags, s3, F_CF, 1);
@@ -1057,9 +1080,6 @@ void emit_rcr8c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int s
             BFIw(xFlags, s3, F_OF, 1);
         }
     }
-    IFX(X_PEND) {
-        STRB_U12(s1, xEmu, offsetof(x64emu_t, res));
-    }
 }
 
 // emit RCL16 instruction, from s1 , constant c, store result in s1 using s3 and s4 as scratch
@@ -1082,9 +1102,6 @@ void emit_rcl16c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int
     }
     ORRx_REG_LSL(s1, s1, s1, 17);    // insert s1 again
     LSRx_IMM(s1, s1, 17-c); // do the rcl
-    IFX(X_PEND) {
-        STRH_U12(s1, xEmu, offsetof(x64emu_t, res));
-    }
     IFX(X_CF) {
         BFIw(xFlags, s3, F_CF, 1);
     }
@@ -1123,9 +1140,6 @@ void emit_rcr16c(dynarec_arm_t* dyn, int ninst, int s1, uint32_t c, int s3, int
             BFIw(xFlags, s3, F_OF, 1);
         }
     }
-    IFX(X_PEND) {
-        STRH_U12(s1, xEmu, offsetof(x64emu_t, res));
-    }
 }
 
 // emit RCL32/RCL64 instruction, from s1 , constant c, store result in s1 using s3 and s4 as scratch
@@ -1133,6 +1147,8 @@ void emit_rcl32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, i
 {
     MAYUSE(s1); MAYUSE(s3); MAYUSE(s4);
 
+    if(!c) return;
+
     SET_DFNONE(s4);
 
     IFX(X_OF|X_CF) {
@@ -1146,9 +1162,6 @@ void emit_rcl32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, i
         BFIxw(s4, xFlags, c-1, 1);
         ORRxw_REG_LSR(s1, s4, s1, (rex.w?65:33)-c);
     }
-    IFX(X_PEND) {
-        STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
-    }
     IFX(X_CF) {
         BFIw(xFlags, s3, F_CF, 1);
     }
@@ -1164,6 +1177,8 @@ void emit_rcr32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, i
 {
     MAYUSE(s1); MAYUSE(s3); MAYUSE(s4);
 
+    if(!c) return;
+
     SET_DFNONE(s4);
 
     IFX(X_OF) {
@@ -1186,9 +1201,6 @@ void emit_rcr32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, i
     IFX(X_CF) {
         BFIw(wFlags, s3, 0, 1);
     }
-    IFX(X_PEND) {
-        STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
-    }
 }
 
 // emit SHRD32 instruction, from s1, fill s2 , constant c, store result in s1 using s3 and s4 as scratch
@@ -1213,7 +1225,7 @@ void emit_shrd32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, int s2, uint
         BFXILxw(xFlags, s1, c-1, 1);    // set CF
     }
     IFX(X_OF) {
-        if(c==1) {
+        if((c==1) || box64_dynarec_test) {
             LSRxw(s4, s1, rex.w?63:31);
             BFIw(xFlags, s4, F_OF, 1);  // store sign for later use
         }
@@ -1222,14 +1234,21 @@ void emit_shrd32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, int s2, uint
     IFX(X_PEND) {
         STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
     }
+    int need_tst = 0;
+    IFX(X_ZF) need_tst = 1;
+    IFXNATIVE(X_SF, NF_SF) need_tst = 1;
+    if(need_tst) TSTxw_REG(s1, s1);
     IFX(X_ZF) {
-        TSTxw_REG(s1, s1);
-        CSETw(s4, cEQ);
-        BFIw(xFlags, s4, F_ZF, 1);
+        IFNATIVE(NF_EQ) {} else {
+            CSETw(s4, cEQ);
+            BFIw(xFlags, s4, F_ZF, 1);
+        }
     }
     IFX(X_SF) {
-        LSRxw(s4, s1, (rex.w)?63:31);
-        BFIx(xFlags, s4, F_SF, 1);
+        IFNATIVE(NF_SF) {} else {
+            LSRxw(s4, s1, (rex.w)?63:31);
+            BFIx(xFlags, s4, F_SF, 1);
+        }
     }
     IFX(X_OF) {
         if(c==1) {
@@ -1263,8 +1282,7 @@ void emit_shld32c(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, int s2, uint
         return;
     }
     IFX(X_CF) {
-        LSRxw(s3, s1, (rex.w?64:32)-c);
-        BFIxw(xFlags, s3, F_CF, 1);
+        BFXILx(xFlags, s1, (rex.w?64:32)-c, 1);
     }
     IFX(X_OF) {
         LSRxw(s3, s1, rex.w?63:31);
@@ -1325,18 +1343,25 @@ void emit_shrd32(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
     IFX(X_PEND) {
         STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
     }
+    int need_tst = 0;
+    IFX(X_ZF) need_tst = 1;
+    IFXNATIVE(X_SF, NF_SF) need_tst = 1;
+    if(need_tst) TSTxw_REG(s1, s1);
     IFX(X_ZF) {
-        TSTxw_REG(s1, s1);
-        CSETw(s4, cEQ);
-        BFIw(xFlags, s4, F_ZF, 1);
+        IFNATIVE(NF_EQ) {} else {
+            CSETw(s4, cEQ);
+            BFIw(xFlags, s4, F_ZF, 1);
+        }
     }
     IFX(X_SF) {
-        LSRxw(s4, s1, (rex.w)?63:31);
-        BFIw(xFlags, s4, F_SF, 1);
+        IFNATIVE(NF_SF) {} else {
+            LSRxw(s4, s1, (rex.w)?63:31);
+            BFIx(xFlags, s4, F_SF, 1);
+        }
     }
     IFX(X_OF) {
-        CMPSw_U12(s5, 1);
-        Bcond(cNE, 4+2*4);
+        SUBw_U12(s3, s5, 1);
+        CBNZw(s3, 4+2*4);    //flagless jump
             UBFXx(s3, s1, rex.w?63:31, 1);
             EORw_REG_LSL(xFlags, xFlags, s3, F_OF);  // OF is set if sign changed
     }
@@ -1371,18 +1396,25 @@ void emit_shld32(dynarec_arm_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
     IFX(X_PEND) {
         STRxw_U12(s1, xEmu, offsetof(x64emu_t, res));
     }
+    int need_tst = 0;
+    IFX(X_ZF) need_tst = 1;
+    IFXNATIVE(X_SF, NF_SF) need_tst = 1;
+    if(need_tst) TSTxw_REG(s1, s1);
     IFX(X_ZF) {
-        TSTxw_REG(s1, s1);
-        CSETw(s4, cEQ);
-        BFIw(xFlags, s4, F_ZF, 1);
+        IFNATIVE(NF_EQ) {} else {
+            CSETw(s4, cEQ);
+            BFIw(xFlags, s4, F_ZF, 1);
+        }
     }
     IFX(X_SF) {
-        LSRxw(s4, s1, (rex.w)?63:31);
-        BFIx(xFlags, s4, F_SF, 1);
+        IFNATIVE(NF_SF) {} else {
+            LSRxw(s4, s1, (rex.w)?63:31);
+            BFIx(xFlags, s4, F_SF, 1);
+        }
     }
     IFX(X_OF) {
-        CMPSw_U12(s5, 1);
-        Bcond(cNE, 4+2*4);
+        SUBw_U12(s3, s5, 1);
+        CBNZw(s3, 4+2*4);    //flagless jump
             UBFXx(s3, s1, rex.w?63:31, 1);
             EORw_REG_LSL(xFlags, xFlags, s3, F_OF);  // OF is set if sign changed
     }
@@ -1462,8 +1494,8 @@ void emit_shrd16(dynarec_arm_t* dyn, int ninst, int s1, int s2, int s5, int s3,
     }
     COMP_ZFSF(s1, 16)
     IFX(X_OF) {
-        CMPSw_U12(s5, 1);
-        Bcond(cNE, 4+2*4);
+        SUBw_U12(s3, s5, 1);
+        CBNZw(s3, 4+2*4);
             UBFXw(s3, s1, 15, 1);
             EORw_REG_LSL(xFlags, xFlags, s3, F_OF);  // OF is set if sign changed
     }
@@ -1553,8 +1585,8 @@ void emit_shld16(dynarec_arm_t* dyn, int ninst, int s1, int s2, int s5, int s3,
     }
     COMP_ZFSF(s1, 16)
     IFX(X_OF) {
-        CMPSw_U12(s5, 1);
-        Bcond(cNE, 4+2*4);
+        SUBw_U12(s3, s5, 1);
+        CBNZw(s3, 4+2*4);
             UBFXw(s3, s1, 15, 1);
             EORw_REG_LSL(xFlags, xFlags, s3, F_OF);  // OF is set if sign changed
     }
