@@ -9,6 +9,20 @@ typedef struct instsize_s instsize_t;
 
 #define BARRIER_MAYBE   8
 
+#define NF_EQ   (1<<0)
+#define NF_SF   (1<<1)
+#define NF_VF   (1<<2)
+#define NF_CF   (1<<3)
+
+// Nothing happens to the native flags
+#define NAT_FLAG_OP_NONE        0
+// Native flags are touched on this opcode
+#define NAT_FLAG_OP_TOUCH       1
+// Native flags are destroyed and unusable
+#define NAT_FLAG_OP_UNUSABLE    2
+// Native flags usaged are canceled here
+#define NAT_FLAG_OP_CANCELED    3
+
 #define NEON_CACHE_NONE     0
 #define NEON_CACHE_ST_D     1
 #define NEON_CACHE_ST_F     2
@@ -89,11 +103,25 @@ typedef struct instruction_arm64_s {
     uint16_t            ymm0_in;    // bitmap of ymm to zero at purge
     uint16_t            ymm0_add;   // the ymm0 added by the opcode
     uint16_t            ymm0_sub;   // the ymm0 removed by the opcode
-    uint16_t            ymm0_out;   // the ymmm0 at th end of the opcode
+    uint16_t            ymm0_out;   // the ymm0 at th end of the opcode
     uint16_t            ymm0_pass2, ymm0_pass3;
     uint8_t             barrier_maybe;
-    uint8_t             will_write;
-    uint8_t             last_write;
+    uint8_t             will_write:2; // [strongmem] will write to memory
+    uint8_t             last_write:1; // [strongmem] the last write in a SEQ
+    uint8_t             lock:1;       // [strongmem] lock semantic
+    uint8_t             wfe:1;        // opcode uses sevl + wfe
+    uint8_t             set_nat_flags;  // 0 or combinaison of native flags define
+    uint8_t             use_nat_flags;  // 0 or combinaison of native flags define
+    uint8_t             use_nat_flags_before;  // 0 or combinaison of native flags define
+    uint8_t             nat_flags_op:4;// what happens to native flags here
+    uint8_t             nat_flags_op_before:4;// what happens to native flags here
+    uint8_t             before_nat_flags;  // 0 or combinaison of native flags define
+    uint8_t             need_nat_flags;
+    unsigned            gen_inverted_carry:1;
+    unsigned            normal_carry:1;
+    unsigned            normal_carry_before:1;
+    unsigned            invert_carry:1; // this opcode force an inverted carry
+    unsigned            df_notneeded:1;
     flagcache_t         f_exit;     // flags status at end of instruction
     neoncache_t         n;          // neoncache at end of instruction (but before poping)
     flagcache_t         f_entry;    // flags status before the instruction begin
@@ -132,7 +160,6 @@ typedef struct dynarec_arm_s {
     int                 forward_ninst;  // ninst at the forward point
     uint16_t            ymm_zero;   // bitmap of ymm to zero at purge
     uint8_t             smwrite;    // for strongmem model emulation
-    uint8_t             smread;
     uint8_t             doublepush;
     uint8_t             doublepop;
     uint8_t             always_test;
@@ -154,9 +181,11 @@ void CreateJmpNext(void* addr, void* next);
 #define GO_TRACE(A, B, s0)  \
     GETIP(addr);            \
     MOVx_REG(x1, xRIP);     \
+    MRS_nzcv(s0);           \
     STORE_XEMU_CALL(xRIP);  \
     MOV32w(x2, B);          \
-    CALL(A, -1);            \
+    CALL_(A, -1, s0);       \
+    MSR_nzcv(s0);           \
     LOAD_XEMU_CALL(xRIP)
 
 #endif //__DYNAREC_ARM_PRIVATE_H_
