@@ -19,6 +19,7 @@
 #include "dynarec_private.h"
 #include "elfloader.h"
 #include "bridge.h"
+#include "signals.h"
 
 #include "dynarec_native.h"
 #include "native_lock.h"
@@ -198,9 +199,13 @@ static dynablock_t* internalDBGetBlock(x64emu_t* emu, uintptr_t addr, uintptr_t
 {
     if(hasAlternate((void*)addr))
         return NULL;
+    const uint32_t req_prot = (box64_pagesize==4096)?(PROT_EXEC|PROT_READ):PROT_READ;
     dynablock_t* block = getDB(addr);
-    if(block || !create)
+    if(block || !create) {
+        if(block && getNeedTest(addr) && (getProtection(addr)&req_prot)!=req_prot)
+            block = NULL;
         return block;
+    }
 
     if(need_lock) {
         if(box64_dynarec_wait) {
@@ -211,11 +216,17 @@ static dynablock_t* internalDBGetBlock(x64emu_t* emu, uintptr_t addr, uintptr_t
         }
         block = getDB(addr);    // just in case
         if(block) {
+            if(block && getNeedTest(addr) && (getProtection_fast(addr)&req_prot)!=req_prot)
+                block = NULL;
             mutex_unlock(&my_context->mutex_dyndump);
             return block;
         }
     }
-    
+    if((getProtection_fast(addr)&req_prot)!=req_prot) {// cannot be run, get out of the Dynarec
+        if(need_lock)
+            mutex_unlock(&my_context->mutex_dyndump);
+        return NULL;
+    }
     block = AddNewDynablock(addr);
 
     // fill the block
