@@ -19,7 +19,7 @@
 
 #include "la64_printer.h"
 #include "dynarec_la64_private.h"
-#include "dynarec_la64_helper.h"
+#include "../dynarec_helper.h"
 #include "dynarec_la64_functions.h"
 
 
@@ -60,7 +60,7 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             GETGD;
             SMDMB();
             if ((nextop & 0xC0) == 0xC0) {
-                ed = TO_LA64((nextop & 7) + (rex.b << 3));
+                ed = TO_NAT((nextop & 7) + (rex.b << 3));
                 emit_add32(dyn, ninst, rex, ed, gd, x3, x4, x5);
             } else {
                 addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, 0);
@@ -75,9 +75,103 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             }
             SMDMB();
             break;
+        case 0x09:
+            INST_NAME("LOCK OR Ed, Gd");
+            SETFLAGS(X_ALL, SF_SET_PENDING);
+            nextop = F8;
+            GETGD;
+            SMDMB();
+            if (MODREG) {
+                ed = TO_NAT((nextop & 7) + (rex.b << 3));
+                emit_or32(dyn, ninst, rex, ed, gd, x3, x4);
+            } else {
+                addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, 0);
+                MARKLOCK;
+                LLxw(x1, wback, 0);
+                OR(x4, x1, gd);
+                SCxw(x4, wback, 0);
+                BEQZ_MARKLOCK(x4);
+                IFX (X_ALL | X_PEND)
+                    emit_or32(dyn, ninst, rex, x1, gd, x3, x4);
+            }
+            SMDMB();
+            break;
         case 0x0F:
             nextop = F8;
             switch (nextop) {
+                case 0xB0:
+                    switch (rep) {
+                        case 0:
+                            INST_NAME("LOCK CMPXCHG Eb, Gb");
+                            SETFLAGS(X_ALL, SF_SET_PENDING);
+                            nextop = F8;
+                            ANDI(x6, xRAX, 0xff); // AL
+                            SMDMB();
+                            if (MODREG) {
+                                if (rex.rex) {
+                                    wback = TO_NAT((nextop & 7) + (rex.b << 3));
+                                    wb2 = 0;
+                                } else {
+                                    wback = (nextop & 7);
+                                    wb2 = (wback >> 2) * 8;
+                                    wback = TO_NAT(wback & 3);
+                                }
+                                BSTRPICK_D(x2, wback, wb2 + 7, wb2);
+                                wb1 = 0;
+                                ed = x2;
+                                UFLAG_IF {
+                                    emit_cmp8(dyn, ninst, x6, ed, x3, x4, x5, x1);
+                                }
+                                BNE_MARK2(x6, x2);
+                                BSTRPICK_D(wback, x2, wb2 + 7, wb2);
+                                GETGB(x1);
+                                MV(ed, gd);
+                                MARK2;
+                                BSTRINS_D(xRAX, x2, 7, 0);
+                                B_NEXT_nocond;
+                            } else {
+                                if (rex.rex) {
+                                    gb1 = TO_NAT(((nextop & 0x38) >> 3) + (rex.r << 3));
+                                    gb2 = 0;
+                                } else {
+                                    gd = (nextop & 0x38) >> 3;
+                                    gb2 = ((gd & 4) >> 2) * 8;
+                                    gb1 = TO_NAT(gd & 3);
+                                }
+                                addr = geted(dyn, addr, ninst, nextop, &wback, x3, x2, &fixedaddress, rex, LOCK_LOCK, 0, 0);
+                                ANDI(x5, wback, 0b11);
+                                SLLI_D(x5, x5, 3); // shamt
+                                MARKLOCK;
+                                ADDI_D(x7, xZR, ~0b11);
+                                AND(x7, wback, x7); // align to 32bit
+                                LD_WU(x1, x7, 0);
+                                LL_W(x4, x7, 0);
+                                SRL_D(x4, x4, x5);
+                                ANDI(x4, x4, 0xff);
+                                BNE_MARK(x6, x4); // compare AL with m8
+                                // AL == m8, r8 is loaded into m8
+                                ADDI_D(x2, xZR, 0xff);
+                                SLL_D(x2, x2, x5);
+                                NOR(x2, x2, xZR);
+                                AND(x2, x1, x2);
+                                BSTRPICK_D(x1, gb1, gb2 + 7, gb2);
+                                SLL_D(x1, x1, x5);
+                                OR(x1, x1, x2);
+                                SC_W(x1, x7, 0);
+                                BEQZ_MARKLOCK(x1);
+                                // done
+                                MARK;
+                                UFLAG_IF { emit_cmp8(dyn, ninst, x6, x4, x1, x2, x3, x5); }
+                                // load m8 into AL
+                                ANDI(xRAX, xRAX, ~0xff);
+                                OR(xRAX, xRAX, x4);
+                            }
+                            SMDMB();
+                            break;
+                        default:
+                            DEFAULT;
+                    }
+                    break;
                 case 0xB1:
                     switch (rep) {
                         case 0:
@@ -86,7 +180,7 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                             nextop = F8;
                             GETGD;
                             if (MODREG) {
-                                ed = TO_LA64((nextop & 7) + (rex.b << 3));
+                                ed = TO_NAT((nextop & 7) + (rex.b << 3));
                                 wback = 0;
                                 UFLAG_IF { emit_cmp32(dyn, ninst, rex, xRAX, ed, x3, x4, x5, x6); }
                                 MV(x1, ed); // save value
@@ -142,7 +236,7 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                             GETGD;
                             SMDMB();
                             if (MODREG) {
-                                ed = TO_LA64((nextop & 7) + (rex.b << 3));
+                                ed = TO_NAT((nextop & 7) + (rex.b << 3));
                                 MVxw(x1, ed);
                                 MVxw(ed, gd);
                                 MVxw(gd, x1);
@@ -167,21 +261,29 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                     }
                     break;
                 case 0xC7:
-                    switch (rep) {
-                        case 0:
+                    // rep has no impact here
+                    nextop = F8;
+                    switch ((nextop >> 3) & 7) {
+                        case 1:
                             if (rex.w) {
                                 INST_NAME("LOCK CMPXCHG16B Gq, Eq");
+                                if (!la64_scq) {
+                                    static int warned = 0;
+                                    PASS3(if (!warned) dynarec_log(LOG_INFO, "Warning, LOCK CMPXCHG16B is not well supported on LoongArch without SCQ and issues are expected.\n"));
+                                    warned = 1;
+                                }
                             } else {
                                 INST_NAME("LOCK CMPXCHG8B Gq, Eq");
                             }
                             SETFLAGS(X_ZF, SF_SUBSET);
-                            nextop = F8;
                             addr = geted(dyn, addr, ninst, nextop, &wback, x1, x2, &fixedaddress, rex, LOCK_LOCK, 0, 0);
-                            if (la64_lbt) {
-                                X64_SET_EFLAGS(xZR, X_ZF);
-                            } else {
-                                ADDI_D(x2, xZR, ~(1 << F_ZF));
-                                AND(xFlags, xFlags, x2);
+                            UFLAG_IF {
+                                if (la64_lbt) {
+                                    X64_SET_EFLAGS(xZR, X_ZF);
+                                } else {
+                                    ADDI_D(x2, xZR, ~(1 << F_ZF));
+                                    AND(xFlags, xFlags, x2);
+                                }
                             }
                             if (rex.w) {
                                 if (la64_scq) {
@@ -194,11 +296,13 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                                     MV(x5, xRBX);
                                     SC_Q(x5, xRCX, wback);
                                     BEQZ_MARKLOCK(x5);
-                                    if (la64_lbt) {
-                                        ADDI_D(x5, xZR, -1);
-                                        X64_SET_EFLAGS(x5, X_ZF);
-                                    } else {
-                                        ORI(xFlags, xFlags, 1 << F_ZF);
+                                    UFLAG_IF {
+                                        if (la64_lbt) {
+                                            ADDI_D(x5, xZR, -1);
+                                            X64_SET_EFLAGS(x5, X_ZF);
+                                        } else {
+                                            ORI(xFlags, xFlags, 1 << F_ZF);
+                                        }
                                     }
                                     B_MARK3_nocond;
                                     MARK;
@@ -222,11 +326,13 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                                     BNE_MARK(x3, xRDX);
                                     ST_D(xRBX, wback, 0);
                                     ST_D(xRCX, wback, 8);
-                                    if (la64_lbt) {
-                                        ADDI_D(x5, xZR, -1);
-                                        X64_SET_EFLAGS(x5, X_ZF);
-                                    } else {
-                                        ORI(xFlags, xFlags, 1 << F_ZF);
+                                    UFLAG_IF {
+                                        if (la64_lbt) {
+                                            ADDI_D(x5, xZR, -1);
+                                            X64_SET_EFLAGS(x5, X_ZF);
+                                        } else {
+                                            ORI(xFlags, xFlags, 1 << F_ZF);
+                                        }
                                     }
                                     B_MARK3_nocond;
                                     MARK;
@@ -239,28 +345,51 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                                 }
                             } else {
                                 SMDMB();
-                                AND(x3, xRAX, xMASK);
-                                SLLI_D(x2, xRDX, 32);
-                                OR(x3, x3, x2);
-                                AND(x4, xRBX, xMASK);
-                                SLLI_D(x2, xRCX, 32);
-                                OR(x4, x4, x2);
+                                BSTRINS_D(x3, xRAX, 31, 0);
+                                BSTRINS_D(x3, xRDX, 63, 32);
+                                BSTRINS_D(x4, xRBX, 31, 0);
+                                BSTRINS_D(x4, xRCX, 63, 32);
+                                ANDI(x2, wback, 7);
+                                BNEZ_MARK2(x2);
+                                // Aligned
                                 MARKLOCK;
                                 LL_D(x2, wback, 0);
                                 BNE_MARK(x2, x3); // EDX_EAX != Ed
                                 MV(x5, x4);
                                 SC_D(x5, wback, 0);
                                 BEQZ_MARKLOCK(x5);
-                                if (la64_lbt) {
-                                    ADDI_D(x5, xZR, -1);
-                                    X64_SET_EFLAGS(x5, X_ZF);
-                                } else {
-                                    ORI(xFlags, xFlags, 1 << F_ZF);
+                                UFLAG_IF {
+                                    if (la64_lbt) {
+                                        ADDI_D(x5, xZR, -1);
+                                        X64_SET_EFLAGS(x5, X_ZF);
+                                    } else {
+                                        ORI(xFlags, xFlags, 1 << F_ZF);
+                                    }
+                                }
+                                B_MARK3_nocond;
+                                MARK2;
+                                // Unaligned
+                                ADDI_W(x5, xZR, 0xFF8);
+                                AND(x5, wback, x5);
+                                MARKLOCK2;
+                                LD_D(x2, wback, 0);
+                                LL_D(x6, x5, 0);
+                                BNE_MARK(x2, x3); // EDX_EAX != Ed
+                                SC_D(x6, x5, 0);
+                                BEQZ_MARKLOCK2(x6);
+                                ST_D(x4, wback, 0);
+                                UFLAG_IF {
+                                    if (la64_lbt) {
+                                        ADDI_D(x5, xZR, -1);
+                                        X64_SET_EFLAGS(x5, X_ZF);
+                                    } else {
+                                        ORI(xFlags, xFlags, 1 << F_ZF);
+                                    }
                                 }
                                 B_MARK3_nocond;
                                 MARK;
                                 SLLI_D(xRDX, x2, 32);
-                                AND(xRAX, x2, xMASK);
+                                ZEROUP2(xRAX, x2);
                                 MARK3;
                                 SMDMB();
                             }
@@ -273,6 +402,31 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                     DEFAULT;
             }
             break;
+        case 0x11:
+            INST_NAME("LOCK ADC Ed, Gd");
+            READFLAGS(X_CF);
+            SETFLAGS(X_ALL, SF_SET_PENDING);
+            nextop = F8;
+            GETGD;
+            SMDMB();
+            if (MODREG) {
+                ed = TO_NAT((nextop & 7) + (rex.b << 3));
+                emit_adc32(dyn, ninst, rex, ed, gd, x3, x4, x5, x6);
+            } else {
+                addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, 0);
+                MARKLOCK;
+                LLxw(x1, wback, 0);
+                ADDxw(x3, x1, gd);
+                ANDI(x4, xFlags, 1 << F_CF);
+                ADDxw(x3, x3, x4);
+                SCxw(x3, wback, 0);
+                BEQZ_MARKLOCK(x3);
+                IFX (X_ALL | X_PEND) {
+                    emit_adc32(dyn, ninst, rex, x1, gd, x3, x4, x5, x6);
+                }
+            }
+            SMDMB();
+            break;
         case 0x29:
             INST_NAME("LOCK SUB Ed, Gd");
             SETFLAGS(X_ALL, SF_SET_PENDING);
@@ -280,7 +434,7 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             GETGD;
             SMDMB();
             if (MODREG) {
-                ed = TO_LA64((nextop & 7) + (rex.b << 3));
+                ed = TO_NAT((nextop & 7) + (rex.b << 3));
                 emit_sub32(dyn, ninst, rex, ed, gd, x3, x4, x5);
             } else {
                 addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, 0);
@@ -311,7 +465,7 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                             i64 = F32S;
                         else
                             i64 = F8S;
-                        ed = TO_LA64((nextop & 7) + (rex.b << 3));
+                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
                         emit_add32c(dyn, ninst, rex, ed, i64, x3, x4, x5, x6);
                     } else {
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, (opcode == 0x81) ? 4 : 1);
@@ -349,7 +503,7 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                             i64 = F32S;
                         else
                             i64 = F8S;
-                        ed = TO_LA64((nextop & 7) + (rex.b << 3));
+                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
                         emit_or32c(dyn, ninst, rex, ed, i64, x3, x4);
                     } else {
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, (opcode == 0x81) ? 4 : 1);
@@ -387,7 +541,7 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                             i64 = F32S;
                         else
                             i64 = F8S;
-                        ed = TO_LA64((nextop & 7) + (rex.b << 3));
+                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
                         emit_sub32c(dyn, ninst, rex, ed, i64, x3, x4, x5, x6);
                     } else {
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, (opcode == 0x81) ? 4 : 1);
@@ -446,7 +600,7 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                     SETFLAGS(X_ALL & ~X_CF, SF_SUBSET_PENDING);
                     SMDMB();
                     if (MODREG) {
-                        ed = TO_LA64((nextop & 7) + (rex.b << 3));
+                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
                         emit_inc32(dyn, ninst, rex, ed, x3, x4, x5, x6);
                     } else {
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, 0);
@@ -464,7 +618,7 @@ uintptr_t dynarec64_F0(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                     SETFLAGS(X_ALL & ~X_CF, SF_SUBSET_PENDING);
                     SMDMB();
                     if (MODREG) {
-                        ed = TO_LA64((nextop & 7) + (rex.b << 3));
+                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
                         emit_dec32(dyn, ninst, rex, ed, x3, x4, x5, x6);
                     } else {
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, LOCK_LOCK, 0, 0);
