@@ -24,7 +24,7 @@
 #include "la64_printer.h"
 #include "dynarec_la64_private.h"
 #include "dynarec_la64_functions.h"
-#include "dynarec_la64_helper.h"
+#include "../dynarec_helper.h"
 
 #define SCRATCH 31
 
@@ -60,12 +60,12 @@ uintptr_t geted(dynarec_la64_t* dyn, uintptr_t addr, int ninst, uint8_t nextop,
                 if (sib_reg != 4) {
                     if (tmp && ((tmp < -2048) || (tmp > maxval) || !i12)) {
                         MOV64x(scratch, tmp);
-                        ADDSL(ret, scratch, TO_LA64(sib_reg), sib >> 6, ret);
+                        ADDSL(ret, scratch, TO_NAT(sib_reg), sib >> 6, ret);
                     } else {
                         if (sib >> 6) {
-                            SLLI_D(ret, TO_LA64(sib_reg), (sib >> 6));
+                            SLLI_D(ret, TO_NAT(sib_reg), (sib >> 6));
                         } else {
-                            ret = TO_LA64(sib_reg);
+                            ret = TO_NAT(sib_reg);
                         }
                         *fixaddress = tmp;
                     }
@@ -80,9 +80,9 @@ uintptr_t geted(dynarec_la64_t* dyn, uintptr_t addr, int ninst, uint8_t nextop,
                 }
             } else {
                 if (sib_reg != 4) {
-                    ADDSL(ret, TO_LA64(sib_reg2), TO_LA64(sib_reg), sib >> 6, scratch);
+                    ADDSL(ret, TO_NAT(sib_reg2), TO_NAT(sib_reg), sib >> 6, scratch);
                 } else {
-                    ret = TO_LA64(sib_reg2);
+                    ret = TO_NAT(sib_reg2);
                 }
             }
         } else if ((nextop & 7) == 5) {
@@ -118,7 +118,7 @@ uintptr_t geted(dynarec_la64_t* dyn, uintptr_t addr, int ninst, uint8_t nextop,
                     break;
             }
         } else {
-            ret = TO_LA64((nextop & 7) + (rex.b << 3));
+            ret = TO_NAT((nextop & 7) + (rex.b << 3));
         }
     } else {
         int64_t i64;
@@ -133,41 +133,41 @@ uintptr_t geted(dynarec_la64_t* dyn, uintptr_t addr, int ninst, uint8_t nextop,
             i64 = F32S;
         else
             i64 = F8S;
-        if (i64 == 0 || ((i64 >= -2048) && (i64 <= 2047) && i12)) {
+        if (i64 == 0 || ((i64 >= -2048) && (i64 <= maxval) && i12)) {
             *fixaddress = i64;
             if ((nextop & 7) == 4) {
                 if (sib_reg != 4) {
-                    ADDSL(ret, TO_LA64(sib_reg2), TO_LA64(sib_reg), sib >> 6, scratch);
+                    ADDSL(ret, TO_NAT(sib_reg2), TO_NAT(sib_reg), sib >> 6, scratch);
                 } else {
-                    ret = TO_LA64(sib_reg2);
+                    ret = TO_NAT(sib_reg2);
                 }
             } else {
-                ret = TO_LA64((nextop & 0x07) + (rex.b << 3));
+                ret = TO_NAT((nextop & 0x07) + (rex.b << 3));
             }
         } else {
             if (i64 >= -2048 && i64 <= 2047) {
                 if ((nextop & 7) == 4) {
                     if (sib_reg != 4) {
-                        ADDSL(scratch, TO_LA64(sib_reg2), TO_LA64(sib_reg), sib >> 6, scratch);
+                        ADDSL(scratch, TO_NAT(sib_reg2), TO_NAT(sib_reg), sib >> 6, scratch);
                     } else {
-                        scratch = TO_LA64(sib_reg2);
+                        scratch = TO_NAT(sib_reg2);
                     }
                 } else {
-                    scratch = TO_LA64((nextop & 0x07) + (rex.b << 3));
+                    scratch = TO_NAT((nextop & 0x07) + (rex.b << 3));
                 }
                 ADDI_D(ret, scratch, i64);
             } else {
                 MOV64x(scratch, i64);
                 if ((nextop & 7) == 4) {
                     if (sib_reg != 4) {
-                        ADD_D(scratch, scratch, TO_LA64(sib_reg2));
-                        ADDSL(ret, scratch, TO_LA64(sib_reg), sib >> 6, ret);
+                        ADD_D(scratch, scratch, TO_NAT(sib_reg2));
+                        ADDSL(ret, scratch, TO_NAT(sib_reg), sib >> 6, ret);
                     } else {
-                        PASS3(int tmp = TO_LA64(sib_reg2));
+                        PASS3(int tmp = TO_NAT(sib_reg2));
                         ADD_D(ret, tmp, scratch);
                     }
                 } else {
-                    PASS3(int tmp = TO_LA64((nextop & 0x07) + (rex.b << 3)));
+                    PASS3(int tmp = TO_NAT((nextop & 0x07) + (rex.b << 3)));
                     ADD_D(ret, tmp, scratch);
                 }
             }
@@ -201,18 +201,20 @@ static uintptr_t geted_32(dynarec_la64_t* dyn, uintptr_t addr, int ninst, uint8_
                 int64_t tmp = F32S;
                 if (sib_reg != 4) {
                     if (tmp && ((tmp < -2048) || (tmp > maxval) || !i12)) {
-                        MOV32w(scratch, tmp);
+                        // no need to zero up, as we did it below
+                        la64_move32(dyn, ninst, scratch, tmp, 0);
                         if ((sib >> 6)) {
-                            SLLI_D(ret, TO_LA64(sib_reg), sib >> 6);
+                            SLLI_D(ret, TO_NAT(sib_reg), sib >> 6);
                             ADD_W(ret, ret, scratch);
                         } else {
-                            ADD_W(ret, TO_LA64(sib_reg), scratch);
+                            ADD_W(ret, TO_NAT(sib_reg), scratch);
                         }
+                        ZEROUP(ret);
                     } else {
                         if (sib >> 6) {
-                            SLLI_D(ret, TO_LA64(sib_reg), (sib >> 6));
+                            SLLI_D(ret, TO_NAT(sib_reg), (sib >> 6));
                         } else {
-                            ret = TO_LA64(sib_reg);
+                            ret = TO_NAT(sib_reg);
                         }
                         *fixaddress = tmp;
                     }
@@ -228,13 +230,14 @@ static uintptr_t geted_32(dynarec_la64_t* dyn, uintptr_t addr, int ninst, uint8_
             } else {
                 if (sib_reg != 4) {
                     if ((sib >> 6)) {
-                        SLLI_D(ret, TO_LA64(sib_reg), (sib >> 6));
-                        ADD_W(ret, ret, TO_LA64(sib_reg2));
+                        SLLI_D(ret, TO_NAT(sib_reg), (sib >> 6));
+                        ADD_W(ret, ret, TO_NAT(sib_reg2));
                     } else {
-                        ADD_W(ret, TO_LA64(sib_reg2), TO_LA64(sib_reg));
+                        ADD_W(ret, TO_NAT(sib_reg2), TO_NAT(sib_reg));
                     }
+                    ZEROUP(ret);
                 } else {
-                    ret = TO_LA64(sib_reg2);
+                    ret = TO_NAT(sib_reg2);
                 }
             }
         } else if ((nextop & 7) == 5) {
@@ -247,9 +250,9 @@ static uintptr_t geted_32(dynarec_la64_t* dyn, uintptr_t addr, int ninst, uint8_
                     break;
             }
         } else {
-            ret = TO_LA64((nextop & 7));
+            ret = TO_NAT((nextop & 7));
             if (ret == hint) {
-                AND(hint, ret, xMASK); // to clear upper part
+                ZEROUP2(hint, ret); // to clear upper part
             }
         }
     } else {
@@ -265,58 +268,62 @@ static uintptr_t geted_32(dynarec_la64_t* dyn, uintptr_t addr, int ninst, uint8_
             i32 = F32S;
         else
             i32 = F8S;
-        if (i32 == 0 || ((i32 >= -2048) && (i32 <= 2047) && i12)) {
+        if (i32 == 0 || ((i32 >= -2048) && (i32 <= maxval) && i12)) {
             *fixaddress = i32;
             if ((nextop & 7) == 4) {
                 if (sib_reg != 4) {
                     if (sib >> 6) {
-                        SLLI_D(ret, TO_LA64(sib_reg), (sib >> 6));
-                        ADD_W(ret, ret, TO_LA64(sib_reg2));
+                        SLLI_D(ret, TO_NAT(sib_reg), (sib >> 6));
+                        ADD_W(ret, ret, TO_NAT(sib_reg2));
                     } else {
-                        ADD_W(ret, TO_LA64(sib_reg2), TO_LA64(sib_reg));
+                        ADD_W(ret, TO_NAT(sib_reg2), TO_NAT(sib_reg));
                     }
+                    ZEROUP(ret);
                 } else {
-                    ret = TO_LA64(sib_reg2);
+                    ret = TO_NAT(sib_reg2);
                 }
             } else {
-                ret = TO_LA64((nextop & 0x07));
+                ret = TO_NAT((nextop & 0x07));
             }
         } else {
             if (i32 >= -2048 && i32 <= 2047) {
                 if ((nextop & 7) == 4) {
                     if (sib_reg != 4) {
                         if (sib >> 6) {
-                            SLLI_D(scratch, TO_LA64(sib_reg), sib >> 6);
-                            ADD_W(scratch, scratch, TO_LA64(sib_reg2));
+                            SLLI_D(scratch, TO_NAT(sib_reg), sib >> 6);
+                            ADD_W(scratch, scratch, TO_NAT(sib_reg2));
                         } else {
-                            ADD_W(scratch, TO_LA64(sib_reg2), TO_LA64(sib_reg));
+                            ADD_W(scratch, TO_NAT(sib_reg2), TO_NAT(sib_reg));
                         }
                     } else {
-                        scratch = TO_LA64(sib_reg2);
+                        scratch = TO_NAT(sib_reg2);
                     }
                 } else {
-                    scratch = TO_LA64((nextop & 0x07));
+                    scratch = TO_NAT((nextop & 0x07));
                 }
                 ADDI_W(ret, scratch, i32);
+                ZEROUP(ret);
             } else {
-                MOV32w(scratch, i32);
+                // no need to zero up, as we did it below
+                la64_move32(dyn, ninst, scratch, i32, 0);
                 if ((nextop & 7) == 4) {
                     if (sib_reg != 4) {
-                        ADD_W(scratch, scratch, TO_LA64(sib_reg2));
+                        ADD_W(scratch, scratch, TO_NAT(sib_reg2));
                         if (sib >> 6) {
-                            SLLI_D(ret, TO_LA64(sib_reg), (sib >> 6));
+                            SLLI_D(ret, TO_NAT(sib_reg), (sib >> 6));
                             ADD_W(ret, ret, scratch);
                         } else {
-                            ADD_W(ret, scratch, TO_LA64(sib_reg));
+                            ADD_W(ret, scratch, TO_NAT(sib_reg));
                         }
                     } else {
-                        PASS3(int tmp = TO_LA64(sib_reg2));
+                        PASS3(int tmp = TO_NAT(sib_reg2));
                         ADD_W(ret, tmp, scratch);
                     }
                 } else {
-                    PASS3(int tmp = TO_LA64((nextop & 0x07)));
+                    PASS3(int tmp = TO_NAT((nextop & 0x07)));
                     ADD_W(ret, tmp, scratch);
                 }
+                ZEROUP(ret);
             }
         }
     }
@@ -352,15 +359,17 @@ uintptr_t geted32(dynarec_la64_t* dyn, uintptr_t addr, int ninst, uint8_t nextop
                     if (tmp && ((tmp < -2048) || (tmp > maxval) || !i12)) {
                         MOV64x(scratch, tmp);
                         if ((sib >> 6)) {
-                            SLLI_D(ret, TO_LA64(sib_reg), sib >> 6);
+                            SLLI_D(ret, TO_NAT(sib_reg), sib >> 6);
                             ADD_W(ret, ret, scratch);
-                        } else
-                            ADD_W(ret, TO_LA64(sib_reg), scratch);
+                        } else {
+                            ADD_W(ret, TO_NAT(sib_reg), scratch);
+                        }
+                        ZEROUP(ret);
                     } else {
                         if (sib >> 6)
-                            SLLI_D(ret, TO_LA64(sib_reg), (sib >> 6));
+                            SLLI_D(ret, TO_NAT(sib_reg), (sib >> 6));
                         else
-                            ret = TO_LA64(sib_reg);
+                            ret = TO_NAT(sib_reg);
                         *fixaddress = tmp;
                     }
                 } else {
@@ -375,19 +384,23 @@ uintptr_t geted32(dynarec_la64_t* dyn, uintptr_t addr, int ninst, uint8_t nextop
             } else {
                 if (sib_reg != 4) {
                     if ((sib >> 6)) {
-                        SLLI_D(ret, TO_LA64(sib_reg), (sib >> 6));
-                        ADD_W(ret, ret, TO_LA64(sib_reg2));
-                    } else
-                        ADD_W(ret, TO_LA64(sib_reg2), TO_LA64(sib_reg));
+                        SLLI_D(ret, TO_NAT(sib_reg), (sib >> 6));
+                        ADD_W(ret, ret, TO_NAT(sib_reg2));
+                    } else {
+                        ADD_W(ret, TO_NAT(sib_reg2), TO_NAT(sib_reg));
+                    }
+                    ZEROUP(ret);
                 } else {
-                    ret = TO_LA64(sib_reg2);
+                    ret = TO_NAT(sib_reg2);
                 }
             }
         } else if ((nextop & 7) == 5) {
             uint32_t tmp = F32;
-            MOV32w(ret, tmp);
+            // no need to zero up, as we did it below
+            la64_move32(dyn, ninst, ret, tmp, 0);
             GETIP(addr + delta);
             ADD_W(ret, ret, xRIP);
+            ZEROUP(ret);
             switch (lock) {
                 case 1: addLockAddress(addr + delta + tmp); break;
                 case 2:
@@ -395,9 +408,9 @@ uintptr_t geted32(dynarec_la64_t* dyn, uintptr_t addr, int ninst, uint8_t nextop
                     break;
             }
         } else {
-            ret = TO_LA64((nextop & 7) + (rex.b << 3));
+            ret = TO_NAT((nextop & 7) + (rex.b << 3));
             if (ret == hint) {
-                AND(hint, ret, xMASK); // to clear upper part
+                ZEROUP2(hint, ret); // to clear upper part
             }
         }
     } else {
@@ -413,54 +426,59 @@ uintptr_t geted32(dynarec_la64_t* dyn, uintptr_t addr, int ninst, uint8_t nextop
             i64 = F32S;
         else
             i64 = F8S;
-        if (i64 == 0 || ((i64 >= -2048) && (i64 <= 2047) && i12)) {
+        if (i64 == 0 || ((i64 >= -2048) && (i64 <= maxval) && i12)) {
             *fixaddress = i64;
             if ((nextop & 7) == 4) {
                 if (sib_reg != 4) {
                     if (sib >> 6) {
-                        SLLI_D(ret, TO_LA64(sib_reg), (sib >> 6));
-                        ADD_W(ret, ret, TO_LA64(sib_reg2));
-                    } else
-                        ADD_W(ret, TO_LA64(sib_reg2), TO_LA64(sib_reg));
+                        SLLI_D(ret, TO_NAT(sib_reg), (sib >> 6));
+                        ADD_W(ret, ret, TO_NAT(sib_reg2));
+                    } else {
+                        ADD_W(ret, TO_NAT(sib_reg2), TO_NAT(sib_reg));
+                    }
+                    ZEROUP(ret);
                 } else {
-                    ret = TO_LA64(sib_reg2);
+                    ret = TO_NAT(sib_reg2);
                 }
             } else {
-                ret = TO_LA64((nextop & 0x07) + (rex.b << 3));
+                ret = TO_NAT((nextop & 0x07) + (rex.b << 3));
             }
         } else {
             if (i64 >= -2048 && i64 <= 2047) {
                 if ((nextop & 7) == 4) {
                     if (sib_reg != 4) {
                         if (sib >> 6) {
-                            SLLI_D(scratch, TO_LA64(sib_reg), sib >> 6);
-                            ADD_W(scratch, scratch, TO_LA64(sib_reg2));
+                            SLLI_D(scratch, TO_NAT(sib_reg), sib >> 6);
+                            ADD_W(scratch, scratch, TO_NAT(sib_reg2));
                         } else
-                            ADD_W(scratch, TO_LA64(sib_reg2), TO_LA64(sib_reg));
+                            ADD_W(scratch, TO_NAT(sib_reg2), TO_NAT(sib_reg));
                     } else {
-                        scratch = TO_LA64(sib_reg2);
+                        scratch = TO_NAT(sib_reg2);
                     }
                 } else
-                    scratch = TO_LA64((nextop & 0x07) + (rex.b << 3));
+                    scratch = TO_NAT((nextop & 0x07) + (rex.b << 3));
                 ADDI_W(ret, scratch, i64);
+                ZEROUP(ret);
             } else {
-                MOV32w(scratch, i64);
+                // no need to zero up, as we did it below
+                la64_move32(dyn, ninst, scratch, i64, 0);
                 if ((nextop & 7) == 4) {
                     if (sib_reg != 4) {
-                        ADD_W(scratch, scratch, TO_LA64(sib_reg2));
+                        ADD_W(scratch, scratch, TO_NAT(sib_reg2));
                         if (sib >> 6) {
-                            SLLI_D(ret, TO_LA64(sib_reg), (sib >> 6));
+                            SLLI_D(ret, TO_NAT(sib_reg), (sib >> 6));
                             ADD_W(ret, ret, scratch);
                         } else
-                            ADD_W(ret, scratch, TO_LA64(sib_reg));
+                            ADD_W(ret, scratch, TO_NAT(sib_reg));
                     } else {
-                        PASS3(int tmp = TO_LA64(sib_reg2));
+                        PASS3(int tmp = TO_NAT(sib_reg2));
                         ADD_W(ret, tmp, scratch);
                     }
                 } else {
-                    PASS3(int tmp = TO_LA64((nextop & 0x07) + (rex.b << 3)));
+                    PASS3(int tmp = TO_NAT((nextop & 0x07) + (rex.b << 3)));
                     ADD_W(ret, tmp, scratch);
                 }
+                ZEROUP(ret);
             }
         }
     }
@@ -512,6 +530,9 @@ void jump_to_next(dynarec_la64_t* dyn, uintptr_t ip, int reg, int ninst, int is3
     MAYUSE(ninst);
     MESSAGE(LOG_DUMP, "Jump to next\n");
 
+    if(is32bits)
+        ip &= 0xffffffffLL;
+
     if (reg) {
         if (reg != xRIP) {
             MV(xRIP, reg);
@@ -550,7 +571,7 @@ void jump_to_next(dynarec_la64_t* dyn, uintptr_t ip, int reg, int ninst, int is3
 // MOVx(x3, 15);    no access to PC reg
 #endif
     SMEND();
-    JIRL(xRA, x2, 0x0); // save LR...
+    JIRL((dyn->insts[ninst].x64.has_callret ? xRA : xZR), x2, 0x0); // save LR...
 }
 
 void ret_to_epilog(dynarec_la64_t* dyn, int ninst, rex_t rex)
@@ -563,11 +584,11 @@ void ret_to_epilog(dynarec_la64_t* dyn, int ninst, rex_t rex)
     SMEND();
     if (box64_dynarec_callret) {
         // pop the actual return address from RV64 stack
-        LD_D(x2, xSP, 0);     // native addr
+        LD_D(xRA, xSP, 0);     // native addr
         LD_D(x6, xSP, 8);     // x86 addr
         ADDI_D(xSP, xSP, 16); // pop
         BNE(x6, xRIP, 2 * 4); // is it the right address?
-        BR(x2);
+        BR(xRA);
         // not the correct return address, regular jump, but purge the stack first, it's unsync now...
         ADDI_D(xSP, xSavedSP, -16);
     }
@@ -588,7 +609,7 @@ void ret_to_epilog(dynarec_la64_t* dyn, int ninst, rex_t rex)
     BSTRPICK_D(x2, xRIP, JMPTABL_START0 + JMPTABL_SHIFT0 - 1, JMPTABL_START0);
     ALSL_D(x3, x2, x3, 3);
     LD_D(x2, x3, 0);
-    BR(x2); // save LR
+    BR(x2);
     CLEARIP();
 }
 
@@ -599,7 +620,7 @@ void retn_to_epilog(dynarec_la64_t* dyn, int ninst, rex_t rex, int n)
     MESSAGE(LOG_DUMP, "Retn to epilog\n");
     POP1z(xRIP);
     if (n > 0x7ff) {
-        MOV64x(w1, n);
+        MOV64x(x1, n);
         ADDz(xRSP, xRSP, x1);
     } else {
         ADDIz(xRSP, xRSP, n);
@@ -608,11 +629,11 @@ void retn_to_epilog(dynarec_la64_t* dyn, int ninst, rex_t rex, int n)
     SMEND();
     if (box64_dynarec_callret) {
         // pop the actual return address from RV64 stack
-        LD_D(x2, xSP, 0);     // native addr
+        LD_D(xRA, xSP, 0);     // native addr
         LD_D(x6, xSP, 8);     // x86 addr
         ADDI_D(xSP, xSP, 16); // pop
         BNE(x6, xRIP, 2 * 4); // is it the right address?
-        BR(x2);
+        BR(xRA);
         // not the correct return address, regular jump, but purge the stack first, it's unsync now...
         ADDI_D(xSP, xSavedSP, -16);
     }
@@ -633,7 +654,7 @@ void retn_to_epilog(dynarec_la64_t* dyn, int ninst, rex_t rex, int n)
     BSTRPICK_D(x2, xRIP, JMPTABL_START0 + JMPTABL_SHIFT0 - 1, JMPTABL_START0);
     ALSL_D(x3, x2, x3, 3);
     LD_D(x2, x3, 0);
-    BR(x2); // save LR
+    BR(x2);
     CLEARIP();
 }
 
@@ -685,7 +706,6 @@ void call_c(dynarec_la64_t* dyn, int ninst, void* fnc, int reg, int ret, int sav
             LD_D(xRIP, xEmu, offsetof(x64emu_t, ip));
 #undef GO
     }
-    REGENERATE_MASK();
 
     fpu_popcache(dyn, ninst, reg, 0);
     if (saveflags) {
@@ -740,18 +760,18 @@ int sse_setround(dynarec_la64_t* dyn, int ninst, int s1, int s2)
     // MMX/x87 Round mode: 0..3: Nearest, Down, Up, Chop
     // LA64: 0..3: Nearest, TowardZero, TowardsPositive, TowardsNegative
     // 0->0, 1->3, 2->2, 3->1
-    BEQ(s1, xZR, 32);
+    BEQ(s1, xZR, 4 + 4 * 8); // done + 4
     ADDI_D(s2, xZR, 2);
-    BEQ(s1, s2, 24);
+    BEQ(s1, s2, 4 + 4 * 5); // done
     ADDI_D(s2, xZR, 3);
-    BEQ(s1, s2, 12);
+    BEQ(s1, s2, 4 + 4 * 2);
     ADDI_D(s1, xZR, 3);
     B(8);
     ADDI_D(s1, xZR, 1);
     // done
     SLLI_D(s1, s1, 8);
     MOVFCSR2GR(s2, FCSR3);
-    MOVGR2FCSR(FCSR3, s1); // exange RM with current
+    MOVGR2FCSR(FCSR3, s1); // exchange RM with current
     return s2;
 }
 
@@ -1147,7 +1167,6 @@ static void unloadCache(dynarec_la64_t* dyn, int ninst, int stack_cnt, int s1, i
 
 static void fpuCacheTransform(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3)
 {
-#if STEP > 1
     int i2 = dyn->insts[ninst].x64.jmp_insts;
     if (i2 < 0)
         return;
@@ -1260,12 +1279,10 @@ static void fpuCacheTransform(dynarec_la64_t* dyn, int ninst, int s1, int s2, in
         }
     }
     MESSAGE(LOG_DUMP, "\t---- Cache Transform\n");
-#endif
 }
 
 static void flagsCacheTransform(dynarec_la64_t* dyn, int ninst, int s1)
 {
-#if STEP > 1
     int j64;
     int jmp = dyn->insts[ninst].x64.jmp_insts;
     if(jmp<0)
@@ -1304,7 +1321,6 @@ static void flagsCacheTransform(dynarec_la64_t* dyn, int ninst, int s1)
         CALL_(UpdateFlags, -1, 0);
         MARKF2;
     }
-#endif
 }
 
 void CacheTransform(dynarec_la64_t* dyn, int ninst, int cacheupd, int s1, int s2, int s3) {
@@ -1340,4 +1356,4 @@ void la64_move64(dynarec_la64_t* dyn, int ninst, int reg, int64_t val)
         return;
     }
     LU52I_D(reg, reg, (val >> 52) & 0xfff);
-}
\ No newline at end of file
+}
