@@ -32,67 +32,22 @@
 #define PK64(a) *(uint64_t*)(addr + a)
 #define PKip(a) *(uint8_t*)(ip + a)
 
-// Strong mem emulation helpers
-#define SMREAD_MIN  2
-#define SMWRITE_MIN 1
-// Sequence of Read will trigger a DMB on "first" read if strongmem is >= SMREAD_MIN
-// Sequence of Write will trigger a DMB on "last" write if strongmem is >= 1
-// All Write operation that might use a lock all have a memory barrier if strongmem is >= SMWRITE_MIN
-// Opcode will read
-#define SMREAD()                                                        \
-    if ((dyn->smread == 0) && (box64_dynarec_strongmem > SMREAD_MIN)) { \
-        SMDMB();                                                        \
-    } else                                                              \
-        dyn->smread = 1
-// Opcode will read with option forced lock
-#define SMREADLOCK(lock) \
-    if ((lock) || ((dyn->smread == 0) && (box64_dynarec_strongmem > SMREAD_MIN))) { SMDMB(); }
-// Opcode might read (depend on nextop)
-#define SMMIGHTREAD() \
-    if (!MODREG) { SMREAD(); }
-// Opcode has wrote
-#define SMWRITE() dyn->smwrite = 1
-// Opcode has wrote (strongmem>1 only)
-#define SMWRITE2() \
-    if (box64_dynarec_strongmem > SMREAD_MIN) dyn->smwrite = 1
-// Opcode has wrote with option forced lock
-#define SMWRITELOCK(lock)                                  \
-    if (lock || (box64_dynarec_strongmem > SMWRITE_MIN)) { \
-        SMDMB();                                           \
-    } else                                                 \
-        dyn->smwrite = 1
-// Opcode might have wrote (depend on nextop)
-#define SMMIGHTWRITE() \
-    if (!MODREG) { SMWRITE(); }
-// Start of sequence
-#define SMSTART() SMEND()
-// End of sequence
-#define SMEND()                                               \
-    if (dyn->smwrite && box64_dynarec_strongmem) { DBAR(0); } \
-    dyn->smwrite = 0;                                         \
-    dyn->smread = 0;
-// Force a Data memory barrier (for LOCK: prefix)
-#define SMDMB()       \
-    DBAR(0);          \
-    dyn->smwrite = 0; \
-    dyn->smread = 1
-
 // LOCK_* define
 #define LOCK_LOCK (int*)1
 
 // GETGD    get x64 register in gd
-#define GETGD gd = TO_LA64(((nextop & 0x38) >> 3) + (rex.r << 3));
+#define GETGD gd = TO_NAT(((nextop & 0x38) >> 3) + (rex.r << 3));
 
 // GETGW extract x64 register in gd, that is i
-#define GETGW(i)                                         \
-    gd = TO_LA64(((nextop & 0x38) >> 3) + (rex.r << 3)); \
-    BSTRPICK_D(i, gd, 15, 0);                            \
+#define GETGW(i)                                        \
+    gd = TO_NAT(((nextop & 0x38) >> 3) + (rex.r << 3)); \
+    BSTRPICK_D(i, gd, 15, 0);                           \
     gd = i;
 
 // GETED can use r1 for ed, and r2 for wback. wback is 0 if ed is xEAX..xEDI
 #define GETED(D)                                                                                \
     if (MODREG) {                                                                               \
-        ed = TO_LA64((nextop & 7) + (rex.b << 3));                                              \
+        ed = TO_NAT((nextop & 7) + (rex.b << 3));                                               \
         wback = 0;                                                                              \
     } else {                                                                                    \
         SMREAD();                                                                               \
@@ -103,7 +58,7 @@
 
 #define GETEDz(D)                                                                               \
     if (MODREG) {                                                                               \
-        ed = TO_LA64((nextop & 7) + (rex.b << 3));                                              \
+        ed = TO_NAT((nextop & 7) + (rex.b << 3));                                               \
         wback = 0;                                                                              \
     } else {                                                                                    \
         SMREAD();                                                                               \
@@ -114,7 +69,7 @@
 // GETED32 can use r1 for ed, and r2 for wback. wback is 0 if ed is xEAX..xEDI
 #define GETED32(D)                                                                                \
     if (MODREG) {                                                                                 \
-        ed = TO_LA64((nextop & 7) + (rex.b << 3));                                                \
+        ed = TO_NAT((nextop & 7) + (rex.b << 3));                                                 \
         wback = 0;                                                                                \
     } else {                                                                                      \
         SMREAD();                                                                                 \
@@ -125,7 +80,7 @@
 // GETEDH can use hint for ed, and x1 or x2 for wback (depending on hint), might also use x3. wback is 0 if ed is xEAX..xEDI
 #define GETEDH(hint, D)                                                                                                                 \
     if (MODREG) {                                                                                                                       \
-        ed = TO_LA64((nextop & 7) + (rex.b << 3));                                                                                      \
+        ed = TO_NAT((nextop & 7) + (rex.b << 3));                                                                                       \
         wback = 0;                                                                                                                      \
     } else {                                                                                                                            \
         SMREAD();                                                                                                                       \
@@ -136,7 +91,7 @@
 // GETEWW will use i for ed, and can use w for wback.
 #define GETEWW(w, i, D)                                                                       \
     if (MODREG) {                                                                             \
-        wback = TO_LA64((nextop & 7) + (rex.b << 3));                                         \
+        wback = TO_NAT((nextop & 7) + (rex.b << 3));                                          \
         BSTRPICK_D(i, wback, 15, 0);                                                          \
         ed = i;                                                                               \
         wb1 = 0;                                                                              \
@@ -151,21 +106,27 @@
 #define GETEW(i, D) GETEWW(x3, i, D)
 
 // GETEDO can use r1 for ed, and r2 for wback. wback is 0 if ed is xEAX..xEDI
-#define GETEDO(O, D)                                                                            \
-    if (MODREG) {                                                                               \
-        ed = TO_LA64((nextop & 7) + (rex.b << 3));                                              \
-        wback = 0;                                                                              \
-    } else {                                                                                    \
-        SMREAD();                                                                               \
-        addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, NULL, 0, D); \
-        LDXxw(x1, wback, O);                                                                    \
-        ed = x1;                                                                                \
+#define GETEDO(O, D)                                                                                \
+    if (MODREG) {                                                                                   \
+        ed = TO_NAT((nextop & 7) + (rex.b << 3));                                                   \
+        wback = 0;                                                                                  \
+    } else {                                                                                        \
+        SMREAD();                                                                                   \
+        if (rex.is32bits) {                                                                         \
+            addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, NULL, 1, D); \
+            ADDz(O, wback, O);                                                                      \
+            LD_WU(x1, O, fixedaddress);                                                             \
+        } else {                                                                                    \
+            addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, NULL, 0, D); \
+            LDXxw(x1, wback, O);                                                                    \
+        }                                                                                           \
+        ed = x1;                                                                                    \
     }
 
 // GETSED can use r1 for ed, and r2 for wback. ed will be sign extended!
 #define GETSED(D)                                                                               \
     if (MODREG) {                                                                               \
-        ed = TO_LA64((nextop & 7) + (rex.b << 3));                                              \
+        ed = TO_NAT((nextop & 7) + (rex.b << 3));                                               \
         wback = 0;                                                                              \
         if (!rex.w) {                                                                           \
             ADD_W(x1, ed, xZR);                                                                 \
@@ -188,9 +149,9 @@
     }
 
 // GETGW extract x64 register in gd, that is i, Signed extented
-#define GETSGW(i)                                        \
-    gd = TO_LA64(((nextop & 0x38) >> 3) + (rex.r << 3)); \
-    EXT_W_H(i, gd);                                      \
+#define GETSGW(i)                                       \
+    gd = TO_NAT(((nextop & 0x38) >> 3) + (rex.r << 3)); \
+    EXT_W_H(i, gd);                                     \
     gd = i;
 
 // Write back ed in wback (if wback not 0)
@@ -203,16 +164,21 @@
         SMWRITE();                         \
     }
 
-#define WBACKO(O)            \
-    if (wback) {             \
-        SDXxw(ed, wback, O); \
-        SMWRITE2();          \
+#define WBACKO(O)                \
+    if (wback) {                 \
+        if (rex.is32bits) {      \
+            ADDz(O, wback, O);   \
+            ST_W(ed, O, 0);      \
+        } else {                 \
+            SDXxw(ed, wback, O); \
+        }                        \
+        SMWRITE2();              \
     }
 
 // GETSEW will use i for ed, and can use r3 for wback. This is the Signed version
 #define GETSEW(i, D)                                                                           \
     if (MODREG) {                                                                              \
-        wback = TO_LA64((nextop & 7) + (rex.b << 3));                                          \
+        wback = TO_NAT((nextop & 7) + (rex.b << 3));                                           \
         EXT_W_H(i, wback);                                                                     \
         ed = i;                                                                                \
         wb1 = 0;                                                                               \
@@ -235,18 +201,18 @@
 #define EWBACK EWBACKW(ed)
 
 // Write back gd in correct register
-#define GWBACK BSTRINS_D((TO_LA64(((nextop & 0x38) >> 3) + (rex.r << 3))), gd, 15, 0);
+#define GWBACK BSTRINS_D((TO_NAT(((nextop & 0x38) >> 3) + (rex.r << 3))), gd, 15, 0);
 
 // GETEB will use i for ed, and can use r3 for wback.
 #define GETEB(i, D)                                                                             \
     if (MODREG) {                                                                               \
         if (rex.rex) {                                                                          \
-            wback = TO_LA64((nextop & 7) + (rex.b << 3));                                       \
+            wback = TO_NAT((nextop & 7) + (rex.b << 3));                                        \
             wb2 = 0;                                                                            \
         } else {                                                                                \
             wback = (nextop & 7);                                                               \
             wb2 = (wback >> 2) * 8;                                                             \
-            wback = TO_LA64((wback & 3));                                                       \
+            wback = TO_NAT((wback & 3));                                                        \
         }                                                                                       \
         BSTRPICK_D(i, wback, wb2 + 7, wb2);                                                     \
         wb1 = 0;                                                                                \
@@ -262,12 +228,12 @@
 #define GETSEB(i, D)                                                                            \
     if (MODREG) {                                                                               \
         if (rex.rex) {                                                                          \
-            wback = TO_LA64((nextop & 7) + (rex.b << 3));                                       \
+            wback = TO_NAT((nextop & 7) + (rex.b << 3));                                        \
             wb2 = 0;                                                                            \
         } else {                                                                                \
             wback = (nextop & 7);                                                               \
             wb2 = (wback >> 2) * 8;                                                             \
-            wback = TO_LA64(wback & 3);                                                         \
+            wback = TO_NAT(wback & 3);                                                          \
         }                                                                                       \
         if (wb2) {                                                                              \
             SRLI_D(i, wback, wb2);                                                              \
@@ -279,46 +245,73 @@
         ed = i;                                                                                 \
     } else {                                                                                    \
         SMREAD();                                                                               \
-        addr = geted(dyn, addr, ninst, nextop, &wback, x2, x3, &fixedaddress, rex, NULL, 1, D); \
+        addr = geted(dyn, addr, ninst, nextop, &wback, x3, x2, &fixedaddress, rex, NULL, 1, D); \
         LD_B(i, wback, fixedaddress);                                                           \
         wb1 = 1;                                                                                \
         ed = i;                                                                                 \
     }
+// GETEB32 will use i for ed, and can use r3 for wback.
+#define GETEB32(i, D)                                                                             \
+    if (MODREG) {                                                                                 \
+        if (rex.rex) {                                                                            \
+            wback = TO_NAT((nextop & 7) + (rex.b << 3));                                          \
+            wb2 = 0;                                                                              \
+        } else {                                                                                  \
+            wback = (nextop & 7);                                                                 \
+            wb2 = (wback >> 2) * 8;                                                               \
+            wback = TO_NAT(wback & 3);                                                            \
+        }                                                                                         \
+        BSTRPICK_D(i, wback, wb2 + 7, wb2);                                                       \
+        wb1 = 0;                                                                                  \
+        ed = i;                                                                                   \
+    } else {                                                                                      \
+        SMREAD();                                                                                 \
+        addr = geted32(dyn, addr, ninst, nextop, &wback, x3, x2, &fixedaddress, rex, NULL, 1, D); \
+        LD_BU(i, wback, fixedaddress);                                                            \
+        wb1 = 1;                                                                                  \
+        ed = i;                                                                                   \
+    }
+
 // GETGB will use i for gd
-#define GETGB(i)                                              \
-    if (rex.rex) {                                            \
-        gb1 = TO_LA64(((nextop & 0x38) >> 3) + (rex.r << 3)); \
-        gb2 = 0;                                              \
-    } else {                                                  \
-        gd = (nextop & 0x38) >> 3;                            \
-        gb2 = ((gd & 4) >> 2);                                \
-        gb1 = TO_LA64((gd & 3));                              \
-    }                                                         \
-    gd = i;                                                   \
+#define GETGB(i)                                             \
+    if (rex.rex) {                                           \
+        gb1 = TO_NAT(((nextop & 0x38) >> 3) + (rex.r << 3)); \
+        gb2 = 0;                                             \
+    } else {                                                 \
+        gd = (nextop & 0x38) >> 3;                           \
+        gb2 = ((gd & 4) << 1);                               \
+        gb1 = TO_NAT((gd & 3));                              \
+    }                                                        \
+    gd = i;                                                  \
     BSTRPICK_D(gd, gb1, gb2 + 7, gb2);
 
 // GETEBO will use i for ed, i is also Offset, and can use r3 for wback.
-#define GETEBO(i, D)                                                                            \
-    if (MODREG) {                                                                               \
-        if (rex.rex) {                                                                          \
-            wback = TO_LA64((nextop & 7) + (rex.b << 3));                                       \
-            wb2 = 0;                                                                            \
-        } else {                                                                                \
-            wback = (nextop & 7);                                                               \
-            wb2 = (wback >> 2) * 8;                                                             \
-            wback = TO_LA64(wback & 3);                                                         \
-        }                                                                                       \
-        BSTRPICK_D(i, wback, wb2 + 7, wb2);                                                     \
-        wb1 = 0;                                                                                \
-        ed = i;                                                                                 \
-    } else {                                                                                    \
-        SMREAD();                                                                               \
-        addr = geted(dyn, addr, ninst, nextop, &wback, x3, x2, &fixedaddress, rex, NULL, 1, D); \
-        ADD_D(x3, wback, i);                                                                    \
-        if (wback != x3) wback = x3;                                                            \
-        LD_B(i, wback, fixedaddress);                                                           \
-        wb1 = 1;                                                                                \
-        ed = i;                                                                                 \
+#define GETEBO(i, D)                                                                                \
+    if (MODREG) {                                                                                   \
+        if (rex.rex) {                                                                              \
+            wback = TO_NAT((nextop & 7) + (rex.b << 3));                                            \
+            wb2 = 0;                                                                                \
+        } else {                                                                                    \
+            wback = (nextop & 7);                                                                   \
+            wb2 = (wback >> 2) * 8;                                                                 \
+            wback = TO_NAT(wback & 3);                                                              \
+        }                                                                                           \
+        BSTRPICK_D(i, wback, wb2 + 7, wb2);                                                         \
+        wb1 = 0;                                                                                    \
+        ed = i;                                                                                     \
+    } else {                                                                                        \
+        SMREAD();                                                                                   \
+        if (rex.is32bits) {                                                                         \
+            addr = geted(dyn, addr, ninst, nextop, &wback, x3, x2, &fixedaddress, rex, NULL, 1, D); \
+            ADDz(x3, wback, i);                                                                     \
+            if (wback != x3) wback = x3;                                                            \
+            LD_BU(i, wback, fixedaddress);                                                          \
+        } else {                                                                                    \
+            addr = geted(dyn, addr, ninst, nextop, &wback, x3, x2, &fixedaddress, rex, NULL, 0, D); \
+            LDX_BU(i, wback, i);                                                                    \
+        }                                                                                           \
+        wb1 = 1;                                                                                    \
+        ed = i;                                                                                     \
     }
 
 // Get GX as a quad (might use x1)
@@ -425,6 +418,8 @@
 #define GETMARKSEG  dyn->insts[ninst].markseg
 #define MARKLOCK    dyn->insts[ninst].marklock = dyn->native_size
 #define GETMARKLOCK dyn->insts[ninst].marklock
+#define MARKLOCK2    dyn->insts[ninst].marklock2 = dyn->native_size
+#define GETMARKLOCK2 dyn->insts[ninst].marklock2
 
 #define Bxx_gen(OP, M, reg1, reg2)   \
     j64 = GET##M - dyn->native_size; \
@@ -446,6 +441,8 @@
 #define BNE_MARK3(reg1, reg2) Bxx_gen(NE, MARK3, reg1, reg2)
 // Branch to MARKLOCK if reg1!=reg2 (use j64)
 #define BNE_MARKLOCK(reg1, reg2) Bxx_gen(NE, MARKLOCK, reg1, reg2)
+// Branch to MARKLOCK2 if reg1!=reg2 (use j64)
+#define BNE_MARKLOCK2(reg1, reg2) Bxx_gen(NE, MARKLOCK2, reg1, reg2)
 
 // Branch to MARK if reg1==reg2 (use j64)
 #define BEQ_MARK(reg1, reg2) Bxx_gen(EQ, MARK, reg1, reg2)
@@ -455,12 +452,16 @@
 #define BEQ_MARK3(reg1, reg2) Bxx_gen(EQ, MARK3, reg1, reg2)
 // Branch to MARKLOCK if reg1==reg2 (use j64)
 #define BEQ_MARKLOCK(reg1, reg2) Bxx_gen(EQ, MARKLOCK, reg1, reg2)
+// Branch to MARKLOCK2 if reg1==reg2 (use j64)
+#define BEQ_MARKLOCK2(reg1, reg2) Bxx_gen(EQ, MARKLOCK2, reg1, reg2)
 // Branch to MARK if reg1==0 (use j64)
 #define BEQZ_MARK(reg) BxxZ_gen(EQ, MARK, reg)
 // Branch to MARK2 if reg1==0 (use j64)
 #define BEQZ_MARK2(reg) BxxZ_gen(EQ, MARK2, reg)
 // Branch to MARKLOCK if reg1==0 (use j64)
 #define BEQZ_MARKLOCK(reg) BxxZ_gen(EQ, MARKLOCK, reg)
+// Branch to MARKLOCK2 if reg1==0 (use j64)
+#define BEQZ_MARKLOCK2(reg) BxxZ_gen(EQ, MARKLOCK2, reg)
 
 // Branch to MARK if reg1!=0 (use j64)
 #define BNEZ_MARK(reg) BxxZ_gen(NE, MARK, reg)
@@ -470,6 +471,8 @@
 #define BNEZ_MARK3(reg) BxxZ_gen(NE, MARK3, reg)
 // Branch to MARKLOCK if reg1!=0 (use j64)
 #define BNEZ_MARKLOCK(reg) BxxZ_gen(NE, MARKLOCK, reg)
+// Branch to MARKLOCK2 if reg1!=0 (use j64)
+#define BNEZ_MARKLOCK2(reg) BxxZ_gen(NE, MARKLOCK2, reg)
 
 // Branch to MARK if fcc!=0 (use j64)
 #define BCNEZ_MARK(fcc) BCxxZ_gen(NE, MARK, fcc)
@@ -479,6 +482,8 @@
 #define BCNEZ_MARK3(fcc) BCxxZ_gen(NE, MARK3, fcc)
 // Branch to MARKLOCK if fcc!=0 (use j64)
 #define BCNEZ_MARKLOCK(fcc) BxxZ_gen(NE, MARKLOCK, fcc)
+// Branch to MARKLOCK2 if fcc!=0 (use j64)
+#define BCNEZ_MARKLOCK2(fcc) BxxZ_gen(NE, MARKLOCK2, fcc)
 
 // Branch to MARK if fcc==0 (use j64)
 #define BCEQZ_MARK(fcc) BCxxZ_gen(EQ, MARK, fcc)
@@ -488,6 +493,8 @@
 #define BCEQZ_MARK3(fcc) BCxxZ_gen(EQ, MARK3, fcc)
 // Branch to MARKLOCK if fcc==0 (use j64)
 #define BCEQZ_MARKLOCK(fcc) BxxZ_gen(EQ, MARKLOCK, fcc)
+// Branch to MARKLOCK2 if fcc==0 (use j64)
+#define BCEQZ_MARKLOCK2(fcc) BxxZ_gen(EQ, MARKLOCK2, fcc)
 
 // Branch to MARK if reg1<reg2 (use j64)
 #define BLT_MARK(reg1, reg2) Bxx_gen(LT, MARK, reg1, reg2)
@@ -575,47 +582,57 @@
     LOAD_REG(R14);      \
     LOAD_REG(R15);
 
-#define SET_DFNONE()                             \
-    do {                                         \
-    dyn->f.dfnone_here=1;                        \
-    if (!dyn->f.dfnone) {                        \
-        ST_W(xZR, xEmu, offsetof(x64emu_t, df)); \
-        dyn->f.dfnone = 1;                       \
-    } } while(0);
-#define SET_DF(S, N)                           \
-    if ((N) != d_none) {                       \
-        MOV32w(S, (N));                        \
-        ST_W(S, xEmu, offsetof(x64emu_t, df)); \
-        dyn->f.dfnone = 0;                     \
-    } else                                     \
-        SET_DFNONE()
-#define SET_NODF() dyn->f.dfnone = 0
-#define SET_DFOK() dyn->f.dfnone = 1; dyn->f.dfnone_here=1
+#define SET_DFNONE()                                 \
+    do {                                             \
+        dyn->f.dfnone_here = 1;                      \
+        if (!dyn->f.dfnone) {                        \
+            ST_W(xZR, xEmu, offsetof(x64emu_t, df)); \
+            dyn->f.dfnone = 1;                       \
+        }                                            \
+    } while (0);
 
-#define CLEAR_FLAGS_(s) \
-    MOV64x(s, (1UL << F_AF) | (1UL << F_CF) | (1UL << F_OF) | (1UL << F_ZF) | (1UL << F_SF) | (1UL << F_PF)); ANDN(xFlags, xFlags, s);
+#define SET_DF(S, N)                                           \
+    if ((N) != d_none) {                                       \
+        MOV32w(S, (N));                                        \
+        ST_W(S, xEmu, offsetof(x64emu_t, df));                 \
+        if (dyn->f.pending == SF_PENDING                       \
+            && dyn->insts[ninst].x64.need_after                \
+            && !(dyn->insts[ninst].x64.need_after & X_PEND)) { \
+            CALL_(UpdateFlags, -1, 0);                         \
+            dyn->f.pending = SF_SET;                           \
+            SET_NODF();                                        \
+        }                                                      \
+        dyn->f.dfnone = 0;                                     \
+    } else                                                     \
+        SET_DFNONE()
+
+#define SET_NODF() dyn->f.dfnone = 0
+#define SET_DFOK()     \
+    dyn->f.dfnone = 1; \
+    dyn->f.dfnone_here = 1
+
+#define CLEAR_FLAGS_(s)                                                                                       \
+    MOV64x(s, (1UL << F_AF) | (1UL << F_CF) | (1UL << F_OF) | (1UL << F_ZF) | (1UL << F_SF) | (1UL << F_PF)); \
+    ANDN(xFlags, xFlags, s);
 
 #define CLEAR_FLAGS(s) \
-    IFX(X_ALL) { CLEAR_FLAGS_(s) }
+    IFX (X_ALL) { CLEAR_FLAGS_(s) }
 
 #define CALC_SUB_FLAGS(op1_, op2, res, scratch1, scratch2, width)     \
-    IFX(X_AF | X_CF | X_OF)                                           \
-    {                                                                 \
+    IFX (X_AF | X_CF | X_OF) {                                        \
         /* calc borrow chain */                                       \
         /* bc = (res & (~op1 | op2)) | (~op1 & op2) */                \
         OR(scratch1, op1_, op2);                                      \
         AND(scratch2, res, scratch1);                                 \
         AND(op1_, op1_, op2);                                         \
         OR(scratch2, scratch2, op1_);                                 \
-        IFX(X_AF)                                                     \
-        {                                                             \
+        IFX (X_AF) {                                                  \
             /* af = bc & 0x8 */                                       \
             ANDI(scratch1, scratch2, 8);                              \
             BEQZ(scratch1, 8);                                        \
             ORI(xFlags, xFlags, 1 << F_AF);                           \
         }                                                             \
-        IFX(X_CF)                                                     \
-        {                                                             \
+        IFX (X_CF) {                                                  \
             /* cf = bc & (1<<(width-1)) */                            \
             if ((width) == 8) {                                       \
                 ANDI(scratch1, scratch2, 0x80);                       \
@@ -626,8 +643,7 @@
             BEQZ(scratch1, 8);                                        \
             ORI(xFlags, xFlags, 1 << F_CF);                           \
         }                                                             \
-        IFX(X_OF)                                                     \
-        {                                                             \
+        IFX (X_OF) {                                                  \
             /* of = ((bc >> (width-2)) ^ (bc >> (width-1))) & 0x1; */ \
             SRLI_D(scratch1, scratch2, (width)-2);                    \
             SRLI_D(scratch2, scratch1, 1);                            \
@@ -639,7 +655,9 @@
     }
 
 #ifndef MAYSETFLAGS
-#define MAYSETFLAGS() do {} while (0)
+#define MAYSETFLAGS() \
+    do {              \
+    } while (0)
 #endif
 
 #ifndef READFLAGS
@@ -709,10 +727,12 @@
 #define TABLE64(A, V)
 #endif
 
-#define ARCH_INIT()
+#define ARCH_INIT() SMSTART()
+
+#define ARCH_RESET()
 
 #if STEP < 2
-#define GETIP(A) TABLE64(0, 0)
+#define GETIP(A)  TABLE64(0, 0)
 #define GETIP_(A) TABLE64(0, 0)
 #else
 // put value in the Table64 even if not using it for now to avoid difference between Step2 and Step3. Needs to be optimized later...
@@ -803,6 +823,9 @@ void* la64_next(x64emu_t* emu, uintptr_t addr);
 #define emit_add8c          STEPNAME(emit_add8c)
 #define emit_add16          STEPNAME(emit_add16)
 #define emit_adc32          STEPNAME(emit_adc32)
+#define emit_adc8           STEPNAME(emit_adc8)
+#define emit_adc8c          STEPNAME(emit_adc8c)
+#define emit_adc16          STEPNAME(emit_adc16)
 #define emit_sub16          STEPNAME(emit_sub16)
 #define emit_sub32          STEPNAME(emit_sub32)
 #define emit_sub32c         STEPNAME(emit_sub32c)
@@ -847,6 +870,7 @@ void* la64_next(x64emu_t* emu, uintptr_t addr);
 #define emit_sar32c         STEPNAME(emit_sar32c)
 #define emit_shld32c        STEPNAME(emit_shld32c)
 #define emit_shrd32c        STEPNAME(emit_shrd32c)
+#define emit_ror32          STEPNAME(emit_ror32)
 #define emit_ror32c         STEPNAME(emit_ror32c)
 #define emit_rol32          STEPNAME(emit_rol32)
 #define emit_rol32c         STEPNAME(emit_rol32c)
@@ -855,8 +879,8 @@ void* la64_next(x64emu_t* emu, uintptr_t addr);
 
 #define x87_restoreround  STEPNAME(x87_restoreround)
 #define sse_setround      STEPNAME(sse_setround)
-#define x87_forget       STEPNAME(x87_forget)
-#define sse_purge07cache STEPNAME(sse_purge07cache)
+#define x87_forget        STEPNAME(x87_forget)
+#define sse_purge07cache  STEPNAME(sse_purge07cache)
 #define sse_get_reg       STEPNAME(sse_get_reg)
 #define sse_get_reg_empty STEPNAME(sse_get_reg_empty)
 #define sse_forget_reg    STEPNAME(sse_forget_reg)
@@ -904,6 +928,9 @@ void emit_add8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4);
 void emit_add8c(dynarec_la64_t* dyn, int ninst, int s1, int32_t c, int s2, int s3, int s4);
 void emit_add16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4, int s5);
 void emit_adc32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s3, int s4, int s5, int s6);
+void emit_adc8(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4, int s5);
+void emit_adc8c(dynarec_la64_t* dyn, int ninst, int s1, int32_t c, int s3, int s4, int s5, int s6);
+void emit_adc16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4, int s5);
 void emit_sub16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4, int s5);
 void emit_sub32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s3, int s4, int s5);
 void emit_sub32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int64_t c, int s2, int s3, int s4, int s5);
@@ -948,6 +975,7 @@ void emit_sar16(dynarec_la64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
 void emit_sar32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, int s3, int s4);
 void emit_shld32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, uint32_t c, int s3, int s4);
 void emit_shrd32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, uint32_t c, int s3, int s4);
+void emit_ror32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s3, int s4);
 void emit_ror32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, int s3, int s4);
 void emit_rol32(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s3, int s4);
 void emit_rol32c(dynarec_la64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, int s3, int s4);
@@ -1124,13 +1152,13 @@ uintptr_t dynarec64_F20F(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int
         }
 
 // Restore xFlags from LBT.eflags
-#define RESTORE_EFLAGS(s)               \
-    do {                                \
-        if (la64_lbt) {                 \
-            CLEAR_FLAGS_(s);            \
-            X64_GET_EFLAGS(s, X_ALL);   \
-            OR(xFlags, xFlags, s);      \
-        }                               \
+#define RESTORE_EFLAGS(s)             \
+    do {                              \
+        if (la64_lbt) {               \
+            CLEAR_FLAGS_(s);          \
+            X64_GET_EFLAGS(s, X_ALL); \
+            OR(xFlags, xFlags, s);    \
+        }                             \
     } while (0)
 
 // Spill xFlags to LBT.eflags
@@ -1141,12 +1169,6 @@ uintptr_t dynarec64_F20F(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int
         }                                  \
     } while (0)
 
-#define REGENERATE_MASK()       \
-    do {                        \
-        ADDI_W(xMASK, xZR, -1); \
-        LU32I_D(xMASK, 0);      \
-    } while (0)
-
-#define PURGE_YMM()    /* TODO */
+#define PURGE_YMM() /* TODO */
 
 #endif //__DYNAREC_LA64_HELPER_H__
