@@ -19,13 +19,15 @@
 
 #include "rv64_printer.h"
 #include "dynarec_rv64_private.h"
-#include "dynarec_rv64_helper.h"
+#include "../dynarec_helper.h"
 #include "dynarec_rv64_functions.h"
 
 
 uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ninst, rex_t rex, int rep, int* ok, int* need_epilog)
 {
-    (void)ip; (void)rep; (void)need_epilog;
+    (void)ip;
+    (void)rep;
+    (void)need_epilog;
 
     uint8_t nextop = F8;
     uint8_t ed;
@@ -44,7 +46,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
     MAYUSE(v2);
     MAYUSE(j64);
 
-    switch(nextop) {
+    switch (nextop) {
         case 0xC0:
         case 0xC1:
         case 0xC2:
@@ -54,9 +56,9 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
         case 0xC6:
         case 0xC7:
             INST_NAME("FLD STx");
-            X87_PUSH_OR_FAIL(v2, dyn, ninst, x1, X87_ST(nextop&7));
-            v1 = x87_get_st(dyn, ninst, x1, x2, (nextop&7)+1, X87_COMBINE(0, (nextop&7)+1));
-            if(ST_IS_F(0)) {
+            X87_PUSH_OR_FAIL(v2, dyn, ninst, x1, X87_ST(nextop & 7));
+            v1 = x87_get_st(dyn, ninst, x1, x2, (nextop & 7) + 1, X87_COMBINE(0, (nextop & 7) + 1));
+            if (ST_IS_F(0)) {
                 FMVS(v2, v1);
             } else {
                 FMVD(v2, v1);
@@ -75,9 +77,9 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
         case 0xCF:
             INST_NAME("FXCH STx");
             // swap the cache value, not the double value itself :p
-            x87_get_st(dyn, ninst, x1, x2, nextop&7, X87_ST(nextop&7));
+            x87_get_st(dyn, ninst, x1, x2, nextop & 7, X87_ST(nextop & 7));
             x87_get_st(dyn, ninst, x1, x2, 0, X87_ST0);
-            x87_swapreg(dyn, ninst, x1, x2, 0, nextop&7);
+            x87_swapreg(dyn, ninst, x1, x2, 0, nextop & 7);
             // should set C1 to 0
             break;
 
@@ -98,15 +100,15 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
         case 0xDF:
             INST_NAME("FSTPNCE ST0, STx");
             // copy the cache value for st0 to stx
-            x87_get_st_empty(dyn, ninst, x1, x2, nextop&7, X87_ST(nextop&7));
+            x87_get_st_empty(dyn, ninst, x1, x2, nextop & 7, X87_ST(nextop & 7));
             x87_get_st(dyn, ninst, x1, x2, 0, X87_ST0);
-            x87_swapreg(dyn, ninst, x1, x2, 0, nextop&7);
+            x87_swapreg(dyn, ninst, x1, x2, 0, nextop & 7);
             X87_POP_OR_FAIL(dyn, ninst, x3);
             break;
         case 0xE0:
             INST_NAME("FCHS");
             v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_ST0);
-            if(ST_IS_F(0)) {
+            if (ST_IS_F(0)) {
                 FNEGS(v1, v1);
             } else {
                 FNEGD(v1, v1);
@@ -115,7 +117,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
         case 0xE1:
             INST_NAME("FABS");
             v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_ST0);
-            if(ST_IS_F(0)) {
+            if (ST_IS_F(0)) {
                 FABSS(v1, v1);
             } else {
                 FABSD(v1, v1);
@@ -128,27 +130,27 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             break;
         case 0xE5:
             INST_NAME("FXAM");
-            #if 1
+#if 1
             i1 = x87_get_current_cache(dyn, ninst, 0, EXT_CACHE_ST_D);
             // value put in x14
-            if(i1==-1) {
-                if(fpu_is_st_freed(dyn, ninst, 0)) {
+            if (i1 == -1) {
+                if (fpu_is_st_freed(dyn, ninst, 0)) {
                     MOV32w(x4, 0b100000100000000);
                     B_MARK3_nocond;
                 } else {
                     // not in cache, so check Empty status and load it
                     i2 = -dyn->e.x87stack;
                     LWU(x3, xEmu, offsetof(x64emu_t, fpu_stack));
-                    if(i2) {
+                    if (i2) {
                         ADDI(x3, x3, i2);
                     }
                     MOV32w(x4, 0b100000100000000); // empty: C3,C2,C0 = 101
                     BGE_MARK3(xZR, x3);
                     // x5 will be the actual top
                     LWU(x5, xEmu, offsetof(x64emu_t, top));
-                    if(i2) {
+                    if (i2) {
                         ADDI(x5, x5, i2);
-                        ANDI(x5, x5, 7);    // (emu->top + i)&7
+                        ANDI(x5, x5, 7); // (emu->top + i)&7
                     }
                     // load tag
                     LHU(x3, xEmu, offsetof(x64emu_t, fpu_tags));
@@ -162,7 +164,12 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                     ANDI(x2, x3, 0b11);
                     BNEZ_MARK3(x2); // empty: C3,C2,C0 = 101
                     // load x2 with ST0 anyway, for sign extraction
-                    if(rv64_zba) SH3ADD(x1, x5, xEmu); else {SLLI(x5, x5, 3); ADD(x1, xEmu, x5);}
+                    if (rv64_zba)
+                        SH3ADD(x1, x5, xEmu);
+                    else {
+                        SLLI(x5, x5, 3);
+                        ADD(x1, xEmu, x5);
+                    }
                     LD(x2, x1, offsetof(x64emu_t, x87));
                 }
             } else {
@@ -171,9 +178,9 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                 FMVXD(x2, v1);
             }
             // get exponant in x1
-            SRLI(x1, x2, 20+32);
+            SRLI(x1, x2, 20 + 32);
             ANDI(x1, x1, 0x7ff); // 0x7ff
-            BNEZ_MARK(x1);  // not zero or denormal
+            BNEZ_MARK(x1);       // not zero or denormal
             MOV64x(x3, 0x7fffffffffffffff);
             AND(x1, x2, x3);
             MOV32w(x4, 0b100000000000000); // Zero: C3,C2,C0 = 100
@@ -181,11 +188,11 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             MOV32w(x4, 0b100010000000000); // Denormal: C3,C2,C0 = 110
             B_MARK3_nocond;
             MARK;
-            ADDI(x3, xZR, 0x7ff);   // infinite/NaN?
+            ADDI(x3, xZR, 0x7ff);          // infinite/NaN?
             MOV32w(x4, 0b000010000000000); // normal: C3,C2,C0 = 010
             BNE_MARK3(x1, x3);
             SLLI(x3, x2, 12);
-            SRLI(x3, x3, 12);   // and 0x000fffffffffffff
+            SRLI(x3, x3, 12);              // and 0x000fffffffffffff
             MOV32w(x4, 0b000010100000000); // infinity: C3,C2,C0 = 011
             BEQZ_MARK3(x3);
             MOV32w(x4, 0b000000100000000); // NaN: C3,C2,C0 = 001
@@ -193,25 +200,25 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             // Extract signa & Update SW
             SRLI(x1, x2, 63);
             SLLI(x1, x1, 9);
-            OR(x4, x4, x1); //C1
+            OR(x4, x4, x1); // C1
             LHU(x1, xEmu, offsetof(x64emu_t, sw));
             MOV32w(x2, ~0b0100011100000000);
             AND(x1, x1, x2);
             OR(x4, x4, x1);
             SH(x4, xEmu, offsetof(x64emu_t, sw));
-            #else
+#else
             MESSAGE(LOG_DUMP, "Need Optimization\n");
             x87_refresh(dyn, ninst, x1, x2, 0);
             s0 = x87_stackcount(dyn, ninst, x1);
-            CALL(fpu_fxam, -1);  // should be possible inline, but is it worth it?
+            CALL(fpu_fxam, -1); // should be possible inline, but is it worth it?
             x87_unstackcount(dyn, ninst, x1, s0);
-            #endif
+#endif
             break;
 
         case 0xE8:
             INST_NAME("FLD1");
             X87_PUSH_OR_FAIL(v1, dyn, ninst, x1, EXT_CACHE_ST_F);
-            if(ST_IS_F(0)) {
+            if (ST_IS_F(0)) {
                 MOV32w(x1, 0x3f800000);
                 FMVWX(v1, x1);
             } else {
@@ -247,7 +254,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
         case 0xEE:
             INST_NAME("FLDZ");
             X87_PUSH_OR_FAIL(v1, dyn, ninst, x1, EXT_CACHE_ST_F);
-            if(ST_IS_F(0)) {
+            if (ST_IS_F(0)) {
                 FMVWX(v1, xZR);
             } else {
                 FMVDX(v1, xZR);
@@ -280,7 +287,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             CALL(native_ftan, -1);
             x87_unstackcount(dyn, ninst, x3, s0);
             X87_PUSH_OR_FAIL(v1, dyn, ninst, x1, EXT_CACHE_ST_F);
-            if(ST_IS_F(0)) {
+            if (ST_IS_F(0)) {
                 MOV32w(x1, 0x3f800000);
                 FMVWX(v1, x1);
             } else {
@@ -354,7 +361,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
         case 0xFA:
             INST_NAME("FSQRT");
             v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_ST0);
-            if(ST_IS_F(0)) {
+            if (ST_IS_F(0)) {
                 FSQRTS(v1, v1);
             } else {
                 FSQRTD(v1, v1);
@@ -376,7 +383,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             v2 = fpu_get_scratch(dyn);
             u8 = x87_setround(dyn, ninst, x1, x2);
 
-            if(ST_IS_F(0)) {
+            if (ST_IS_F(0)) {
                 FEQS(x2, v0, v0);
                 BNEZ_MARK(x2);
                 B_NEXT_nocond;
@@ -450,20 +457,20 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             break;
 
         default:
-            switch((nextop>>3)&7) {
+            switch ((nextop >> 3) & 7) {
                 case 0:
                     INST_NAME("FLD ST0, float[ED]");
-                    X87_PUSH_OR_FAIL(v1, dyn, ninst, x1, box64_dynarec_x87double?EXT_CACHE_ST_D:EXT_CACHE_ST_F);
+                    X87_PUSH_OR_FAIL(v1, dyn, ninst, x1, box64_dynarec_x87double ? EXT_CACHE_ST_D : EXT_CACHE_ST_F);
                     addr = geted(dyn, addr, ninst, nextop, &ed, x2, x1, &fixedaddress, rex, NULL, 1, 0);
                     FLW(v1, ed, fixedaddress);
-                    if(!ST_IS_F(0)) {
+                    if (!ST_IS_F(0)) {
                         FCVTDS(v1, v1);
                     }
                     break;
                 case 2:
                     INST_NAME("FST float[ED], ST0");
                     v1 = x87_get_st(dyn, ninst, x1, x2, 0, EXT_CACHE_ST_F);
-                    if(ST_IS_F(0))
+                    if (ST_IS_F(0))
                         s0 = v1;
                     else {
                         s0 = fpu_get_scratch(dyn);
@@ -476,7 +483,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                     INST_NAME("FSTP float[ED], ST0");
                     v1 = x87_get_st(dyn, ninst, x1, x2, 0, EXT_CACHE_ST_F);
                     addr = geted(dyn, addr, ninst, nextop, &ed, x2, x1, &fixedaddress, rex, NULL, 1, 0);
-                    if(!ST_IS_F(0)) {
+                    if (!ST_IS_F(0)) {
                         FCVTSD(v1, v1);
                     }
                     FSW(v1, ed, fixedaddress);
@@ -487,7 +494,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                     MESSAGE(LOG_DUMP, "Need Optimization\n");
                     fpu_purgecache(dyn, ninst, 0, x1, x2, x3); // maybe only x87, not SSE?
                     addr = geted(dyn, addr, ninst, nextop, &ed, x1, x2, &fixedaddress, rex, NULL, 0, 0);
-                    if(ed!=x1) {
+                    if (ed != x1) {
                         MV(x1, ed);
                     }
                     MOV32w(x2, 0);
@@ -496,14 +503,14 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                 case 5:
                     INST_NAME("FLDCW Ew");
                     GETEW(x1, 0);
-                    SH(x1, xEmu, offsetof(x64emu_t, cw));    // hopefully cw is not too far for an imm8
+                    SH(x1, xEmu, offsetof(x64emu_t, cw)); // hopefully cw is not too far for an imm8
                     break;
                 case 6:
                     INST_NAME("FNSTENV Ed");
                     MESSAGE(LOG_DUMP, "Need Optimization\n");
                     fpu_purgecache(dyn, ninst, 0, x1, x2, x3); // maybe only x87, not SSE?
                     addr = geted(dyn, addr, ninst, nextop, &ed, x1, x2, &fixedaddress, rex, NULL, 0, 0);
-                    if(ed!=x1) {
+                    if (ed != x1) {
                         MV(x1, ed);
                     }
                     MOV32w(x2, 0);
