@@ -19,13 +19,15 @@
 
 #include "rv64_printer.h"
 #include "dynarec_rv64_private.h"
-#include "dynarec_rv64_helper.h"
+#include "../dynarec_helper.h"
 #include "dynarec_rv64_functions.h"
 
 
 uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ninst, rex_t rex, int rep, int* ok, int* need_epilog)
 {
-    (void)ip; (void)rep; (void)need_epilog;
+    (void)ip;
+    (void)rep;
+    (void)need_epilog;
 
     uint8_t nextop = F8;
     uint8_t ed;
@@ -42,7 +44,7 @@ uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
     MAYUSE(v1);
     MAYUSE(j64);
 
-    switch(nextop) {
+    switch (nextop) {
         case 0xC0:
         case 0xC1:
         case 0xC2:
@@ -53,14 +55,14 @@ uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
         case 0xC7:
             INST_NAME("FCMOVNB ST0, STx");
             READFLAGS(X_CF);
-            v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop&7));
-            v2 = x87_get_st(dyn, ninst, x1, x2, nextop&7, X87_COMBINE(0, nextop&7));
-            ANDI(x1, xFlags, 1<<F_CF);
+            v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop & 7));
+            v2 = x87_get_st(dyn, ninst, x1, x2, nextop & 7, X87_COMBINE(0, nextop & 7));
+            ANDI(x1, xFlags, 1 << F_CF);
             CBNZ_NEXT(x1);
-            if(ST_IS_F(0)) {
+            if (ST_IS_F(0)) {
                 FMVS(v1, v2);
             } else {
-                FMVD(v1, v2);    // F_CF==0
+                FMVD(v1, v2); // F_CF==0
             }
             break;
         case 0xC8:
@@ -73,14 +75,14 @@ uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
         case 0xCF:
             INST_NAME("FCMOVNE ST0, STx");
             READFLAGS(X_ZF);
-            v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop&7));
-            v2 = x87_get_st(dyn, ninst, x1, x2, nextop&7, X87_COMBINE(0, nextop&7));
-            ANDI(x1, xFlags, 1<<F_ZF);
+            v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop & 7));
+            v2 = x87_get_st(dyn, ninst, x1, x2, nextop & 7, X87_COMBINE(0, nextop & 7));
+            ANDI(x1, xFlags, 1 << F_ZF);
             CBNZ_NEXT(x1);
-            if(ST_IS_F(0)) {
+            if (ST_IS_F(0)) {
                 FMVS(v1, v2);
             } else {
-                FMVD(v1, v2);        // F_ZF==0
+                FMVD(v1, v2); // F_ZF==0
             }
             break;
         case 0xD0:
@@ -92,15 +94,15 @@ uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
         case 0xD6:
         case 0xD7:
             INST_NAME("FCMOVNBE ST0, STx");
-            READFLAGS(X_CF|X_ZF);
-            v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop&7));
-            v2 = x87_get_st(dyn, ninst, x1, x2, nextop&7, X87_COMBINE(0, nextop&7));
-            ANDI(x1, xFlags, (1<<F_CF)|(1<<F_ZF));
+            READFLAGS(X_CF | X_ZF);
+            v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop & 7));
+            v2 = x87_get_st(dyn, ninst, x1, x2, nextop & 7, X87_COMBINE(0, nextop & 7));
+            ANDI(x1, xFlags, (1 << F_CF) | (1 << F_ZF));
             CBNZ_NEXT(x1);
-            if(ST_IS_F(0)) {
+            if (ST_IS_F(0)) {
                 FMVS(v1, v2);
             } else {
-                FMVD(v1, v2);   // F_CF==0 & F_ZF==0
+                FMVD(v1, v2); // F_CF==0 & F_ZF==0
             }
             break;
         case 0xD8:
@@ -113,14 +115,14 @@ uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
         case 0xDF:
             INST_NAME("FCMOVNU ST0, STx");
             READFLAGS(X_PF);
-            v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop&7));
-            v2 = x87_get_st(dyn, ninst, x1, x2, nextop&7, X87_COMBINE(0, nextop&7));
-            ANDI(x1, xFlags, 1<<F_PF);
+            v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop & 7));
+            v2 = x87_get_st(dyn, ninst, x1, x2, nextop & 7, X87_COMBINE(0, nextop & 7));
+            ANDI(x1, xFlags, 1 << F_PF);
             CBNZ_NEXT(x1);
-            if(ST_IS_F(0)) {
+            if (ST_IS_F(0)) {
                 FMVS(v1, v2);
             } else {
-                FMVD(v1, v2);        // F_PF==0
+                FMVD(v1, v2); // F_PF==0
             }
             break;
         case 0xE1:
@@ -130,7 +132,7 @@ uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             INST_NAME("FNCLEX");
             LH(x2, xEmu, offsetof(x64emu_t, sw));
             ANDI(x2, x2, ~(0xff));  // IE .. PE, SF, ES
-            MOV32w(x1, ~(1<<15));   // B
+            MOV32w(x1, ~(1 << 15)); // B
             AND(x2, x2, x1);
             SH(x2, xEmu, offsetof(x64emu_t, sw));
             break;
@@ -149,9 +151,9 @@ uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
         case 0xEE:
         case 0xEF:
             INST_NAME("FUCOMI ST0, STx");
-            SETFLAGS(X_ALL, SF_SET);
-            v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop&7));
-            v2 = x87_get_st(dyn, ninst, x1, x2, nextop&7, X87_COMBINE(0, nextop&7));
+            SETFLAGS(X_ALL, SF_SET, NAT_FLAGS_NOFUSION);
+            v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop & 7));
+            v2 = x87_get_st(dyn, ninst, x1, x2, nextop & 7, X87_COMBINE(0, nextop & 7));
             if (ST_IS_F(0)) {
                 FCOMIS(v1, v2, x1, x2, x3, x4, x5);
             } else {
@@ -168,7 +170,7 @@ uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
         case 0xF6:
         case 0xF7:
             INST_NAME("FCOMI ST0, STx");
-            SETFLAGS(X_ALL, SF_SET);
+            SETFLAGS(X_ALL, SF_SET, NAT_FLAGS_NOFUSION);
             v1 = x87_get_st(dyn, ninst, x1, x2, 0, X87_COMBINE(0, nextop & 7));
             v2 = x87_get_st(dyn, ninst, x1, x2, nextop & 7, X87_COMBINE(0, nextop & 7));
             if (ST_IS_F(0)) {
@@ -187,13 +189,13 @@ uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             break;
 
         default:
-            switch((nextop>>3)&7) {
+            switch ((nextop >> 3) & 7) {
                 case 0:
                     INST_NAME("FILD ST0, Ed");
                     X87_PUSH_OR_FAIL(v1, dyn, ninst, x1, EXT_CACHE_ST_D);
                     addr = geted(dyn, addr, ninst, nextop, &ed, x2, x1, &fixedaddress, rex, NULL, 1, 0);
                     LW(x1, ed, fixedaddress);
-                    FCVTDW(v1, x1, RD_RNE);    // i32 -> double
+                    FCVTDW(v1, x1, RD_RNE); // i32 -> double
                     break;
                 case 1:
                     INST_NAME("FISTTP Ed, ST0");
@@ -223,14 +225,14 @@ uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                     u8 = x87_setround(dyn, ninst, x1, x2);
                     addr = geted(dyn, addr, ninst, nextop, &wback, x2, x3, &fixedaddress, rex, NULL, 1, 0);
                     v2 = fpu_get_scratch(dyn);
-                    if(!box64_dynarec_fastround) {
+                    if (!box64_dynarec_fastround) {
                         FSFLAGSI(0); // reset all bits
                     }
                     FCVTWD(x4, v1, RD_DYN);
                     x87_restoreround(dyn, ninst, u8);
-                    if(!box64_dynarec_fastround) {
-                        FRFLAGS(x5);   // get back FPSR to check the IOC bit
-                        ANDI(x5, x5, 1<<FR_NV);
+                    if (!box64_dynarec_fastround) {
+                        FRFLAGS(x5); // get back FPSR to check the IOC bit
+                        ANDI(x5, x5, 1 << FR_NV);
                         BEQ_MARK2(x5, xZR);
                         MOV32w(x4, 0x80000000);
                     }
@@ -241,28 +243,28 @@ uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                 case 5:
                     INST_NAME("FLD tbyte");
                     addr = geted(dyn, addr, ninst, nextop, &ed, x1, x2, &fixedaddress, rex, NULL, 8, 0);
-                    if((PK(0)==0xDB && ((PK(1)>>3)&7)==7) || (PK(0)>=0x40 && PK(0)<=0x4f && PK(1)==0xDB && ((PK(2)>>3)&7)==7)) {
+                    if ((PK(0) == 0xDB && ((PK(1) >> 3) & 7) == 7) || (PK(0) >= 0x40 && PK(0) <= 0x4f && PK(1) == 0xDB && ((PK(2) >> 3) & 7) == 7)) {
                         // the FLD is immediatly followed by an FSTP
-                        LD(x5, ed, fixedaddress+0);
-                        LH(x6, ed, fixedaddress+8);
+                        LD(x5, ed, fixedaddress + 0);
+                        LH(x6, ed, fixedaddress + 8);
                         // no persistant scratch register, so unrool both instruction here...
                         MESSAGE(LOG_DUMP, "\tHack: FSTP tbyte\n");
-                        nextop = F8;    // 0xDB or rex
-                        if(nextop>=0x40 && nextop<=0x4f) {
+                        nextop = F8; // 0xDB or rex
+                        if (nextop >= 0x40 && nextop <= 0x4f) {
                             rex.rex = nextop;
-                            nextop = F8;    //0xDB
+                            nextop = F8; // 0xDB
                         } else
                             rex.rex = 0;
-                        nextop = F8;    //modrm
+                        nextop = F8; // modrm
                         addr = geted(dyn, addr, ninst, nextop, &ed, x1, x2, &fixedaddress, rex, NULL, 8, 0);
-                        SD(x5, ed, fixedaddress+0);
-                        SH(x6, ed, fixedaddress+8);
+                        SD(x5, ed, fixedaddress + 0);
+                        SH(x6, ed, fixedaddress + 8);
                     } else {
-                        if(box64_x87_no80bits) {
+                        if (box64_x87_no80bits) {
                             X87_PUSH_OR_FAIL(v1, dyn, ninst, x1, EXT_CACHE_ST_D);
                             FLD(v1, ed, fixedaddress);
                         } else {
-                            if(ed!=x1) {
+                            if (ed != x1) {
                                 ADDI(x1, ed, fixedaddress);
                             }
                             X87_PUSH_EMPTY_OR_FAIL(dyn, ninst, x3);
@@ -272,14 +274,14 @@ uintptr_t dynarec64_DB(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                     break;
                 case 7:
                     INST_NAME("FSTP tbyte");
-                    if(box64_x87_no80bits) {
+                    if (box64_x87_no80bits) {
                         v1 = x87_get_st(dyn, ninst, x1, x2, 0, EXT_CACHE_ST_D);
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, x1, &fixedaddress, rex, NULL, 1, 0);
                         FSD(v1, wback, fixedaddress);
                     } else {
                         x87_forget(dyn, ninst, x1, x3, 0);
                         addr = geted(dyn, addr, ninst, nextop, &ed, x1, x2, &fixedaddress, rex, NULL, 0, 0);
-                        if(ed!=x1) {
+                        if (ed != x1) {
                             MV(x1, ed);
                         }
                         s0 = x87_stackcount(dyn, ninst, x3);
