@@ -5,6 +5,9 @@
 
 typedef struct box64context_s box64context_t;
 typedef struct x64_ucontext_s x64_ucontext_t;
+#ifdef BOX32
+typedef struct i386_ucontext_s i386_ucontext_t;
+#endif
 
 #define ERR_UNIMPL  1
 #define ERR_DIVBY0  2
@@ -37,7 +40,6 @@ typedef struct x64test_s {
     int         memsize;
     int         test;
     int         clean;
-    int         notest;
     uint8_t     mem[32];
 } x64test_t;
 
@@ -56,6 +58,8 @@ typedef struct emu_flags_s {
 #define JUMPBUFF struct __jmp_buf_tag
 #endif
 
+#define N_SCRATCH 200
+
 typedef struct x64emu_s {
     // cpu
     reg64_t     regs[16];
@@ -87,9 +91,6 @@ typedef struct x64emu_s {
     multiuint_t op1;
     multiuint_t op2;
     multiuint_t res;
-    multiuint_t op1_sav;    // for dec/inc deferred flags, to be able to compute CF
-    multiuint_t res_sav;
-    deferred_flags_t df_sav;
     uint32_t    *x64emu_parity_tab; // helper
     // segments
     uint16_t    segs[6];        // only 32bits value?
@@ -116,7 +117,8 @@ typedef struct x64emu_s {
     uintptr_t   prev2_ip;
     #endif
     // scratch stack, used for alignment of double and 64bits ints on arm. 200 elements should be enough
-    uint64_t    scratch[200];
+    __int128_t  dummy_align;    // here to have scratch 128bits aligned
+    uint64_t    scratch[N_SCRATCH];
     // local stack, do be deleted when emu is freed
     void*       stack2free; // this is the stack to free (can be NULL)
     void*       init_stack; // initial stack (owned or not)
@@ -126,9 +128,24 @@ typedef struct x64emu_s {
     uintptr_t   old_savedsp;
     #endif
 
-    x64_ucontext_t *uc_link; // to handle setcontext
+    void*       uc_link; // to handle setcontext (can be x64_ucontext_t or a i386_ucontext_t)
 
     int         type;       // EMUTYPE_xxx define
+    #ifdef BOX32
+    int         libc_err;   // copy of errno from libc
+    int         libc_herr;  // copy of h_errno from libc
+    unsigned short          libctype[384];   // copy from __ctype_b address might be too high
+    const unsigned short*   ref_ctype;
+    const unsigned short*   ctype;
+    int         libctolower[384];   // copy from __ctype_b_tolower address might be too high
+    const int*  ref_tolower;
+    const int*  tolower;
+    int         libctoupper[384];   // copy from __ctype_b_toupper address might be too high
+    const int*  ref_toupper;
+    const int*  toupper;
+    void*       res_state_32;  //32bits version of res_state
+    void*       res_state_64;
+    #endif
 } x64emu_t;
 
 #define EMUTYPE_NONE    0
