@@ -27,6 +27,9 @@
 #include "modrm.h"
 
 int my_setcontext(x64emu_t* emu, void* ucp);
+#ifdef BOX32
+int my32_setcontext(x64emu_t* emu, void* ucp);
+#endif
 
 #ifdef TEST_INTERPRETER
 int RunTest(x64test_t *test)
@@ -68,6 +71,7 @@ int Run(x64emu_t *emu, int step)
 #ifdef TEST_INTERPRETER
     test->memsize = 0;
 #else
+    check_exec(emu, R_RIP);
 x64emurun:
     while(1) 
 #endif
@@ -241,6 +245,8 @@ x64emurun:
             }
             emu->segs[_SS] = Pop32(emu);    // no check, no use....
             emu->segs_serial[_SS] = 0;
+            if(ACCESS_FLAG(F_TF))
+                tf_next = 1;
             break;
 
         case 0x1E:                      /* PUSH DS */
@@ -392,7 +398,7 @@ x64emurun:
         case 0x5E:
         case 0x5F:                      /* POP Reg */
             tmp8u = (opcode&7)+(rex.b<<3);
-            emu->regs[tmp8u].q[0] = is32bits?Pop32(emu):Pop64(emu);
+            emu->regs[tmp8u].q[0] = rex.is32bits?Pop32(emu):Pop64(emu);
             break;
         case 0x60:                      /* PUSHAD */
             if(rex.is32bits) {
@@ -465,9 +471,22 @@ x64emurun:
                 R_RIP = addr;
                 goto fini;
             }
-            is32bits = (emu->segs[_CS]==0x23);
-            if(is32bits)
-                running32bits = 1;
+            if(is32bits!=(emu->segs[_CS]==0x23)) {
+                is32bits = (emu->segs[_CS]==0x23);
+                if(is32bits) {
+                    // Zero upper part of the 32bits regs
+                    R_RAX = R_EAX;
+                    R_RBX = R_EBX;
+                    R_RCX = R_ECX;
+                    R_RDX = R_EDX;
+                    R_RSP = R_ESP;
+                    R_RBP = R_EBP;
+                    R_RSI = R_ESI;
+                    R_RDI = R_EDI;
+                }
+                if(is32bits)
+                    running32bits = 1;
+            }
             #endif
             break;
         case 0x65:                      /* GS: prefix */
@@ -483,9 +502,22 @@ x64emurun:
                 R_RIP = addr;
                 goto fini;
             }
-            is32bits = (emu->segs[_CS]==0x23);
-            if(is32bits)
-                running32bits = 1;
+            if(is32bits!=(emu->segs[_CS]==0x23)) {
+                is32bits = (emu->segs[_CS]==0x23);
+                if(is32bits) {
+                    // Zero upper part of the 32bits regs
+                    R_RAX = R_EAX;
+                    R_RBX = R_EBX;
+                    R_RCX = R_ECX;
+                    R_RDX = R_EDX;
+                    R_RSP = R_ESP;
+                    R_RBP = R_EBP;
+                    R_RSI = R_ESI;
+                    R_RDI = R_EDI;
+                }
+                if(is32bits)
+                    running32bits = 1;
+            }
             #endif
             break;
         case 0x66:                      /* 16bits prefix */
@@ -557,19 +589,18 @@ x64emurun:
         case 0x6D:                      /* INSD DX */
         case 0x6E:                      /* OUTSB DX */
         case 0x6F:                      /* OUTSD DX */
-            #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+#ifndef TEST_INTERPRETER
+            if(rex.is32bits && box64_ignoreint3)
+            {
+            } else {
+                emit_signal(emu, SIGSEGV, (void*)R_RIP, 0xbad0);
+            }
             STEP;
-            #else
-            test->notest = 1;
             #endif
             break;
 
         GOCOND(0x70
             ,   tmp8s = F8S; CHECK_FLAGS(emu);
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
-            #endif
             ,   addr += tmp8s;
             ,,STEP2
             )                           /* Jxx Ib */
@@ -795,6 +826,8 @@ x64emurun:
             GETED(0);
             emu->segs[((nextop&0x38)>>3)] = ED->word[0];
             emu->segs_serial[((nextop&0x38)>>3)] = 0;
+            if(((nextop&0x38)>>3)==_SS && ACCESS_FLAG(F_TF))
+                tf_next = 1;
             break;
         case 0x8F:                      /* POP Ed */
             nextop = F8;
@@ -1333,16 +1366,10 @@ x64emurun:
             addr = rex.is32bits?Pop32(emu):Pop64(emu);
             R_RSP += tmp16u;
             STEP2
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
-            #endif
             break;
         case 0xC3:                      /* RET */
             addr = rex.is32bits?Pop32(emu):Pop64(emu);
             STEP2
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
-            #endif
             break;
         case 0xC4:                      /* LES Gd,Ed */
             nextop = F8;
@@ -1472,7 +1499,11 @@ x64emurun:
             }
             emu->segs_serial[_CS] = 0;
             R_RSP += tmp16u;
-            // need to check status of CS register!
+            is32bits = (R_CS==0x23);    // checking if CS changed
+            #ifndef TEST_INTERPRETER
+            if(is32bits)
+                running32bits = 1;
+            #endif
             STEP2;
             break;
         case 0xCB:                      /* FAR RET */
@@ -1484,16 +1515,19 @@ x64emurun:
                 emu->segs[_CS] = Pop64(emu);    // no check, no use....
             }
             emu->segs_serial[_CS] = 0;
-            // need to check status of CS register!
+            is32bits = (R_CS==0x23);    // checking if CS changed
+            #ifndef TEST_INTERPRETER
+            if(is32bits)
+                running32bits = 1;
+            #endif
             STEP2;
             break;
         case 0xCC:                      /* INT 3 */
+            R_RIP = addr;   // update RIP
             #ifndef TEST_INTERPRETER
             x64Int3(emu, &addr);
             if(emu->quit) goto fini;    // R_RIP is up to date when returning from x64Int3
             addr = R_RIP;
-            #else
-            test->notest = 1;
             #endif
             break;
         case 0xCD:                      /* INT n */
@@ -1511,19 +1545,27 @@ x64emurun:
                 printf_log(LOG_DEBUG, "INT 29 called => __fastfail(0x%x)\n", R_ECX);
                 emit_interruption(emu, 0x29, (void*)R_RIP);
             } else if (tmp8u==0x80) {
+                R_RIP = addr;
+                if(ACCESS_FLAG(F_TF))
+                    tf_next = 1;
                 // 32bits syscall
                 #ifndef TEST_INTERPRETER
                 x86Syscall(emu);
-                STEP;
-                #else
-                test->notest = 1;
+                STEP2;
+                #endif
+            } else if (tmp8u==0x03) {
+                R_RIP = addr;
+                #ifndef TEST_INTERPRETER
+                emit_signal(emu, SIGTRAP, NULL, 3);
+                STEP2;
                 #endif
             } else {
+                if(rex.is32bits && tmp8u==0x04) {
+                    R_RIP = addr;
+                }
                 #ifndef TEST_INTERPRETER
-                emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
-                STEP;
-                #else
-                test->notest = 1;
+                emit_interruption(emu, tmp8u, (void*)R_RIP);
+                STEP2;
                 #endif
             }
             break;
@@ -1533,32 +1575,46 @@ x64emurun:
                 goto fini;
             }
             emu->old_ip = R_RIP;
+            R_RIP = addr;
             #ifndef TEST_INTERPRETER
             CHECK_FLAGS(emu);
             if(ACCESS_FLAG(F_OF))
-                emit_signal(emu, SIGSEGV, (void*)R_RIP, 128);
-            STEP;
+                emit_interruption(emu, 4, (void*)R_RIP);
+            STEP2;
             #endif
             break;
         case 0xCF:                      /* IRET */
-            addr = rex.is32bits?Pop32(emu):Pop64(emu);
-            emu->segs[_CS] = (rex.is32bits?Pop32(emu):Pop64(emu))&0xffff;
+            addr = (!rex.w)?Pop32(emu):Pop64(emu);
+            emu->segs[_CS] = ((!rex.w)?Pop32(emu):Pop64(emu))&0xffff;
             emu->segs_serial[_CS] = 0;
-            emu->eflags.x64 = (((rex.is32bits?Pop32(emu):Pop64(emu)) & 0x3F7FD7)/* & (0xffff-40)*/ ) | 0x2; // mask off res2 and res3 and on res1
-            tmp64u = rex.is32bits?Pop32(emu):Pop64(emu);  //RSP
-            emu->segs[_SS] = (rex.is32bits?Pop32(emu):Pop64(emu))&0xffff;
-            emu->segs_serial[_SS] = 0;
-            R_RSP = tmp64u;
+            emu->eflags.x64 = ((((!rex.w)?Pop32(emu):Pop64(emu)) & 0x3F7FD7)/* & (0xffff-40)*/ ) | 0x2; // mask off res2 and res3 and on res1
+            if(!is32bits || (is32bits && emu->segs[_CS]!=0x23)) {
+                tmp64u = (!rex.w)?Pop32(emu):Pop64(emu);  //RSP
+                emu->segs[_SS] = ((!rex.w)?Pop32(emu):Pop64(emu))&0xffff;
+                emu->segs_serial[_SS] = 0;
+                R_RSP = tmp64u;
+            }
             RESET_FLAGS(emu);
             R_RIP = addr;
             STEP;
-            is32bits = (emu->segs[_CS]==0x23);
-            #ifndef TEST_INTERPRETER
-            if(is32bits)
-                running32bits = 1;
-            #else
-            test->notest = 1;
-            #endif
+            if(is32bits!=(emu->segs[_CS]==0x23)) {
+                is32bits = (emu->segs[_CS]==0x23);
+                if(is32bits) {
+                    // Zero upper part of the 32bits regs
+                    R_RAX = R_EAX;
+                    R_RBX = R_EBX;
+                    R_RCX = R_ECX;
+                    R_RDX = R_EDX;
+                    R_RSP = R_ESP;
+                    R_RBP = R_EBP;
+                    R_RSI = R_ESI;
+                    R_RDI = R_EDI;
+                }
+                #ifndef TEST_INTERPRETER
+                if(is32bits)
+                    running32bits = 1;
+                #endif
+            }
             break;
         case 0xD0:                      /* GRP2 Eb,1 */
         case 0xD2:                      /* GRP2 Eb,CL */
@@ -1648,10 +1704,10 @@ x64emurun:
             break;
         case 0xD8:                      /* x87 opcodes */
             #ifdef TEST_INTERPRETER
-            if(!(addr = TestD8(test, rex, addr)))
+            if(!(addr = TestD8(test, rex, addr, 0)))
                 unimp = 1;
             #else
-            if(!(addr = RunD8(emu, rex, addr))) {
+            if(!(addr = RunD8(emu, rex, addr, 0))) {
                 unimp = 1;
                 goto fini;
             }
@@ -1663,10 +1719,10 @@ x64emurun:
             break;
         case 0xD9:                      /* x87 opcodes */
             #ifdef TEST_INTERPRETER
-            if(!(addr = TestD9(test, rex, addr)))
+            if(!(addr = TestD9(test, rex, addr, 0)))
                 unimp = 1;
             #else
-            if(!(addr = RunD9(emu, rex, addr))) {
+            if(!(addr = RunD9(emu, rex, addr, 0))) {
                 unimp = 1;
                 goto fini;
             }
@@ -1801,10 +1857,11 @@ x64emurun:
         case 0xE7:                      /* OUT XX, EAX */
             // this is a privilege opcode...
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            F8;
+            if(rex.is32bits && box64_ignoreint3)
+            {} else
+            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0xbad0);
             STEP;
-            #else
-            test->notest = 1;
             #endif
             break;
         case 0xE8:                      /* CALL Id */
@@ -1819,9 +1876,6 @@ x64emurun:
                 addr += tmp32s;
             addr = (uintptr_t)getAlternate((void*)addr);
             STEP2
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
-            #endif
             break;
         case 0xE9:                      /* JMP Id */
             tmp32s = F32S; // jmp is relative
@@ -1831,18 +1885,12 @@ x64emurun:
                 addr += tmp32s;
             addr = (uintptr_t)getAlternate((void*)addr);
             STEP2
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
-            #endif
             break;
 
         case 0xEB:                      /* JMP Ib */
             tmp32s = F8S; // jump is relative
             addr += tmp32s;
             STEP2
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
-            #endif
             break;
         case 0xEC:                      /* IN AL, DX */
         case 0xED:                      /* IN EAX, DX */
@@ -1850,10 +1898,10 @@ x64emurun:
         case 0xEF:                      /* OUT DX, EAX */
             // this is a privilege opcode...
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            if(rex.is32bits && box64_ignoreint3)
+            {} else
+            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0xbad0);
             STEP;
-            #else
-            test->notest = 1;
             #endif
             break;
         case 0xF0:                      /* LOCK prefix */
@@ -1881,10 +1929,8 @@ x64emurun:
         case 0xF4:                      /* HLT */
             // this is a privilege opcode...
             #ifndef TEST_INTERPRETER
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0xbad0);
             STEP;
-            #else
-            test->notest = 1;
             #endif
             break;
         case 0xF5:                      /* CMC */
@@ -1915,16 +1961,13 @@ x64emurun:
                     break;
                 case 6:                 /* DIV Eb */
                     if(!EB->byte[0])
-                        emit_div0(emu, (void*)R_RIP, 0);
+                        emit_div0(emu, (void*)R_RIP, 1);
                     div8(emu, EB->byte[0]);
                     break;
                 case 7:                 /* IDIV Eb */
                     if(!EB->byte[0])
-                        emit_div0(emu, (void*)R_RIP, 0);
+                        emit_div0(emu, (void*)R_RIP, 1);
                     idiv8(emu, EB->byte[0]);
-                    #ifdef TEST_INTERPRETER
-                    test->notest = 1;
-                    #endif
                     break;
             }
             break;
@@ -1953,16 +1996,13 @@ x64emurun:
                         break;
                     case 6:                 /* DIV Ed */
                         if(!ED->q[0])
-                            emit_div0(emu, (void*)R_RIP, 0);
+                            emit_div0(emu, (void*)R_RIP, 1);
                         div64(emu, ED->q[0]);
                         break;
                     case 7:                 /* IDIV Ed */
                         if(!ED->q[0])
-                            emit_div0(emu, (void*)R_RIP, 0);
+                            emit_div0(emu, (void*)R_RIP, 1);
                         idiv64(emu, ED->q[0]);
-                        #ifdef TEST_INTERPRETER
-                        test->notest = 1;
-                        #endif
                         break;
                 }
             } else {
@@ -1996,14 +2036,14 @@ x64emurun:
                         break;
                     case 6:                 /* DIV Ed */
                         if(!ED->dword[0])
-                            emit_div0(emu, (void*)R_RIP, 0);
+                            emit_div0(emu, (void*)R_RIP, 1);
                         div32(emu, ED->dword[0]);
                         //emu->regs[_AX].dword[1] = 0;  // already put high regs to 0
                         //emu->regs[_DX].dword[1] = 0;
                         break;
                     case 7:                 /* IDIV Ed */
                         if(!ED->dword[0])
-                            emit_div0(emu, (void*)R_RIP, 0);
+                            emit_div0(emu, (void*)R_RIP, 1);
                         idiv32(emu, ED->dword[0]);
                         //emu->regs[_AX].dword[1] = 0;
                         //emu->regs[_DX].dword[1] = 0;
@@ -2021,19 +2061,17 @@ x64emurun:
             break;
         case 0xFA:                      /* CLI */
             // this is a privilege opcode
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            if(rex.is32bits && box64_ignoreint3)
+            {} else
+            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0xbad0);
             STEP;
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
-            #endif
             break;
         case 0xFB:                      /* STI */
             // this is a privilege opcode
-            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            if(rex.is32bits && box64_ignoreint3)
+            {} else
+            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0xbad0);
             STEP;
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
-            #endif
             break;
         case 0xFC:                      /* CLD */
             CLEAR_FLAG(F_DF);
@@ -2092,9 +2130,6 @@ x64emurun:
                     }
                     addr = tmp64u;
                     STEP2
-                    #ifdef TEST_INTERPRETER
-                    test->notest = 1;
-                    #endif
                     break;
                 case 3:                 /* CALL FAR Ed */
                     GETET(0);
@@ -2115,13 +2150,24 @@ x64emurun:
                             R_CS = (ED+1)->word[0];
                         }
                         STEP2;
-                        is32bits = (emu->segs[_CS]==0x23);
-                        #ifndef TEST_INTERPRETER
-                        if(is32bits)
-                            running32bits = 1;
-                        #else
-                        test->notest = 1;
-                        #endif
+                        if(is32bits!=(emu->segs[_CS]==0x23)) {
+                            is32bits = (emu->segs[_CS]==0x23);
+                            if(is32bits) {
+                                // Zero upper part of the 32bits regs
+                                R_RAX = R_EAX;
+                                R_RBX = R_EBX;
+                                R_RCX = R_ECX;
+                                R_RDX = R_EDX;
+                                R_RSP = R_ESP;
+                                R_RBP = R_EBP;
+                                R_RSI = R_ESI;
+                                R_RDI = R_EDI;
+                            }
+                            #ifndef TEST_INTERPRETER
+                            if(is32bits)
+                                running32bits = 1;
+                            #endif
+                        }
                     }
                     break;
                 case 4:                 /* JMP NEAR Ed */
@@ -2147,11 +2193,24 @@ x64emurun:
                             R_CS = (ED+1)->word[0];
                         }
                         STEP2;
-                        is32bits = (emu->segs[_CS]==0x23);
-                        #ifndef TEST_INTERPRETER
-                        if(is32bits)
-                            running32bits = 1;
-                        #endif
+                        if(is32bits!=(emu->segs[_CS]==0x23)) {
+                            is32bits = (emu->segs[_CS]==0x23);
+                            if(is32bits) {
+                                // Zero upper part of the 32bits regs
+                                R_RAX = R_EAX;
+                                R_RBX = R_EBX;
+                                R_RCX = R_ECX;
+                                R_RDX = R_EDX;
+                                R_RSP = R_ESP;
+                                R_RBP = R_EBP;
+                                R_RSI = R_ESI;
+                                R_RDI = R_EDI;
+                            }
+                            #ifndef TEST_INTERPRETER
+                            if(is32bits)
+                                running32bits = 1;
+                            #endif
+                        }
                     }
                     break;
                 case 6:                 /* Push Ed */
@@ -2176,10 +2235,11 @@ x64emurun:
         }
 #ifndef TEST_INTERPRETER
         // check the TRACE flag before going to next
-        if(ACCESS_FLAG(F_TF)) {
-            if(tf_next) {
+        if(ACCESS_FLAG(F_TF) || (tf_next<0)) {
+            if(tf_next>0) {
                 tf_next = 0;
             } else {
+                tf_next = 0;
                 R_RIP = addr;
                 emit_signal(emu, SIGTRAP, (void*)addr, 1);
                 if(emu->quit) goto fini;
@@ -2203,8 +2263,9 @@ if(emu->segs[_CS]!=0x33 && emu->segs[_CS]!=0x23) printf_log(LOG_NONE, "Warning,
 #ifndef TEST_INTERPRETER
     printf_log(LOG_DEBUG, "End of X86 run (%p), RIP=%p, Stack=%p, unimp=%d, emu->fork=%d, emu->uc_link=%p, emu->quit=%d\n", emu, (void*)R_RIP, (void*)R_RSP, unimp, emu->fork, emu->uc_link, emu->quit);
     if(unimp) {
-        emu->quit = 1;
+        //emu->quit = 1;
         UnimpOpcode(emu, is32bits);
+        emit_signal(emu, SIGILL, (void*)R_RIP, 0);
     }
     // fork handling
     if(emu->fork) {
@@ -2220,7 +2281,12 @@ if(emu->segs[_CS]!=0x33 && emu->segs[_CS]!=0x23) printf_log(LOG_NONE, "Warning,
     // setcontext handling
     else if(emu->quit && emu->uc_link) {
         emu->quit = 0;
-        my_setcontext(emu, emu->uc_link);
+        #ifdef BOX32
+        if(box64_is32bits)
+            my32_setcontext(emu, emu->uc_link);
+        else
+        #endif
+            my_setcontext(emu, emu->uc_link);
         addr = R_RIP;
         goto x64emurun;
     }
