@@ -94,8 +94,6 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                     if(R_RCX) {
                         #ifndef TEST_INTERPRETER
                         emit_signal(emu, SIGILL, (void*)R_RIP, 0);
-                        #else
-                        test->notest = 1;
                         #endif
                     } else {
                         R_RAX = 0b111;   // x87 & SSE & AVX for now
@@ -119,9 +117,6 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                     R_RAX = tmp64u & 0xffffffff;
                     R_RDX = tmp64u >> 32;
                     R_RCX = helper_getcpu(emu);
-                    #ifdef TEST_INTERPRETER
-                    test->notest = 1;
-                    #endif
                     break;
                 default:
                     return 0;
@@ -156,8 +151,6 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             #ifndef TEST_INTERPRETER
             R_RIP = addr;
             x64Syscall(emu);
-            #else
-            test->notest = 1;
             #endif
             break;
         case 0x06:                      /* CLTS */
@@ -172,16 +165,12 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             // this is a privilege opcode...
             #ifndef TEST_INTERPRETER
             emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
-            #else
-            test->notest = 1;
             #endif
             break;
 
         case 0x0B:                      /* UD2 */
             #ifndef TEST_INTERPRETER
             emit_signal(emu, SIGILL, (void*)R_RIP, 0);
-            #else
-            test->notest = 1;
             #endif
             break;
 
@@ -189,18 +178,22 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             nextop = F8;
             GETED(0);
             switch((nextop>>3)&7) {
+                case 0: //PREFETCH?
+                    __builtin_prefetch((void*)ED, 0, 0);
+                    break;
                 case 1: //PREFETCHW
                     __builtin_prefetch((void*)ED, 1, 0);
                     break;
-                default:    //???
-                    return 0;
+                case 2: //PREFETCHWT1
+                    __builtin_prefetch((void*)ED, 1, 0);
+                    break;
+                default:    //NOP
+                    break;
             }
             break;
         case 0x0E:                      /* FEMMS */
             #ifndef TEST_INTERPRETER
             emit_signal(emu, SIGILL, (void*)R_RIP, 0);
-            #else
-            test->notest = 1;
             #endif
             break;
 
@@ -385,15 +378,37 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             }
             CLEAR_FLAG(F_OF); CLEAR_FLAG(F_AF); CLEAR_FLAG(F_SF);
             break;
-
+        case 0x30:                      /* WRMSR */
+            // this is a privilege opcode...
+            #ifndef TEST_INTERPRETER
+            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0);
+            #endif
+            break;
         case 0x31:                   /* RDTSC */
             tmp64u = ReadTSC(emu);
             if(box64_rdtsc_shift)
                 tmp64u<<=box64_rdtsc_shift;
             R_RDX = tmp64u>>32;
             R_RAX = tmp64u&0xFFFFFFFF;
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
+            break;
+        case 0x32:                   /* RDMSR */
+            // priviledge instruction
+            #ifndef TEST_INTERPRETER
+            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0xbad0);
+            STEP;
+            #endif
+            break;
+
+        case 0x34:                  /* SYSENTER */
+            #ifndef TEST_INTERPRETER
+            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0xbad0);
+            STEP;
+            #endif
+            break;
+        case 0x35:                  /* SYSEXIT */
+            #ifndef TEST_INTERPRETER
+            emit_signal(emu, SIGSEGV, (void*)R_RIP, 0xbad0);
+            STEP;
             #endif
             break;
 
@@ -710,9 +725,6 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                 else
                     GX->f[i] = 1.0f/sqrtf(EX->f[i]);
             }
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
-            #endif
             break;
         case 0x53:                      /* RCPPS Gx, Ex */
             nextop = F8;
@@ -720,9 +732,6 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             GETGX;
             for(int i=0; i<4; ++i)
                 GX->f[i] = 1.0f/EX->f[i];
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
-            #endif
             break;
         case 0x54:                      /* ANDPS Gx, Ex */
             nextop = F8;
@@ -1089,9 +1098,6 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
             break;
         GOCOND(0x80
             , tmp32s = F32S; CHECK_FLAGS(emu);
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
-            #endif
             , addr += tmp32s;
             ,,
         )                               /* 0x80 -> 0x8F Jxx */ //STEP3
@@ -1118,28 +1124,25 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         case 0xA2:                      /* CPUID */
             tmp32u = R_EAX;
             my_cpuid(emu, tmp32u);
-            #ifdef TEST_INTERPRETER
-            test->notest = 1;
-            #endif
             break;
         case 0xA3:                      /* BT Ed,Gd */
             CHECK_FLAGS(emu);
             nextop = F8;
             GETED(0);
             GETGD;
-            tmp32s = GD->sdword[0];
-            tmp8u=tmp32s&(rex.w?63:31);
-            tmp32s >>= (rex.w?6:5);
+            tmp64s = rex.w?GD->sq[0]:GD->sdword[0];
+            tmp8u=tmp64s&(rex.w?63:31);
+            tmp64s >>= (rex.w?6:5);
             if(!MODREG)
             {
                 #ifdef TEST_INTERPRETER
-                test->memaddr=((test->memaddr)+(tmp32s<<(rex.w?3:2)));
+                test->memaddr=((test->memaddr)+(tmp64s<<(rex.w?3:2)));
                 if(rex.w)
                     *(uint64_t*)test->mem = *(uint64_t*)test->memaddr;
                 else
                     *(uint32_t*)test->mem = *(uint32_t*)test->memaddr;
                 #else
-                ED=(reg64_t*)(((uintptr_t)(ED))+(tmp32s<<(rex.w?3:2)));
+                ED=(reg64_t*)(((uintptr_t)(ED))+(tmp64s<<(rex.w?3:2)));
                 #endif
             }
             if(rex.w) {
@@ -1270,18 +1273,18 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                     #ifdef TEST_INTERPRETER
                     emu->sw.f.F87_TOP = emu->top&7;
                     #else
-                    if(rex.w)
-                        fpu_fxsave64(emu, ED);
-                    else
+                    if(rex.is32bits)
                         fpu_fxsave32(emu, ED);
+                    else
+                        fpu_fxsave64(emu, ED);
                     #endif
                     break;
                 case 1:                 /* FXRSTOR Ed */
                     _GETED(0);
-                    if(rex.w)
-                        fpu_fxrstor64(emu, ED);
-                    else
+                    if(rex.is32bits)
                         fpu_fxrstor32(emu, ED);
+                    else
+                        fpu_fxrstor64(emu, ED);
                     break;
                 case 2:                 /* LDMXCSR Md */
                     GETED(0);
@@ -1300,12 +1303,12 @@ uintptr_t Run0F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                     #ifdef TEST_INTERPRETER
                     emu->sw.f.F87_TOP = emu->top&7;
                     #else
-                    fpu_xsave(emu, ED, rex.is32bits);
+                    fpu_xsave(emu, ED, rex.w?0:1);
                     #endif
                     break;
                 case 5:                 /* XRSTOR Ed */
                     _GETED(0);
-                    fpu_xrstor(emu, ED, rex.is32bits);
+                    fpu_xrstor(emu, ED, rex.w?0:1);
                     break;
                 case 7:                 /* CLFLUSH Ed */
                     _GETED(0);
