@@ -20,6 +20,7 @@
 #include "x87emu_private.h"
 #include "box64context.h"
 #include "bridge.h"
+#include "signals.h"
 
 #include "modrm.h"
 
@@ -93,12 +94,12 @@ uintptr_t RunF20F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         _GETEX(0);
         GETGD;
         if(rex.w)
-            if(isnan(EX->d[0]) || isinf(EX->d[0]) || EX->d[0]>0x7fffffffffffffffLL)
+            if(isnan(EX->d[0]) || isinf(EX->d[0]) || EX->d[0]>=(double)0x7fffffffffffffffLL)
                 GD->q[0] = 0x8000000000000000LL;
             else
                 GD->sq[0] = EX->d[0];
         else {
-            if(isnan(EX->d[0]) || isinf(EX->d[0]) || EX->d[0]>0x7fffffff)
+            if(isnan(EX->d[0]) || isinf(EX->d[0]) || EX->d[0]>(double)0x7fffffff)
                 GD->dword[0] = 0x80000000;
             else
                 GD->sdword[0] = EX->d[0];
@@ -110,7 +111,7 @@ uintptr_t RunF20F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         _GETEX(0);
         GETGD;
         if(rex.w) {
-            if(isnan(EX->d[0]) || isinf(EX->d[0]) || EX->d[0]>0x7fffffffffffffffLL)
+            if(isnan(EX->d[0]) || isinf(EX->d[0]) || EX->d[0]>=(double)0x7fffffffffffffffLL)
                 GD->q[0] = 0x8000000000000000LL;
             else
                 switch(emu->mxcsr.f.MXCSR_RC) {
@@ -132,7 +133,7 @@ uintptr_t RunF20F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
                         break;
                 }
         } else {
-            if(isnan(EX->d[0]) || isinf(EX->d[0]) || EX->d[0]>0x7fffffff)
+            if(isnan(EX->d[0]) || isinf(EX->d[0]) || EX->d[0]>(double)0x7fffffff)
                 GD->dword[0] = 0x80000000;
             else
                 switch(emu->mxcsr.f.MXCSR_RC) {
@@ -214,20 +215,26 @@ uintptr_t RunF20F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         if((isinf(GX->d[0]) && isinf(EX->d[0]) && (EX->q[0]&0x8000000000000000LL)!=(GX->q[0]&0x8000000000000000LL)))
             GX->d[0] = -NAN;
         else
-        #endif
-        GX->d[0] += EX->d[0];
+#endif
+        {
+            NAN_PROPAGATION(GX->d[0], EX->d[0], break);
+            GX->d[0] += EX->d[0];
+        }
         break;
     case 0x59:  /* MULSD Gx, Ex */
         nextop = F8;
         _GETEX(0);
         GETGX;
         #ifndef NOALIGN
-            // mul generate a -NAN only if doing (+/-)inf * (+/-)0
-            if((isinf(GX->d[0]) && EX->d[0]==0.0) || (isinf(EX->d[0]) && GX->d[0]==0.0))
-                GX->d[0] = -NAN;
-            else
-        #endif
-        GX->d[0] *= EX->d[0];
+        // mul generate a -NAN only if doing (+/-)inf * (+/-)0
+        if ((isinf(GX->d[0]) && EX->d[0] == 0.0) || (isinf(EX->d[0]) && GX->d[0] == 0.0))
+            GX->d[0] = -NAN;
+        else
+#endif
+        {
+            NAN_PROPAGATION(GX->d[0], EX->d[0], break);
+            GX->d[0] *= EX->d[0];
+        }
         break;
     case 0x5A:  /* CVTSD2SS Gx, Ex */
         nextop = F8;
@@ -241,12 +248,15 @@ uintptr_t RunF20F(x64emu_t *emu, rex_t rex, uintptr_t addr, int *step)
         _GETEX(0);
         GETGX;
         #ifndef NOALIGN
-            // sub generate a -NAN only if doing inf - inf
-            if((isinf(GX->d[0]) && isinf(EX->d[0]) && (EX->q[0]&0x8000000000000000LL)==(GX->q[0]&0x8000000000000000LL)))
-                GX->d[0] = -NAN;
-            else
-        #endif
-        GX->d[0] -= EX->d[0];
+        // sub generate a -NAN only if doing inf - inf
+        if ((isinf(GX->d[0]) && isinf(EX->d[0]) && (EX->q[0] & 0x8000000000000000LL) == (GX->q[0] & 0x8000000000000000LL)))
+            GX->d[0] = -NAN;
+        else
+#endif
+        {
+            NAN_PROPAGATION(GX->d[0], EX->d[0], break);
+            GX->d[0] -= EX->d[0];
+        }
         break;
     case 0x5D:  /* MINSD Gx, Ex */
         nextop = F8;
