@@ -104,12 +104,12 @@ uintptr_t RunF30F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         GETEX(0);
         GETGD;
         if (rex.w) {
-            if(isnanf(EX->f[0]) || isinff(EX->f[0]) || EX->f[0]>(float)0x7fffffffffffffffLL)
+            if(isnanf(EX->f[0]) || isinff(EX->f[0]) || EX->f[0]>=(float)0x7fffffffffffffffLL)
                 GD->q[0] = 0x8000000000000000LL;
             else
                 GD->sq[0] = EX->f[0];
         } else {
-            if(isnanf(EX->f[0]) || isinff(EX->f[0]) || EX->f[0]>0x7fffffff)
+            if(isnanf(EX->f[0]) || isinff(EX->f[0]) || EX->f[0]>=(float)0x7fffffff)
                 GD->dword[0] = 0x80000000;
             else
                 GD->sdword[0] = EX->f[0];
@@ -121,7 +121,7 @@ uintptr_t RunF30F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         GETEX(0);
         GETGD;
         if(rex.w) {
-            if(isnanf(EX->f[0]) || isinff(EX->f[0]) || EX->f[0]>(float)0x7fffffffffffffffLL)
+            if(isnanf(EX->f[0]) || isinff(EX->f[0]) || EX->f[0]>=(float)0x7fffffffffffffffLL)
                 GD->q[0] = 0x8000000000000000LL;
             else
                 switch(emu->mxcsr.f.MXCSR_RC) {
@@ -211,18 +211,21 @@ uintptr_t RunF30F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         nextop = F8;
         GETEX(0);
         GETGX;
+        NAN_PROPAGATION(GX->f[0], EX->f[0], break);
         GX->f[0] = sqrtf(EX->f[0]);
         break;
     case 0x52:  /* RSQRTSS Gx, Ex */
         nextop = F8;
         GETEX(0);
         GETGX;
+        NAN_PROPAGATION(GX->f[0], EX->f[0], break);
         GX->f[0] = 1.0f/sqrtf(EX->f[0]);
         break;
     case 0x53:  /* RCPSS Gx, Ex */
         nextop = F8;
         GETEX(0);
         GETGX;
+        NAN_PROPAGATION(GX->f[0], EX->f[0], break);
         GX->f[0] = 1.0f/EX->f[0];
         break;
 
@@ -230,12 +233,14 @@ uintptr_t RunF30F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         nextop = F8;
         GETEX(0);
         GETGX;
+        NAN_PROPAGATION(GX->f[0], EX->f[0], break);
         GX->f[0] += EX->f[0];
         break;
     case 0x59:  /* MULSS Gx, Ex */
         nextop = F8;
         GETEX(0);
         GETGX;
+        NAN_PROPAGATION(GX->f[0], EX->f[0], break);
         GX->f[0] *= EX->f[0];
         break;
     case 0x5A:  /* CVTSS2SD Gx, Ex */
@@ -264,6 +269,7 @@ uintptr_t RunF30F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         nextop = F8;
         GETEX(0);
         GETGX;
+        NAN_PROPAGATION(GX->f[0], EX->f[0], break);
         GX->f[0] -= EX->f[0];
         break;
     case 0x5D:  /* MINSS Gx, Ex */
@@ -277,6 +283,7 @@ uintptr_t RunF30F(x64emu_t *emu, rex_t rex, uintptr_t addr)
         nextop = F8;
         GETEX(0);
         GETGX;
+        NAN_PROPAGATION(GX->f[0], EX->f[0], break);
         GX->f[0] /= EX->f[0];
         break;
     case 0x5F:  /* MAXSS Gx, Ex */
