@@ -101,6 +101,7 @@ static const scwrap_t syscallwrap[] = {
     [16] = {__NR_ioctl, 3},
     [17] = {__NR_pread64, 4},
     [18] = {__NR_pwrite64, 4},
+    [19] = {__NR_readv, 3},
     [20] = {__NR_writev, 3},
     #ifdef __NR_access
     [21] = {__NR_access, 2},
@@ -183,6 +184,9 @@ static const scwrap_t syscallwrap[] = {
     [118] = {__NR_getresuid, 3},
     [120] = {__NR_getresgid, 3},
     [121] = {__NR_getpgid, 1},
+    [122] = {__NR_setfsuid, 1},
+    [123] = {__NR_setfsgid, 1},
+    [124] = {__NR_getsid, 1},
     [125] = {__NR_capget, 2},
     [126] = {__NR_capset, 2},
     [127] = {__NR_rt_sigpending, 2},
@@ -203,7 +207,11 @@ static const scwrap_t syscallwrap[] = {
     [160] = {__NR_setrlimit, 2},
     #endif
     [161] = {__NR_chroot, 1},
+    [165] = {__NR_mount, 5},
     [186] = {__NR_gettid, 0 },    //0xBA
+    [194] = {__NR_listxattr, 3},
+    [195] = {__NR_llistxattr, 3},
+    [196] = {__NR_flistxattr, 3},
     [200] = {__NR_tkill, 2 },
     #ifdef __NR_time
     [201] = {__NR_time, 1},
@@ -260,6 +268,7 @@ static const scwrap_t syscallwrap[] = {
     [272] = {__NR_unshare, 1},
     [273] = {__NR_set_robust_list, 2},
     [274] = {__NR_get_robust_list, 3},
+    [280] = {__NR_utimensat, 4},
     #ifdef NOALIGN
     [281] = {__NR_epoll_pwait, 6},
     #endif
@@ -297,6 +306,15 @@ static const scwrap_t syscallwrap[] = {
     #ifdef __NR_fchmodat4
     [434] = {__NR_fchmodat4, 4},
     #endif
+    #ifdef __NR_landlock_create_ruleset	
+    [444] = {__NR_landlock_create_ruleset, 3},
+    #endif
+    #ifdef __NR_landlock_add_rule
+    [445] = {__NR_landlock_add_rule, 4},
+    #endif
+    #ifdef __NR_landlock_restrict_self
+    [446] = {__NR_landlock_restrict_self, 2},
+    #endif
     //[449] = {__NR_futex_waitv, 5},
 };
 
@@ -570,6 +588,7 @@ void EXPORT x64Syscall(x64emu_t *emu)
                 {
                     void* stack_base = (void*)R_RSI;
                     int stack_size = 0;
+                    uintptr_t sp = R_RSI;
                     if(!R_RSI) {
                         // allocate a new stack...
                         int currstack = 0;
@@ -578,18 +597,19 @@ void EXPORT x64Syscall(x64emu_t *emu)
                         stack_size = (currstack && emu->size_stack)?emu->size_stack:(1024*1024);
                         stack_base = mmap(NULL, stack_size, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_GROWSDOWN, -1, 0);
                         // copy value from old stack to new stack
-                        if(currstack)
+                        if(currstack) {
                             memcpy(stack_base, emu->init_stack, stack_size);
-                        else {
+                            sp = (uintptr_t)emu->init_stack + R_RSP - (uintptr_t)stack_base;
+                        } else {
                             int size_to_copy = (uintptr_t)emu->init_stack + emu->size_stack - (R_RSP);
                             memcpy(stack_base+stack_size-size_to_copy, (void*)R_RSP, size_to_copy);
+                            sp = (uintptr_t)stack_base+stack_size-size_to_copy;
                         }
                     }
                     x64emu_t * newemu = NewX64Emu(emu->context, R_RIP, (uintptr_t)stack_base, stack_size, (R_RSI)?0:1);
                     SetupX64Emu(newemu, emu);
-                    //CloneEmu(newemu, emu);
-                    Push64(newemu, 0);
-                    PushExit(newemu);
+                    CloneEmu(newemu, emu);
+                    newemu->regs[_SP].q[0] = sp;  // setup new stack pointer
                     void* mystack = NULL;
                     if(my_context->stack_clone_used) {
                         mystack = box_malloc(1024*1024);  // stack for own process... memory leak, but no practical way to remove it
@@ -716,6 +736,10 @@ void EXPORT x64Syscall(x64emu_t *emu)
                 S_RAX = -errno;
             break;
         #endif
+        case 175: // sys_init_module
+            // huh?
+            S_RAX = -EPERM;
+            break;
         #ifndef __NR_time
         case 201: // sys_time
             R_RAX = (uintptr_t)time((void*)R_RDI);
@@ -776,7 +800,7 @@ void EXPORT x64Syscall(x64emu_t *emu)
                 sigset_t * set = (sigset_t *)R_RSI;
                 if(sigismember(set, SIGSEGV)) {
                     sigdelset(set, SIGSEGV);
-                    printf_log(LOG_INFO, "Warning, signalfd on SIGSEGV unsuported\n");
+                    printf_log(LOG_INFO, "Warning, signalfd on SIGSEGV unsupported\n");
                 }
                 S_RAX = signalfd(S_EDI, set, 0);
                 if(S_RAX==-1)
@@ -923,6 +947,7 @@ long EXPORT my_syscall(x64emu_t *emu)
             {
                 void* stack_base = (void*)R_RDX;
                 int stack_size = 0;
+                uintptr_t sp = R_RDX;
                 if(!stack_base) {
                     // allocate a new stack...
                     int currstack = 0;
@@ -931,18 +956,19 @@ long EXPORT my_syscall(x64emu_t *emu)
                     stack_size = (currstack)?emu->size_stack:(1024*1024);
                     stack_base = mmap(NULL, stack_size, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANONYMOUS|MAP_GROWSDOWN, -1, 0);
                     // copy value from old stack to new stack
-                    if(currstack)
+                    if(currstack) {
                         memcpy(stack_base, emu->init_stack, stack_size);
-                    else {
+                        sp = (uintptr_t)emu->init_stack + R_RSP - (uintptr_t)stack_base;
+                    } else {
                         int size_to_copy = (uintptr_t)emu->init_stack + emu->size_stack - (R_RSP);
                         memcpy(stack_base+stack_size-size_to_copy, (void*)R_RSP, size_to_copy);
+                        sp = (uintptr_t)stack_base+stack_size-size_to_copy;
                     }
                 }
                 x64emu_t * newemu = NewX64Emu(emu->context, R_RIP, (uintptr_t)stack_base, stack_size, (R_RDX)?0:1);
                 SetupX64Emu(newemu, emu);
-                //CloneEmu(newemu, emu);
-                Push64(newemu, 0);
-                PushExit(newemu);
+                CloneEmu(newemu, emu);
+                newemu->regs[_SP].q[0] = sp;  // setup new stack pointer
                 void* mystack = NULL;
                 if(my_context->stack_clone_used) {
                     mystack = box_malloc(1024*1024);  // stack for own process... memory leak, but no practical way to remove it
@@ -1023,6 +1049,10 @@ long EXPORT my_syscall(x64emu_t *emu)
         case 160:
             return setrlimit(S_ESI, (void*)R_RDX);
         #endif
+        case 175: // sys_init_module
+            // huh?
+            errno = -EPERM;
+            return -1;
         #ifndef __NR_time
         case 201: // sys_time
             return (intptr_t)time((void*)R_RSI);
@@ -1060,7 +1090,7 @@ long EXPORT my_syscall(x64emu_t *emu)
                 sigset_t * set = (sigset_t *)R_RDX;
                 if(sigismember(set, SIGSEGV)) {
                     sigdelset(set, SIGSEGV);
-                    printf_log(LOG_INFO, "Warning, signalfd on SIGSEGV unsuported\n");
+                    printf_log(LOG_INFO, "Warning, signalfd on SIGSEGV unsupported\n");
                 }
                 return signalfd(S_ESI, set, 0);
             }
