@@ -9,11 +9,13 @@
 #include "debug.h"
 typedef struct x64emu_s x64emu_t;
 
-#define PI		3.14159265358979323846
-#define L2E		1.4426950408889634
-#define L2T		3.3219280948873623
-#define LN2		0.69314718055994531
-#define LG2		0.3010299956639812
+#define PI      3.14159265358979323846
+#define L2E     1.4426950408889634
+#define L2T     3.3219280948873623
+#define LN2     0.69314718055994531
+#define LG2     0.3010299956639812
+
+#define TAGS_EMPTY 0b1111111111111111
 
 #define ST0 emu->x87[emu->top]
 #define ST1 emu->x87[(emu->top+1)&7]
@@ -32,7 +34,8 @@ static inline void fpu_do_push(x64emu_t* emu)
     }*/
     if(emu->fpu_stack<8)
         ++emu->fpu_stack; 
-    emu->p_regs[newtop].tag = 0;    // full
+    emu->fpu_tags<<=2;  // st0 full
+    emu->fpu_tags &= TAGS_EMPTY;
     emu->top = newtop;
 }
 
@@ -47,8 +50,16 @@ static inline void fpu_do_pop(x64emu_t* emu)
     if(emu->fpu_stack>0)
         --emu->fpu_stack;
     
-    emu->p_regs[curtop].tag = 0b11;    // empty
+    emu->fpu_tags>>=2;
+    emu->fpu_tags |= 0b1100000000000000;    // top empty
     emu->top = (emu->top+1)&7;
+    // check tags
+    /*while((emu->fpu_tags&0b11) && emu->fpu_stack) {
+        --emu->fpu_stack;
+        emu->top = (emu->top+1)&7;
+        emu->fpu_tags>>=2;
+        emu->fpu_tags |= 0b1100000000000000;    // top empty
+    }*/
 }
 
 void fpu_do_free(x64emu_t* emu, int i);
@@ -62,6 +73,10 @@ static inline void fpu_fcom(x64emu_t* emu, double b)
         emu->sw.f.F87_C0 = 1;
         emu->sw.f.F87_C2 = 1;
         emu->sw.f.F87_C3 = 1;
+    } else if ((b==-INFINITY) || (ST0.d==INFINITY)) {
+        emu->sw.f.F87_C0 = 0;
+        emu->sw.f.F87_C2 = 0;
+        emu->sw.f.F87_C3 = 0;
     } else if (isgreater(ST0.d, b)) {
         emu->sw.f.F87_C0 = 0;
         emu->sw.f.F87_C2 = 0;
@@ -89,6 +104,10 @@ static inline void fpu_fcomi(x64emu_t* emu, double b)
         SET_FLAG(F_CF);
         SET_FLAG(F_PF);
         SET_FLAG(F_ZF);
+    } else if ((b==-INFINITY) || (ST0.d==INFINITY)) {
+        CLEAR_FLAG(F_CF);
+        CLEAR_FLAG(F_PF);
+        CLEAR_FLAG(F_ZF);
     } else if (isgreater(ST0.d, b)) {
         CLEAR_FLAG(F_CF);
         CLEAR_FLAG(F_PF);
@@ -128,33 +147,45 @@ static inline double fpu_round(x64emu_t* emu, double d) {
 
 static inline void fpu_fxam(x64emu_t* emu) {
     emu->sw.f.F87_C1 = (ST0.ud[1]&0x80000000)?1:0;
-    if(!emu->fpu_stack) {
+    if((emu->fpu_stack<=0) || (emu->fpu_tags&0b11)) {
+        //Empty
         emu->sw.f.F87_C3 = 1;
         emu->sw.f.F87_C2 = 0;
         emu->sw.f.F87_C0 = 1;
         return;
     }
-    if(isinf(ST0.d)) 
-    {  // TODO: Unsupported and denormal not analysed...
+    if(isinf(ST0.d))
+    {
+        //Infinity
         emu->sw.f.F87_C3 = 0;
         emu->sw.f.F87_C2 = 1;
         emu->sw.f.F87_C0 = 1;
         return;
     }
     if(isnan(ST0.d))
-    {  // TODO: Unsupported and denormal not analysed...
+    {
+        //NaN
         emu->sw.f.F87_C3 = 0;
         emu->sw.f.F87_C2 = 0;
         emu->sw.f.F87_C0 = 1;
         return;
     }
-    if(ST0.d==0.0)
+    if((ST0.ud[0]|(ST0.ud[1]&0x7fffffff))==0)
     {
+        //Zero
         emu->sw.f.F87_C3 = 1;
         emu->sw.f.F87_C2 = 0;
         emu->sw.f.F87_C0 = 0;
         return;
     }
+    if((ST0.ud[1]&0x7FF00000)==0)
+    {
+        // denormals
+        emu->sw.f.F87_C3 = 1;
+        emu->sw.f.F87_C2 = 1;
+        emu->sw.f.F87_C0 = 0;
+        return;
+    }
     // normal...
     emu->sw.f.F87_C3 = 0;
     emu->sw.f.F87_C2 = 1;
@@ -193,5 +224,11 @@ void fpu_fxsave32(x64emu_t* emu, void* ed);
 void fpu_fxrstor32(x64emu_t* emu, void* ed);
 void fpu_fxsave64(x64emu_t* emu, void* ed);
 void fpu_fxrstor64(x64emu_t* emu, void* ed);
+void fpu_xsave(x64emu_t* emu, void* ed, int is32bits);
+void fpu_xsave_mask(x64emu_t* emu, void* ed, int is32bits, uint64_t mask);
+void fpu_xrstor(x64emu_t* emu, void* ed, int is32bits);
+
+uint32_t cvtf16_32(uint16_t v);
+uint16_t cvtf32_16(uint32_t v, uint8_t rounding);
 
 #endif //__X87RUN_PRIVATE_H_
