@@ -16,6 +16,8 @@ extern int box64_ignoreint3;
 extern int box64_rdtsc;
 extern int box64_rdtsc_1ghz;
 extern uint8_t box64_rdtsc_shift;
+extern int box64_is32bits;
+extern int box64_x11sync;
 #ifdef DYNAREC
 extern int box64_dynarec_dump;
 extern int box64_dynarec_trace;
@@ -25,6 +27,8 @@ extern uintptr_t box64_dynarec_test_start, box64_dynarec_test_end;
 extern int box64_dynarec_bigblock;
 extern int box64_dynarec_forward;
 extern int box64_dynarec_strongmem;
+extern int box64_dynarec_weakbarrier;
+extern int box64_dynarec_pause;
 extern int box64_dynarec_fastnan;
 extern int box64_dynarec_fastround;
 extern int box64_dynarec_x87double;
@@ -36,6 +40,7 @@ extern int box64_dynarec_tbb;
 extern int box64_dynarec_wait;
 extern int box64_dynarec_missing;
 extern int box64_dynarec_aligned_atomics;
+extern int box64_dynarec_nativeflags;
 #ifdef ARM64
 extern int arm64_asimd;
 extern int arm64_aes;
@@ -55,6 +60,8 @@ extern int rv64_zbb;
 extern int rv64_zbc;
 extern int rv64_zbs;
 extern int rv64_vector;
+extern int rv64_xtheadvector; // rvv 1.0 or xtheadvector
+extern int rv64_vlen;
 extern int rv64_xtheadba;
 extern int rv64_xtheadbb;
 extern int rv64_xtheadbs;
@@ -97,6 +104,7 @@ extern int box64_dummy_crashhandler;
 extern int box64_sse_flushto0;
 extern int box64_x87_no80bits;
 extern int box64_sync_rounding;
+extern int box64_shaext;
 extern int box64_sse42;
 extern int box64_avx;
 extern int box64_avx2;
@@ -104,7 +112,9 @@ extern int allow_missing_libs;
 extern int box64_mapclean;
 extern int box64_prefer_wrapped;
 extern int box64_prefer_emulated;
+extern int box64_wrap_egl;
 extern int box64_steam;
+extern int box64_steamcmd;
 extern int box64_wine;
 extern int box64_musl;
 extern int box64_nopulse;   // disabling the use of wrapped pulseaudio
@@ -118,7 +128,7 @@ extern int box64_x11threads;
 extern int box64_x11glx;
 extern char* box64_libGL;
 extern uintptr_t fmod_smc_start, fmod_smc_end; // to handle libfmod (from Unreal) SMC (self modifying code)
-extern uint32_t default_gs;
+extern uint32_t default_gs, default_fs;
 extern int jit_gdb; // launch gdb when a segfault is trapped
 extern int box64_tcmalloc_minimal;  // when using tcmalloc_minimal
 #define LOG_NONE 0
@@ -171,4 +181,30 @@ extern char* box_strdup(const char* s);
 extern char* box_realpath(const char* path, char* ret);
 #endif
 
+//use actual_XXXX for internal memory that should be in 32bits space when box32 is active
+//use box_XXX for internal memory that doesn't need anything special
+#ifdef BOX32
+int isCustomAddr(void* p);
+void* box32_calloc(size_t n, size_t s);
+void* box32_malloc(size_t s);
+void* box32_realloc(void* p, size_t s);
+void box32_free(void* p);
+void* box32_memalign(size_t align, size_t s);
+size_t box32_malloc_usable_size(void* p);
+
+#define actual_calloc(A, B)             (box64_is32bits?box32_calloc(A, B):box_calloc(A, B))
+#define actual_malloc(A)                (box64_is32bits?box32_malloc(A):box_malloc(A))
+#define actual_realloc(A, B)            (box64_is32bits?box32_realloc(A, B):box_realloc(A, B))
+#define actual_free(A)                  (box64_is32bits?box32_free(A):box_free(A))
+#define actual_memalign(A, B)           (box64_is32bits?box32_memalign(A, B):box_memalign(A, B))
+#define actual_malloc_usable_size(A)    (box64_is32bits?box32_malloc_usable_size(A):box_malloc_usable_size(A))
+#else
+#define actual_calloc(A, B)             box_calloc(A, B)
+#define actual_malloc(A)                box_malloc(A)
+#define actual_realloc(A, B)            box_realloc(A, B)
+#define actual_free(A)                  box_free(A)
+#define actual_memalign(A, B)           box_memalign(A, B)
+#define actual_malloc_usable_size(A)    box_malloc_usable_size(A)
+#endif
+
 #endif //__DEBUG_H_
