@@ -182,13 +182,64 @@ SUPER()
 #undef GO2
 #undef GO
 
+#ifdef BOX32
+int isCustomAddr(void* p);
+#define SPACE32 (void*)0x100000000LL
+void* box32_calloc(size_t n, size_t s)
+{
+    void* ret = box_calloc(n, s);
+    if(ret<SPACE32) return ret;
+    box_free(ret);
+    return customCalloc32(n, s);
+}
+void* box32_malloc(size_t s)
+{
+    void* ret = box_malloc(s);
+    if(ret<SPACE32) return ret;
+    box_free(ret);
+    return customMalloc32(s);
+}
+void* box32_realloc(void* p, size_t s)
+{
+    if(isCustomAddr(p))
+        return customRealloc32(p, s);
+    void* ret = box_realloc(p, s);
+    if(ret<SPACE32) return ret;
+    void* newret = customMalloc32(s);
+    memcpy(newret, ret, s);
+    box_free(ret);
+    return newret;
+}
+void box32_free(void* p)
+{
+    if(isCustomAddr(p))
+        customFree32(p);
+    else
+        box_free(p);
+}
+void* box32_memalign(size_t align, size_t s)
+{
+    void* ret = box_memalign(align, s);
+    if(ret<SPACE32) return ret;
+    box_free(ret);
+    return customMemAligned32(align, s);
+}
+size_t box32_malloc_usable_size(void* p)
+{
+    if(isCustomAddr(p))
+        return customGetUsableSize(p);
+    else
+        return box_malloc_usable_size(p);
+}
+#endif
+
 // redefining all libc memory allocation routines
 EXPORT void* malloc(size_t l)
 {
     if(malloc_hack_2 && ALLOC && real_malloc) {
         return (void*)RunFunctionFmt(real_malloc, "L", l);
     }
-    return box_calloc(1, l);
+    return actual_calloc(1, l);
 }
 
 EXPORT void free(void* p)
@@ -202,7 +253,7 @@ EXPORT void free(void* p)
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void* calloc(size_t n, size_t s)
@@ -210,7 +261,7 @@ EXPORT void* calloc(size_t n, size_t s)
     if(malloc_hack_2 && ALLOC && real_calloc) {
         return (void*)RunFunctionFmt(real_calloc, "LL", n,s);
     }
-    return box_calloc(n, s);
+    return actual_calloc(n, s);
 }
 
 EXPORT void* realloc(void* p, size_t s)
@@ -222,7 +273,7 @@ EXPORT void* realloc(void* p, size_t s)
                 ret = (void*)RunFunctionFmt(real_realloc, "pL", p, s);
             } else {
                 // found! Will realloc using regular malloc then copy from old address as much as possible, but need to check size first
-                ret = box_malloc(s);
+                ret = actual_malloc(s);
                 printf_log(LOG_DEBUG, "Malloc_Hack_2: hacking realloc(%p, %zu)", p, s);
                 while(s && !(getProtection((uintptr_t)p+s)&PROT_READ)) {if(s>box64_pagesize) s-=box64_pagesize; else s=0;}
                 memcpy(ret, p, s);
@@ -233,15 +284,7 @@ EXPORT void* realloc(void* p, size_t s)
             }
             return ret;
         }
-    return box_realloc(p, s);
-}
-
-EXPORT void* aligned_alloc(size_t align, size_t size)
-{
-    if(malloc_hack_2 && ALLOC && real_aligned_alloc) {
-        return (void*)RunFunctionFmt(real_aligned_alloc, "LL", align, size);
-    }
-    return box_memalign(align, size);
+    return actual_realloc(p, s);
 }
 
 EXPORT void* memalign(size_t align, size_t size)
@@ -249,7 +292,20 @@ EXPORT void* memalign(size_t align, size_t size)
     if(malloc_hack_2 && ALLOC && real_aligned_alloc) {
         return (void*)RunFunctionFmt(real_aligned_alloc, "LL", align, size);
     }
-    return box_memalign(align, size);
+    if(box64_is32bits && align==4)
+        align = sizeof(void*);
+    return actual_memalign(align, size);
+}
+
+EXPORT void* aligned_alloc(size_t align, size_t size)
+{
+    if(malloc_hack_2 && ALLOC && real_aligned_alloc) {
+        return (void*)RunFunctionFmt(real_aligned_alloc, "LL", align, size);
+    }
+    if(box64_is32bits && align==4) {
+        return memalign(align, size);
+    }
+    return actual_memalign(align, size);
 }
 
 EXPORT int posix_memalign(void** p, size_t align, size_t size)
@@ -257,9 +313,11 @@ EXPORT int posix_memalign(void** p, size_t align, size_t size)
     if(malloc_hack_2 && ALLOC && real_posix_memalign) {
         return RunFunctionFmt(real_posix_memalign, "pLL", p, align, size);
     }
-    if(align%sizeof(void*) || pot(align)!=align)
+    if(box64_is32bits && align==4)
+        align = sizeof(void*);
+    if((align%sizeof(void*)) || (pot(align)!=align))
         return EINVAL;
-    void* ret = box_memalign(align, size);
+    void* ret = actual_memalign(align, size);
     if(!ret)
         return ENOMEM;
     *p = ret;
@@ -271,7 +329,7 @@ EXPORT void* valloc(size_t size)
     if(malloc_hack_2 && ALLOC && real_valloc) {
         return (void*)RunFunctionFmt(real_valloc, "L", size);
     }
-    return box_memalign(box64_pagesize, size);
+    return actual_memalign(box64_pagesize, size);
 }
 
 EXPORT void* pvalloc(size_t size)
@@ -279,7 +337,7 @@ EXPORT void* pvalloc(size_t size)
     if(malloc_hack_2 && ALLOC && real_pvalloc) {
         return (void*)RunFunctionFmt(real_pvalloc, "L", size);
     }
-    return box_memalign(box64_pagesize, (size+box64_pagesize-1)&~(box64_pagesize-1));
+    return actual_memalign(box64_pagesize, (size+box64_pagesize-1)&~(box64_pagesize-1));
 }
 
 EXPORT void cfree(void* p)
@@ -293,7 +351,7 @@ EXPORT void cfree(void* p)
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT size_t malloc_usable_size(void* p)
@@ -302,7 +360,7 @@ EXPORT size_t malloc_usable_size(void* p)
         if(getMmapped((uintptr_t)p))
             return RunFunctionFmt(real_malloc_usable_size, "p", p);
     }
-    return box_malloc_usable_size(p);
+    return actual_malloc_usable_size(p);
 }
 
 EXPORT void* my__Znwm(size_t sz)   //operator new(size_t)
@@ -310,7 +368,7 @@ EXPORT void* my__Znwm(size_t sz)   //operator new(size_t)
     if(malloc_hack_2 && real__Znwm) {
         return (void*)RunFunctionFmt(real__Znwm, "L", sz);
     }
-    return box_malloc(sz);
+    return actual_malloc(sz);
 }
 
 EXPORT void* my__ZnwmRKSt9nothrow_t(size_t sz, void* p)   //operator new(size_t, std::nothrow_t const&)
@@ -318,7 +376,7 @@ EXPORT void* my__ZnwmRKSt9nothrow_t(size_t sz, void* p)   //operator new(size_t,
     if(malloc_hack_2 && real__ZnwmRKSt9nothrow_t) {
         return (void*)RunFunctionFmt(real__ZnwmRKSt9nothrow_t, "Lp", sz, p);
     }
-    return box_malloc(sz);
+    return actual_malloc(sz);
 }
 
 EXPORT void* my__Znam(size_t sz)   //operator new[](size_t)
@@ -326,7 +384,7 @@ EXPORT void* my__Znam(size_t sz)   //operator new[](size_t)
     if(malloc_hack_2 && real__Znam) {
         return (void*)RunFunctionFmt(real__Znam, "L", sz);
     }
-    return box_malloc(sz);
+    return actual_malloc(sz);
 }
 
 EXPORT void* my__ZnamRKSt9nothrow_t(size_t sz, void* p)   //operator new[](size_t, std::nothrow_t const&)
@@ -334,7 +392,7 @@ EXPORT void* my__ZnamRKSt9nothrow_t(size_t sz, void* p)   //operator new[](size_
     if(malloc_hack_2 && real__ZnamRKSt9nothrow_t) {
         return (void*)RunFunctionFmt(real__ZnamRKSt9nothrow_t, "Lp", sz, p);
     }
-    return box_malloc(sz);
+    return actual_malloc(sz);
 }
 
 
@@ -349,7 +407,7 @@ EXPORT void my__ZdaPv(void* p)   //operator delete[](void*)
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my__ZdaPvm(void* p, size_t sz)   //operator delete[](void*, size_t)
@@ -363,7 +421,7 @@ EXPORT void my__ZdaPvm(void* p, size_t sz)   //operator delete[](void*, size_t)
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my__ZdaPvmSt11align_val_t(void* p, size_t sz, size_t align)   //operator delete[](void*, unsigned long, std::align_val_t)
@@ -377,7 +435,7 @@ EXPORT void my__ZdaPvmSt11align_val_t(void* p, size_t sz, size_t align)   //oper
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my__ZdlPv(void* p)   //operator delete(void*)
@@ -391,7 +449,7 @@ EXPORT void my__ZdlPv(void* p)   //operator delete(void*)
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my__ZdlPvm(void* p, size_t sz)   //operator delete(void*, size_t)
@@ -405,7 +463,7 @@ EXPORT void my__ZdlPvm(void* p, size_t sz)   //operator delete(void*, size_t)
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void* my__ZnwmSt11align_val_t(size_t sz, size_t align)  //// operator new(unsigned long, std::align_val_t)
@@ -413,7 +471,7 @@ EXPORT void* my__ZnwmSt11align_val_t(size_t sz, size_t align)  //// operator new
     if(malloc_hack_2 && real__ZnwmSt11align_val_t) {
         return (void*)RunFunctionFmt(real__ZnwmSt11align_val_t, "LL", sz, align);
     }
-    return box_memalign(align, sz);
+    return actual_memalign(align, sz);
 }
 
 EXPORT void* my__ZnwmSt11align_val_tRKSt9nothrow_t(size_t sz, size_t align, void* p)  //// operator new(unsigned long, std::align_val_t, std::nothrow_t const&)
@@ -421,7 +479,7 @@ EXPORT void* my__ZnwmSt11align_val_tRKSt9nothrow_t(size_t sz, size_t align, void
     if(malloc_hack_2 && real__ZnwmSt11align_val_tRKSt9nothrow_t) {
         return (void*)RunFunctionFmt(real__ZnwmSt11align_val_tRKSt9nothrow_t, "LLp", sz, align, p);
     }
-    return box_memalign(align, sz);
+    return actual_memalign(align, sz);
 }
 
 EXPORT void* my__ZnamSt11align_val_t(size_t sz, size_t align)  //// operator new[](unsigned long, std::align_val_t)
@@ -429,7 +487,7 @@ EXPORT void* my__ZnamSt11align_val_t(size_t sz, size_t align)  //// operator new
     if(malloc_hack_2 && real__ZnamSt11align_val_t) {
         return (void*)RunFunctionFmt(real__ZnamSt11align_val_t, "LL", sz, align);
     }
-    return box_memalign(align, sz);
+    return actual_memalign(align, sz);
 }
 
 EXPORT void* my__ZnamSt11align_val_tRKSt9nothrow_t(size_t sz, size_t align, void* p)  //// operator new[](unsigned long, std::align_val_t, std::nothrow_t const&)
@@ -437,7 +495,7 @@ EXPORT void* my__ZnamSt11align_val_tRKSt9nothrow_t(size_t sz, size_t align, void
     if(malloc_hack_2 && real__ZnamSt11align_val_tRKSt9nothrow_t) {
         return (void*)RunFunctionFmt(real__ZnamSt11align_val_tRKSt9nothrow_t, "LLp", sz, align, p);
     }
-    return box_memalign(align, sz);
+    return actual_memalign(align, sz);
 }
 
 EXPORT void my__ZdlPvRKSt9nothrow_t(void* p, void* n)   //operator delete(void*, std::nothrow_t const&)
@@ -451,7 +509,7 @@ EXPORT void my__ZdlPvRKSt9nothrow_t(void* p, void* n)   //operator delete(void*,
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my__ZdaPvSt11align_val_tRKSt9nothrow_t(void* p, size_t align, void* n)   //operator delete[](void*, std::align_val_t, std::nothrow_t const&)
@@ -466,7 +524,7 @@ EXPORT void my__ZdaPvSt11align_val_tRKSt9nothrow_t(void* p, size_t align, void*
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my__ZdlPvmSt11align_val_t(void* p, size_t sz, size_t align)   //operator delete(void*, unsigned long, std::align_val_t)
@@ -480,7 +538,7 @@ EXPORT void my__ZdlPvmSt11align_val_t(void* p, size_t sz, size_t align)   //oper
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my__ZdaPvRKSt9nothrow_t(void* p, void* n)   //operator delete[](void*, std::nothrow_t const&)
@@ -494,7 +552,7 @@ EXPORT void my__ZdaPvRKSt9nothrow_t(void* p, void* n)   //operator delete[](void
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my__ZdaPvSt11align_val_t(void* p, size_t align)   //operator delete[](void*, std::align_val_t)
@@ -508,7 +566,7 @@ EXPORT void my__ZdaPvSt11align_val_t(void* p, size_t align)   //operator delete[
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my__ZdlPvSt11align_val_t(void* p, size_t align)   //operator delete(void*, std::align_val_t)
@@ -522,7 +580,7 @@ EXPORT void my__ZdlPvSt11align_val_t(void* p, size_t align)   //operator delete(
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my__ZdlPvSt11align_val_tRKSt9nothrow_t(void* p, size_t align, void* n)   //operator delete(void*, std::align_val_t, std::nothrow_t const&)
@@ -536,92 +594,92 @@ EXPORT void my__ZdlPvSt11align_val_tRKSt9nothrow_t(void* p, size_t align, void*
             return;
         }
     }
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void* my_tc_calloc(size_t n, size_t s)
 {
-    return box_calloc(n, s);
+    return actual_calloc(n, s);
 }
 
 EXPORT void my_tc_cfree(void* p)
 {
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my_tc_delete(void* p)
 {
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my_tc_deletearray(void* p)
 {
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my_tc_deletearray_nothrow(void* p, void* n)
 {
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my_tc_delete_nothrow(void* p, void* n)
 {
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void my_tc_free(void* p)
 {
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void* my_tc_malloc(size_t s)
 {
-    return box_calloc(1, s);
+    return actual_calloc(1, s);
 }
 
 EXPORT size_t my_tc_malloc_size(void* p)
 {
-    return box_malloc_usable_size(p);
+    return actual_malloc_usable_size(p);
 }
 
 EXPORT void* my_tc_new(size_t s)
 {
-    return box_calloc(1, s);
+    return actual_calloc(1, s);
 }
 
 EXPORT void* my_tc_new_nothrow(size_t s, void* n)
 {
-        return box_calloc(1, s);
+    return actual_calloc(1, s);
 }
 
 EXPORT void* my_tc_newarray(size_t s)
 {
-        return box_calloc(1, s);
+    return actual_calloc(1, s);
 }
 
 EXPORT void* my_tc_newarray_nothrow(size_t s, void* n)
 {
-        return box_calloc(1, s);
+    return actual_calloc(1, s);
 }
 
 EXPORT void* my_tc_pvalloc(size_t size)
 {
-    return box_memalign(box64_pagesize, (size+box64_pagesize-1)&~(box64_pagesize-1));
+    return actual_memalign(box64_pagesize, (size+box64_pagesize-1)&~(box64_pagesize-1));
 }
 
 EXPORT void* my_tc_valloc(size_t size)
 {
-    return box_memalign(box64_pagesize, size);
+    return actual_memalign(box64_pagesize, size);
 }
 
 EXPORT void* my_tc_memalign(size_t align, size_t size)
 {
-    return box_memalign(align, size);
+    return actual_memalign(align, size);
 }
 
 EXPORT void* my_tc_malloc_skip_new_handler_weak(size_t s)
 {
-    return box_calloc(1, s);
+    return actual_calloc(1, s);
 }
 
 EXPORT int my_tc_mallocopt(int param, int value)
@@ -646,13 +704,13 @@ EXPORT int my_tc_set_new_mode(int mode)
 */
 EXPORT void* my_tc_malloc_skip_new_handler(size_t s)
 {
-    return box_calloc(1, s);
+    return actual_calloc(1, s);
 }
 
 EXPORT void* my_tc_mallinfo(void* p)
 {
     // ignored, returning null stuffs
-    memset(p, 0, sizeof(size_t)*10);
+    memset(p, 0, (box64_is32bits?sizeof(ptr_t):sizeof(size_t))*10);
     return p;
 }
 
@@ -660,7 +718,7 @@ EXPORT int my_tc_posix_memalign(void** p, size_t align, size_t size)
 {
     if(align%sizeof(void*) || pot(align)!=align)
         return EINVAL;
-    void* ret = box_memalign(align, size);
+    void* ret = actual_memalign(align, size);
     if(!ret)
         return ENOMEM;
     *p = ret;
@@ -669,7 +727,7 @@ EXPORT int my_tc_posix_memalign(void** p, size_t align, size_t size)
 
 EXPORT void* my_tc_realloc(void* p, size_t s)
 {
-    return box_realloc(p, s);
+    return actual_realloc(p, s);
 }
 /*
 EXPORT int my_tc_version(int i)
@@ -685,34 +743,34 @@ EXPORT void* my_safer_scalable_aligned_realloc(void* p, size_t size, size_t alig
         return NULL;
     }
     if(align <= 8)
-        return box_realloc(p, size);
-    size_t old_size = box_malloc_usable_size(p);
+        return actual_realloc(p, size);
+    size_t old_size = actual_malloc_usable_size(p);
     if(old_size>=size)
         return p;
-    void* new_p = box_memalign(align, size);
+    void* new_p = actual_memalign(align, size);
     memcpy(new_p, p, (old_size<size)?old_size:size);
-    box_free(p);
+    actual_free(p);
     return p;
 }
 
 EXPORT void my_safer_scalable_free(void*p , void* old)
 {
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT size_t my_safer_scalable_msize(void* p, void* old)
 {
-    return box_malloc_usable_size(p);
+    return actual_malloc_usable_size(p);
 }
 
 EXPORT void* my_safer_scalable_realloc(void* p, size_t size, void* old)
 {
-    return box_realloc(p, size);
+    return actual_realloc(p, size);
 }
 
 EXPORT void my_scalable_aligned_free(void* p)
 {
-    box_free(p);
+    actual_free(p);
 }
 
 EXPORT void* my_scalable_aligned_malloc(size_t size, size_t align)
@@ -722,8 +780,8 @@ EXPORT void* my_scalable_aligned_malloc(size_t size, size_t align)
         return NULL;
     }
     if(align <= 8)
-        return box_malloc(size);
-    return box_memalign(align, size);
+        return actual_malloc(size);
+    return actual_memalign(align, size);
 }
 
 EXPORT void* my_scalable_aligned_realloc(void* p, size_t size, size_t align)
@@ -733,19 +791,19 @@ EXPORT void* my_scalable_aligned_realloc(void* p, size_t size, size_t align)
         return NULL;
     }
     if(align <= 8)
-        return box_realloc(p, size);
-    size_t old_size = box_malloc_usable_size(p);
+        return actual_realloc(p, size);
+    size_t old_size = actual_malloc_usable_size(p);
     if(old_size>=size)
         return p;
-    void* new_p = box_memalign(align, size);
+    void* new_p = actual_memalign(align, size);
     memcpy(new_p, p, (old_size<size)?old_size:size);
-    box_free(p);
+    actual_free(p);
     return p;
 }
 
 EXPORT size_t my_scalable_msize(void* p)
 {
-    return box_malloc_usable_size(p);
+    return actual_malloc_usable_size(p);
 }
 
 
@@ -776,11 +834,15 @@ void checkHookedSymbols(elfheader_t* h)
     int hooked_symtab = 0;
     if(box64_malloc_hack==1)
         return;
+    if(box64_is32bits) {
+        /* TODO? */
+        return;
+    }
     for (size_t i=0; i<h->numSymTab; ++i) {
-        int type = ELF64_ST_TYPE(h->SymTab[i].st_info);
-        int sz = ELF64_ST_TYPE(h->SymTab[i].st_size);
-        if(type==STT_FUNC && sz) {
-            const char * symname = h->StrTab+h->SymTab[i].st_name;
+        int type = ELF64_ST_TYPE(h->SymTab._64[i].st_info);
+        int sz = ELF64_ST_TYPE(h->SymTab._64[i].st_size);
+        if((type==STT_FUNC) && sz && (h->SymTab._64[i].st_shndx!=0 && h->SymTab._64[i].st_shndx<=65521)) {
+            const char * symname = h->StrTab+h->SymTab._64[i].st_name;
             #define GO(A, B) if(!strcmp(symname, #A)) ++hooked; else if(!strcmp(symname, "__libc_" #A)) ++hooked;
             #define GO2(A, B)
             SUPER()
@@ -790,14 +852,14 @@ void checkHookedSymbols(elfheader_t* h)
     }
     if(hooked<2) {
         for (size_t i=0; i<h->numDynSym && hooked<2; ++i) {
-            const char * symname = h->DynStr+h->DynSym[i].st_name;
-            int bind = ELF64_ST_BIND(h->DynSym[i].st_info);
-            int type = ELF64_ST_TYPE(h->DynSym[i].st_info);
-            int vis = h->DynSym[i].st_other&0x3;
+            const char * symname = h->DynStr+h->DynSym._64[i].st_name;
+            int bind = ELF64_ST_BIND(h->DynSym._64[i].st_info);
+            int type = ELF64_ST_TYPE(h->DynSym._64[i].st_info);
+            int vis = h->DynSym._64[i].st_other&0x3;
             if((type==STT_FUNC) 
-            && (vis==STV_DEFAULT || vis==STV_PROTECTED) && (h->DynSym[i].st_shndx!=0 && h->DynSym[i].st_shndx<=65521)) {
-                uintptr_t offs = h->DynSym[i].st_value + h->delta;
-                size_t sz = h->DynSym[i].st_size;
+            && (vis==STV_DEFAULT || vis==STV_PROTECTED) && (h->DynSym._64[i].st_shndx!=0 && h->DynSym._64[i].st_shndx<=65521)) {
+                uintptr_t offs = h->DynSym._64[i].st_value + h->delta;
+                size_t sz = h->DynSym._64[i].st_size;
                 if(bind!=STB_LOCAL && bind!=STB_WEAK && sz>=sizeof(reloc_jmp_t)) {
                     #define GO(A, B) if(!strcmp(symname, #A)) ++hooked; else if(!strcmp(symname, "__libc_" #A)) ++hooked;
                     #define GO2(A, B)
@@ -814,11 +876,11 @@ void checkHookedSymbols(elfheader_t* h)
     printf_log(LOG_INFO, "Redirecting overridden malloc%s from %s function for %s\n", malloc_hack_2?" with hack":"", hooked_symtab?"symtab":"dynsym", ElfName(h));
     if(hooked_symtab) {
         for (size_t i=0; i<h->numSymTab; ++i) {
-            int type = ELF64_ST_TYPE(h->SymTab[i].st_info);
+            int type = ELF64_ST_TYPE(h->SymTab._64[i].st_info);
             if(type==STT_FUNC) {
-                const char * symname = h->StrTab+h->SymTab[i].st_name;
-                uintptr_t offs = h->SymTab[i].st_value + h->delta;
-                size_t sz = h->SymTab[i].st_size;
+                const char * symname = h->StrTab+h->SymTab._64[i].st_name;
+                uintptr_t offs = h->SymTab._64[i].st_value + h->delta;
+                size_t sz = h->SymTab._64[i].st_size;
                 #define GO(A, B) if(!strcmp(symname, "__libc_" #A)) {uintptr_t alt = AddCheckBridge(my_context->system, B, A, 0, #A); printf_log(LOG_DEBUG, "Redirecting %s function from %p (%s)\n", symname, (void*)offs, ElfName(h)); addRelocJmp((void*)offs, (void*)alt, sz, "__libc_" #A, h, NULL);}
                 #define GO2(A, B)
                 SUPER()
@@ -833,14 +895,14 @@ void checkHookedSymbols(elfheader_t* h)
         }
     } else {
         for (size_t i=0; i<h->numDynSym; ++i) {
-            const char * symname = h->DynStr+h->DynSym[i].st_name;
-            int bind = ELF64_ST_BIND(h->DynSym[i].st_info);
-            int type = ELF64_ST_TYPE(h->DynSym[i].st_info);
-            int vis = h->DynSym[i].st_other&0x3;
+            const char * symname = h->DynStr+h->DynSym._64[i].st_name;
+            int bind = ELF64_ST_BIND(h->DynSym._64[i].st_info);
+            int type = ELF64_ST_TYPE(h->DynSym._64[i].st_info);
+            int vis = h->DynSym._64[i].st_other&0x3;
             if((type==STT_FUNC) 
-            && (vis==STV_DEFAULT || vis==STV_PROTECTED) && (h->DynSym[i].st_shndx!=0 && h->DynSym[i].st_shndx<=65521)) {
-                uintptr_t offs = h->DynSym[i].st_value + h->delta;
-                size_t sz = h->DynSym[i].st_size;
+            && (vis==STV_DEFAULT || vis==STV_PROTECTED) && (h->DynSym._64[i].st_shndx!=0 && h->DynSym._64[i].st_shndx<=65521)) {
+                uintptr_t offs = h->DynSym._64[i].st_value + h->delta;
+                size_t sz = h->DynSym._64[i].st_size;
                 if(bind!=STB_LOCAL && bind!=STB_WEAK) {
                     #define GO(A, B) if(!strcmp(symname, "__libc_" #A)) {uintptr_t alt = AddCheckBridge(my_context->system, B, A, 0, #A); printf_log(LOG_DEBUG, "Redirecting %s function from %p (%s)\n", symname, (void*)offs, ElfName(h)); addRelocJmp((void*)offs, (void*)alt, sz, "__libc_" #A, h, NULL);}
                     #define GO2(A, B)
