@@ -68,11 +68,27 @@ void RV64_Detect_Function()
     BR(xRA);
     rv64_zbs = Check(my_block);
 
-    // Test Vector v1.0 with CSRR zero, vcsr
     block = (uint32_t*)my_block;
-    CSRRS(xZR, xZR, 0x00f);
+    CSRRS(xZR, xZR, 0xc22 /* vlenb */);
     BR(xRA);
-    rv64_vector = Check(my_block); // TODO: also check vlen >= 128
+    rv64_vector = Check(my_block);
+
+    if (rv64_vector) {
+        block = (uint32_t*)my_block;
+        CSRRS(xZR, xZR, 0x00f /* vcsr */); // vcsr does not exists in xtheadvector
+        BR(xRA);
+        rv64_xtheadvector = !Check(my_block);
+    }
+
+    if (rv64_vector) {
+        int vlenb = 0;
+        asm volatile("csrr %0, 0xc22" : "=r"(vlenb));
+        rv64_vlen = vlenb * 8;
+        if (vlenb < 16) {
+            // we need vlen >= 128
+            rv64_vector = 0;
+        }
+    }
 
     // THead vendor extensions
     if (!rv64_zba) {
