@@ -39,19 +39,31 @@ void pressure_vessel(int argc, const char** argv, int nextarg, const char* prog)
                 if(is_usr) {
                     // transform RESSURE_VESSEL_APP_LD_LIBRARY_PATH to BOX86_ / BOX64_ LD_LIBRARY_PATH
                     char tmp[strlen(argv[nextarg])+150];
-                    strcpy(tmp, "BOX86_LD_LIBRARY_PATH=/lib/box86:/usr/lib/box86:/lib/i386-linux-gnu:/usr/lib/i386-linux-gnu:/usr/lib/i686-pc-linux-gnu:/usr/lib32:");
+                    strcpy(tmp, "BOX86_LD_LIBRARY_PATH=/lib/box86:/usr/lib/box86:/lib/i386-linux-gnu:/usr/lib/i386-linux-gnu:/usr/lib/box86-i386-linux-gnu:/usr/lib/box64-i386-linux-gnu:/usr/lib/i686-pc-linux-gnu:/usr/lib32:");
                     strcat(tmp, argv[nextarg]+strlen("--env-if-host=STEAM_RUNTIME_LIBRARY_PATH="));
                     char *p = strchr(tmp, '=');
                     *p ='\0'; ++p;
                     setenv(tmp, p, 1);
                     printf_log(LOG_DEBUG, "setenv(%s, %s, 1)\n", tmp, p);
-                    strcpy(tmp, "BOX64_LD_LIBRARY_PATH=/lib/x86_64-linux-gnu:/usr/lib/x86_64-linux-gnu:");
+                    strcpy(tmp, "BOX64_LD_LIBRARY_PATH=/lib/x86_64-linux-gnu:/usr/lib/x86_64-linux-gnu:/usr/lib/box64-x86_64-linux-gnu:/usr/lib/box64-i386-linux-gnu:");
                     strcat(tmp, argv[nextarg]+strlen("--env-if-host=STEAM_RUNTIME_LIBRARY_PATH="));
                     p = strchr(tmp, '=');
                     *p ='\0'; ++p;
                     setenv(tmp, p, 1);
                     printf_log(LOG_DEBUG, "setenv(%s, %s, 1)\n", tmp, p);
                 }
+            } else if(strstr(argv[nextarg], "--ld-preloads=")==argv[nextarg]) {
+                // transform to BOX86_ / BOX64_ LD_PRELOAD
+                char tmp[strlen(argv[nextarg])+4];
+                strcpy(tmp, "BOX64_LD_PRELOAD=");
+                strcat(tmp, strchr(argv[nextarg], '=')+1);
+                char *p = strchr(tmp, '=');
+                *p ='\0'; ++p;
+                setenv(tmp, p, 1);
+                printf_log(LOG_DEBUG, "setenv(%s, %s, 1)\n", tmp, p);
+                tmp[3] = '8'; tmp[4] = '6';
+                setenv(tmp, p, 1);
+                printf_log(LOG_DEBUG, "setenv(%s, %s, 1)\n", tmp, p);
             } else if(!strcmp(argv[nextarg], "--")) {
                 printf_log(LOG_DEBUG, "End of pressure-vessel-wrap parameters\n");
             }else {
@@ -137,10 +149,11 @@ void pressure_vessel(int argc, const char** argv, int nextarg, const char* prog)
     my_context = NewBox64Context(argc - nextarg);
     int x86 = my_context->box86path?FileIsX86ELF(argv[nextarg]):0;
     int x64 = my_context->box64path?FileIsX64ELF(argv[nextarg]):0;
+    int sh = my_context->bashpath?FileIsShell(argv[nextarg]):0;
     // create the new argv array
     const char** newargv = (const char**)box_calloc((argc-nextarg)+1+((x86 || x64)?1:0), sizeof(char*));
-    if(x86 || x64) {
-        newargv[0] = x64?my_context->box64path:my_context->box86path;
+    if(x86 || x64 || sh) {
+        newargv[0] = x86?my_context->box86path:my_context->box64path;
         printf_log(LOG_DEBUG, "argv[%d]=\"%s\"\n", 0, newargv[0]);    
         for(int i=nextarg; i<argc; ++i) {
             printf_log(LOG_DEBUG, "argv[%d]=\"%s\"\n", 1+i-nextarg, argv[i]);    
