@@ -19,11 +19,9 @@
 #include "gtkclass.h"
 #include "myalign.h"
 
-#ifdef ANDROID
-    const char* gobject2Name = "libgobject-2.0.so";
-#else
-    const char* gobject2Name = "libgobject-2.0.so.0";
-#endif
+const char* gobject2Name = "libgobject-2.0.so.0";
+#define ALTNAME "libgobject-2.0.so"
+
 #define LIBNAME gobject2
 
 typedef size_t(*LFv_t)(void);
@@ -48,94 +46,6 @@ static void addGObject2Alternate(library_t* lib);
 
 #include "wrappercallback.h"
 
-static int signal_cb(void* a, void* b, void* c, void* d, void* e)
-{
-    // signal can have many signature... so first job is to find the data!
-    // hopefully, no callback have more than 4 arguments...
-    my_signal_t* sig = NULL;
-    int i = 0;
-    if(my_signal_is_valid(a)) {
-        sig = (my_signal_t*)a;
-        i = 1;
-    }
-    if(!sig && my_signal_is_valid(b)) {
-        sig = (my_signal_t*)b;
-        i = 2;
-    }
-    if(!sig && my_signal_is_valid(c))  {
-        sig = (my_signal_t*)c;
-        i = 3;
-    }
-    if(!sig && my_signal_is_valid(d)) {
-        sig = (my_signal_t*)d;
-        i = 4;
-    }
-    if(!sig && my_signal_is_valid(e)) {
-        sig = (my_signal_t*)e;
-        i = 5;
-        printf_log(LOG_DEBUG, "Warning, GObject2 signal callback with 5 args found, sig=%p!\n", sig);
-    }
-    printf_log(LOG_DEBUG, "gobject2 Signal called, sig=%p, handler=%p, NArgs=%d\n", sig, sig?(void*)sig->c_handler:NULL, i);
-    switch(i) {
-        case 1: return (int)RunFunctionFmt(sig->c_handler, "p", sig->data);
-        case 2: return (int)RunFunctionFmt(sig->c_handler, "pp", a, sig->data);
-        case 3: return (int)RunFunctionFmt(sig->c_handler, "ppp", a, b, sig->data);
-        case 4: return (int)RunFunctionFmt(sig->c_handler, "pppp", a, b, c, sig->data);
-        case 5: return (int)RunFunctionFmt(sig->c_handler, "ppppp", a, b, c, d, sig->data);
-    }
-    printf_log(LOG_NONE, "Warning, GObject2 signal callback but no data found!\n");
-    return 0;
-}
-static int signal_cb_swapped(my_signal_t* sig, void* b, void* c, void* d)
-{
-    // data is in front here...
-    printf_log(LOG_DEBUG, "gobject2 swaped4 Signal called, sig=%p\n", sig);
-    return (int)RunFunctionFmt(sig->c_handler, "pppp", sig->data, b, c, d);
-}
-static int signal_cb_5(void* a, void* b, void* c, void* d, my_signal_t* sig)
-{
-    printf_log(LOG_DEBUG, "gobject2 5 Signal called, sig=%p\n", sig);
-    return (int)RunFunctionFmt(sig->c_handler, "ppppp", a, b, c, d, sig->data);
-}
-static int signal_cb_swapped_5(my_signal_t* sig, void* b, void* c, void* d, void* e)
-{
-    // data is in front here...
-    printf_log(LOG_DEBUG, "gobject2 swaped5 Signal called, sig=%p\n", sig);
-    return (int)RunFunctionFmt(sig->c_handler, "ppppp", sig->data, b, c, d, e);
-}
-static int signal_cb_6(void* a, void* b, void* c, void* d, void* e, my_signal_t* sig)
-{
-    printf_log(LOG_DEBUG, "gobject2 6 Signal called, sig=%p\n", sig);
-    return (int)RunFunctionFmt(sig->c_handler, "pppppp", a, b, c, d, e, sig->data);
-}
-static int signal_cb_swapped_6(my_signal_t* sig, void* b, void* c, void* d, void* e, void* f)
-{
-    // data is in front here...
-    printf_log(LOG_DEBUG, "gobject2 swaped6 Signal called, sig=%p\n", sig);
-    return (int)RunFunctionFmt(sig->c_handler, "pppppp", sig->data, b, c, d, e, f);
-}
-static int signal_cb_8(void* a, void* b, void* c, void* d, void* e, void* f, void* g, my_signal_t* sig)
-{
-    printf_log(LOG_DEBUG, "gobject2 8 Signal called, sig=%p\n", sig);
-    return (int)RunFunctionFmt(sig->c_handler, "pppppppp", a, b, c, d, e, f, g, sig->data);
-}
-static int signal_cb_swapped_8(my_signal_t* sig, void* b, void* c, void* d, void* e, void* f, void* g, void* h)
-{
-    // data is in front here...
-    printf_log(LOG_DEBUG, "gobject2 swaped8 Signal called, sig=%p\n", sig);
-    return (int)RunFunctionFmt(sig->c_handler, "pppppppp", sig->data, b, c, d, e, f, g, h);
-}
-
-static void signal_delete(my_signal_t* sig, void* b)
-{
-    uintptr_t d = sig->destroy;
-    if(d) {
-        RunFunctionFmt(d, "pp", sig->data, b);
-    }
-    printf_log(LOG_DEBUG, "gobject2 Signal deleted, sig=%p, destroy=%p\n", sig, (void*)d);
-    free(sig);
-}
-
 static void addGObject2Alternate(library_t* lib)
 {
     #define GO(A, W) AddAutomaticBridge(lib->w.bridge, W, dlsym(lib->w.lib, #A), 0, #A)
@@ -184,83 +94,8 @@ static void addGObject2Alternate(library_t* lib)
     GO(g_cclosure_marshal_BOOLEAN__FLAGSv,          vFpppppip);
     GO(g_cclosure_marshal_BOOLEAN__BOXED_BOXEDv,    vFpppppip);
     #undef GO
-    #define GO(A, W) AddAutomaticBridge(lib->w.bridge, W, A, 0, #A)
-    GO(signal_cb, iFpppp);
-    GO(signal_cb_swapped, iFpppp);
-    GO(signal_cb_5, iFppppp);
-    GO(signal_cb_swapped_5, iFppppp);
-    GO(signal_cb_6, iFpppppp);
-    GO(signal_cb_swapped_6, iFpppppp);
-    GO(signal_cb_8, iFpppppppp);
-    GO(signal_cb_swapped_8, iFpppppppp);
-    GO(signal_delete, vFpp);
-    #undef GO
 }
 
-EXPORT uintptr_t my_g_signal_connect_data(x64emu_t* emu, void* instance, void* detailed, void* c_handler, void* data, void* closure, uint32_t flags)
-{
-    //TODO: get the type of instance to be more precise below
-
-    my_signal_t *sig = new_mysignal(c_handler, data, closure);
-    uintptr_t ret = 0;
-    #define GO(A, B) if(strcmp((const char*)detailed, A)==0) ret = my->g_signal_connect_data(instance, detailed, (flags&2)?((void*)signal_cb_swapped_##B):((void*)signal_cb_##B), sig, signal_delete, flags);
-    GO("query-tooltip", 6)  // GtkWidget
-    else GO("query_tooltip", 6)
-    else GO("selection-get", 5) //GtkWidget
-    else GO("selection_get", 5)
-    else GO("drag-data-get", 5) //GtkWidget
-    else GO("drag-data-received", 8)    //GtkWidget
-    else GO("drag_data_received", 8)
-    else GO("drag-drop", 6) //GtkWidget
-    else GO("drag_drop", 6)
-    else GO("drag-motion", 6)   //GtkWidget
-    else GO("drag_motion", 6)
-    else GO("expand-collapse-cursor-row", 5)    //GtkTreeView
-    else GO("expand_collapse_cursor_row", 5)
-    else GO("insert-text", 5)   // GtkEditable
-    else GO("insert_text", 5)
-    else GO("move-cursor", 5)   // GtkEntry
-    else GO("move_cursor", 5)
-    else
-        ret = my->g_signal_connect_data(instance, detailed, (flags&2)?((void*)signal_cb_swapped):((void*)signal_cb), sig, signal_delete, flags);
-    #undef GO
-    printf_log(LOG_DEBUG, "Connecting gobject2 %p signal \"%s\" with sig=%p to %p, flags=%d\n", instance, (char*)detailed, sig, c_handler, flags);
-    return ret;
-}
-
-
-EXPORT void* my_g_object_connect(x64emu_t* emu, void* object, void* signal_spec, void** b)
-{
-        //gobject2_my_t *my = (gobject2_my_t*)my_lib->w.p2;
-
-    char* spec = (char*)signal_spec;
-    while(spec) {
-        // loop on each triplet...
-        if(strstr(spec, "signal::")==spec) {
-            my_g_signal_connect_data(emu, object, spec+strlen("signal::"), b[0], b[1], NULL, 0);
-            b+=2;
-            spec = (char*)*(b++);
-        } else if(strstr(spec, "swapped_signal::")==spec || strstr(spec, "swapped-signal::")==spec) {
-            my_g_signal_connect_data(emu, object, spec+strlen("swapped_signal::"), b[0], b[1], NULL, 2);
-            b+=2;
-            spec = (char*)*(b++);
-        } else if(strstr(spec, "signal_after::")==spec || strstr(spec, "signal-after::")==spec) {
-            my_g_signal_connect_data(emu, object, spec+strlen("signal_after::"), b[0], b[1], NULL, 1);
-            b+=2;
-            spec = (char*)*(b++);
-        } else if(strstr(spec, "swapped_signal_after::")==spec || strstr(spec, "swapped-signal-after::")==spec) {
-            my_g_signal_connect_data(emu, object, spec+strlen("swapped_signal_after::"), b[0], b[1], NULL, 1|2);
-            b+=2;
-            spec = (char*)*(b++);
-        } else {
-            printf_log(LOG_NONE, "Warning, don't know how to handle signal spec \"%s\" in g_object_connect\n", spec);
-            spec = NULL;
-        }
-    }
-    return object;
-}
-
-
 #define SUPER() \
 GO(0)   \
 GO(1)   \
@@ -511,15 +346,15 @@ static my_GParamSpecTypeInfo_t* findFreeGParamSpecTypeInfo(my_GParamSpecTypeInfo
     #undef GO
     #define GO(A) if(ref_GParamSpecTypeInfo_##A == 0) {   \
         ref_GParamSpecTypeInfo_##A = fcts;                 \
-        my_GParamSpecTypeInfo_##A.instance_init = (fcts->instance_init)?((GetNativeFnc((uintptr_t)fcts->instance_init))?GetNativeFnc((uintptr_t)fcts->instance_init):my_funcs_instance_init_##A):NULL;    \
+        my_GParamSpecTypeInfo_##A.instance_init = (fcts->instance_init)?((GetNativeFnc((uintptr_t)fcts->instance_init))?(void (*)(void *))GetNativeFnc((uintptr_t)fcts->instance_init):my_funcs_instance_init_##A):NULL;    \
         fct_funcs_instance_init_##A = (uintptr_t)fcts->instance_init;                            \
-        my_GParamSpecTypeInfo_##A.finalize = (fcts->finalize)?((GetNativeFnc((uintptr_t)fcts->finalize))?GetNativeFnc((uintptr_t)fcts->finalize):my_funcs_finalize_##A):NULL;    \
+        my_GParamSpecTypeInfo_##A.finalize = (fcts->finalize)?((GetNativeFnc((uintptr_t)fcts->finalize))?(void (*)(void *))GetNativeFnc((uintptr_t)fcts->finalize):my_funcs_finalize_##A):NULL;    \
         fct_funcs_finalize_##A = (uintptr_t)fcts->finalize;                            \
-        my_GParamSpecTypeInfo_##A.value_set_default = (fcts->value_set_default)?((GetNativeFnc((uintptr_t)fcts->value_set_default))?GetNativeFnc((uintptr_t)fcts->value_set_default):my_funcs_value_set_default_##A):NULL;    \
+        my_GParamSpecTypeInfo_##A.value_set_default = (fcts->value_set_default)?((GetNativeFnc((uintptr_t)fcts->value_set_default))?(void (*)(void *, void *))GetNativeFnc((uintptr_t)fcts->value_set_default):my_funcs_value_set_default_##A):NULL;    \
         fct_funcs_value_set_default_##A = (uintptr_t)fcts->value_set_default;                            \
-        my_GParamSpecTypeInfo_##A.value_validate = (fcts->value_validate)?((GetNativeFnc((uintptr_t)fcts->value_validate))?GetNativeFnc((uintptr_t)fcts->value_validate):my_funcs_value_validate_##A):NULL;    \
+        my_GParamSpecTypeInfo_##A.value_validate = (fcts->value_validate)?((GetNativeFnc((uintptr_t)fcts->value_validate))?(int (*)(void *, void *))GetNativeFnc((uintptr_t)fcts->value_validate):my_funcs_value_validate_##A):NULL;    \
         fct_funcs_value_validate_##A = (uintptr_t)fcts->value_validate;                            \
-        my_GParamSpecTypeInfo_##A.values_cmp = (fcts->values_cmp)?((GetNativeFnc((uintptr_t)fcts->values_cmp))?GetNativeFnc((uintptr_t)fcts->values_cmp):my_funcs_values_cmp_##A):NULL;    \
+        my_GParamSpecTypeInfo_##A.values_cmp = (fcts->values_cmp)?((GetNativeFnc((uintptr_t)fcts->values_cmp))?(int (*)(void *, void *, void *))GetNativeFnc((uintptr_t)fcts->values_cmp):my_funcs_values_cmp_##A):NULL;    \
         fct_funcs_values_cmp_##A = (uintptr_t)fcts->values_cmp;                            \
         return &my_GParamSpecTypeInfo_##A;                \
     }
@@ -623,9 +458,89 @@ static void* findGCallbackFct(void* fct)
     printf_log(LOG_NONE, "Warning, no more slot for gobject generic GCallback\n");
     return NULL;
 }
-
+// EmissionHook
+#define GO(A)   \
+static uintptr_t my_EmissionHook_fct_##A = 0;                           \
+static void my_EmissionHook_##A(void* a, uint32_t b, void* c, void* d)  \
+{                                                                       \
+    RunFunctionFmt(my_EmissionHook_fct_##A, "pupp", a, b, c, d);        \
+}
+SUPER()
+#undef GO
+static void* findEmissionHookFct(void* fct)
+{
+    if(!fct) return fct;
+    if(GetNativeFnc((uintptr_t)fct))  return GetNativeFnc((uintptr_t)fct);
+    #define GO(A) if(my_EmissionHook_fct_##A == (uintptr_t)fct) return my_EmissionHook_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my_EmissionHook_fct_##A == 0) {my_EmissionHook_fct_##A = (uintptr_t)fct; return my_EmissionHook_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for gobject2 generic EmissionHook callback\n");
+    return NULL;
+}
+// Event
+#define GO(A)   \
+static uintptr_t my_Event_fct_##A = 0;                                                                              \
+static void my_Event_##A(void* a, void* b, void* c, void* d, void* e, void* f, void* g, void* h, void* i, void* j)  \
+{                                                                                                                   \
+    RunFunctionFmt(my_Event_fct_##A, "pppppppppp", a, b, c, d, e, f, g, h, i, j);                                   \
+}
+SUPER()
+#undef GO
+static void* findEventFct(void* fct)
+{
+    if(!fct) return fct;
+    if(GetNativeFnc((uintptr_t)fct))  return GetNativeFnc((uintptr_t)fct);
+    #define GO(A) if(my_Event_fct_##A == (uintptr_t)fct) return my_Event_##A;
+    SUPER()
+    #undef GO
+    #define GO(A) if(my_Event_fct_##A == 0) {my_Event_fct_##A = (uintptr_t)fct; return my_Event_##A; }
+    SUPER()
+    #undef GO
+    printf_log(LOG_NONE, "Warning, no more slot for gobject2 generic Event callback\n");
+    return NULL;
+}
 #undef SUPER
 
+EXPORT uintptr_t my_g_signal_connect_data(x64emu_t* emu, void* instance, void* detailed, void* c_handler, void* data, void* closure, uint32_t flags)
+{
+    return my->g_signal_connect_data(instance, detailed, findEventFct(c_handler), data, findGClosureNotify_Fct(closure), flags);
+}
+
+
+EXPORT void* my_g_object_connect(x64emu_t* emu, void* object, void* signal_spec, void** b)
+{
+        //gobject2_my_t *my = (gobject2_my_t*)my_lib->w.p2;
+
+    char* spec = (char*)signal_spec;
+    while(spec) {
+        // loop on each triplet...
+        if(strstr(spec, "signal::")==spec) {
+            my_g_signal_connect_data(emu, object, spec+strlen("signal::"), b[0], b[1], NULL, 0);
+            b+=2;
+            spec = (char*)*(b++);
+        } else if(strstr(spec, "swapped_signal::")==spec || strstr(spec, "swapped-signal::")==spec) {
+            my_g_signal_connect_data(emu, object, spec+strlen("swapped_signal::"), b[0], b[1], NULL, 2);
+            b+=2;
+            spec = (char*)*(b++);
+        } else if(strstr(spec, "signal_after::")==spec || strstr(spec, "signal-after::")==spec) {
+            my_g_signal_connect_data(emu, object, spec+strlen("signal_after::"), b[0], b[1], NULL, 1);
+            b+=2;
+            spec = (char*)*(b++);
+        } else if(strstr(spec, "swapped_signal_after::")==spec || strstr(spec, "swapped-signal-after::")==spec) {
+            my_g_signal_connect_data(emu, object, spec+strlen("swapped_signal_after::"), b[0], b[1], NULL, 1|2);
+            b+=2;
+            spec = (char*)*(b++);
+        } else {
+            printf_log(LOG_NONE, "Warning, don't know how to handle signal spec \"%s\" in g_object_connect\n", spec);
+            spec = NULL;
+        }
+    }
+    return object;
+}
+
 EXPORT uintptr_t my_g_signal_connect_object(x64emu_t* emu, void* instance, void* detailed, void* c_handler, void* object, uint32_t flags)
 {
     //TODO: get the type of instance to be more precise below
@@ -640,7 +555,7 @@ EXPORT int my_g_boxed_type_register_static(x64emu_t* emu, void* name, void* boxe
     return my->g_boxed_type_register_static(name, bc, bf);
 }
 
-EXPORT uint32_t my_g_signal_new(x64emu_t* emu, void* name, size_t itype, int flags, uint32_t offset, void* acc, void* accu_data, void* marsh, size_t rtype, uint32_t n, void** b)
+EXPORT uint32_t my_g_signal_new(x64emu_t* emu, void* name, size_t itype, uint32_t flags, uint32_t offset, void* acc, void* accu_data, void* marsh, size_t rtype, uint32_t n, void** b)
 {
     printf_log(LOG_DEBUG, "g_signal_new for \"%s\", with offset=%d and %d args\n", (const char*)name, offset, n);
 
@@ -661,14 +576,14 @@ EXPORT uint32_t my_g_signal_new(x64emu_t* emu, void* name, size_t itype, int fla
     return my->g_signal_new(name, itype, flags, offset, cb_acc, accu_data, cb_marsh, rtype, n, b[0], b[1], b[2]);
 }
 
-EXPORT uint32_t my_g_signal_newv(x64emu_t* emu, void* name, size_t itype, int flags, void* closure, void* acc, void* accu_data, void* marsh, size_t rtype, uint32_t n, void* types)
+EXPORT uint32_t my_g_signal_newv(x64emu_t* emu, void* name, size_t itype, uint32_t flags, void* closure, void* acc, void* accu_data, void* marsh, size_t rtype, uint32_t n, void* types)
 {
     printf_log(LOG_DEBUG, "g_signal_newv for \"%s\", with %d args\n", (const char*)name, n);
 
     return my->g_signal_newv(name, itype, flags, closure, findAccumulatorFct(acc), accu_data, findMarshalFct(marsh), rtype, n, types);
 }
 
-EXPORT uint32_t my_g_signal_new_valist(x64emu_t* emu, void* name, size_t itype, int flags, void* closure, void* acc, void* accu_data, void* marsh, size_t rtype, uint32_t n, x64_va_list_t b)
+EXPORT uint32_t my_g_signal_new_valist(x64emu_t* emu, void* name, size_t itype, uint32_t flags, void* closure, void* acc, void* accu_data, void* marsh, size_t rtype, uint32_t n, x64_va_list_t b)
 {
     printf_log(LOG_DEBUG, "g_signal_new_valist for \"%s\", with %d args\n", (const char*)name, n);
     #ifdef CONVERT_VALIST
@@ -679,7 +594,7 @@ EXPORT uint32_t my_g_signal_new_valist(x64emu_t* emu, void* name, size_t itype,
     return my->g_signal_new_valist(name, itype, flags, closure, findAccumulatorFct(acc), accu_data, findMarshalFct(marsh), rtype, n, VARARGS);
 }
 
-EXPORT uint32_t my_g_signal_handlers_block_matched(x64emu_t* emu, void* instance, int mask, uint32_t signal, void* detail, void* closure, void* fnc, void* data)
+EXPORT uint32_t my_g_signal_handlers_block_matched(x64emu_t* emu, void* instance, uint32_t mask, uint32_t signal, uint32_t detail, void* closure, void* fnc, void* data)
 {
     // NOTE that I have no idea of the fnc signature!...
     if (fnc) printf_log(LOG_DEBUG, "Warning, gobject g_signal_handlers_block_matched called with non null function \n");
@@ -687,7 +602,7 @@ EXPORT uint32_t my_g_signal_handlers_block_matched(x64emu_t* emu, void* instance
     return my->g_signal_handlers_block_matched(instance, mask, signal, detail, closure, fnc, data);
 }
 
-EXPORT uint32_t my_g_signal_handlers_unblock_matched(x64emu_t* emu, void* instance, int mask, uint32_t signal, void* detail, void* closure, void* fnc, void* data)
+EXPORT uint32_t my_g_signal_handlers_unblock_matched(x64emu_t* emu, void* instance, uint32_t mask, uint32_t signal, uint32_t detail, void* closure, void* fnc, void* data)
 {
     // NOTE that I have no idea of the fnc signature!...
     if (fnc) printf_log(LOG_DEBUG, "Warning, gobject g_signal_handlers_unblock_matched called with non null function \n");
@@ -695,7 +610,7 @@ EXPORT uint32_t my_g_signal_handlers_unblock_matched(x64emu_t* emu, void* instan
     return my->g_signal_handlers_unblock_matched(instance, mask, signal, detail, closure, fnc, data);
 }
 
-EXPORT uint32_t my_g_signal_handlers_disconnect_matched(x64emu_t* emu, void* instance, int mask, uint32_t signal, void* detail, void* closure, void* fnc, void* data)
+EXPORT uint32_t my_g_signal_handlers_disconnect_matched(x64emu_t* emu, void* instance, uint32_t mask, uint32_t signal, uint32_t detail, void* closure, void* fnc, void* data)
 {
     // NOTE that I have no idea of the fnc signature!...
     if (fnc) printf_log(LOG_DEBUG, "Warning, gobject g_signal_handlers_disconnect_matched called with non null function \n");
@@ -703,7 +618,7 @@ EXPORT uint32_t my_g_signal_handlers_disconnect_matched(x64emu_t* emu, void* ins
     return my->g_signal_handlers_disconnect_matched(instance, mask, signal, detail, closure, fnc, data);
 }
 
-EXPORT unsigned long my_g_signal_handler_find(x64emu_t* emu, void* instance, int mask, uint32_t signal, void* detail, void* closure, void* fnc, void* data)
+EXPORT unsigned long my_g_signal_handler_find(x64emu_t* emu, void* instance, uint32_t mask, uint32_t signal, uint32_t detail, void* closure, void* fnc, void* data)
 {
     // NOTE that I have no idea of the fnc signature!...
     if (fnc) printf_log(LOG_DEBUG, "Warning, gobject g_signal_handler_find called with non null function \n");
@@ -751,20 +666,9 @@ EXPORT void my_g_value_register_transform_func(x64emu_t* emu, size_t src, size_t
     my->g_value_register_transform_func(src, dst, findValueTransformFct(f));
 }
 
-static int my_signal_emission_hook(void* ihint, uint32_t n, void* values, my_signal_t* sig)
+EXPORT unsigned long my_g_signal_add_emission_hook(x64emu_t* emu, uint32_t signal, uint32_t detail, void* f, void* data, void* notify)
 {
-    printf_log(LOG_DEBUG, "gobject2 Signal Emission Hook called, sig=%p\n", sig);
-    return (int)RunFunctionFmt(sig->c_handler, "pupp", ihint, n, values, sig->data);
-}
-EXPORT unsigned long my_g_signal_add_emission_hook(x64emu_t* emu, uint32_t signal, void* detail, void* f, void* data, void* notify)
-{
-    // there can be many signals connected, so something "light" is needed here
-
-    if(!f)
-        return my->g_signal_add_emission_hook(signal, detail, f, data, notify);
-    my_signal_t* sig = new_mysignal(f, data, notify);
-    printf_log(LOG_DEBUG, "gobject2 Signal Emission Hook for signal %d created for %p, sig=%p\n", signal, f, sig);
-    return my->g_signal_add_emission_hook(signal, detail, my_signal_emission_hook, sig, my_signal_delete);
+    return my->g_signal_add_emission_hook(signal, detail, findEmissionHookFct(f), data, findDestroyFct(notify));
 }
 
 EXPORT size_t my_g_type_register_static_simple(x64emu_t* emu, size_t parent, void* name, uint32_t class_size, void* class_init, uint32_t instance_size, void* instance_init, uint32_t flags)
@@ -801,7 +705,7 @@ EXPORT void my_g_param_spec_set_qdata_full(x64emu_t* emu, void* pspec, uint32_t
     my->g_param_spec_set_qdata_full(pspec, quark, data, findFreeFct(notify));
 }
 
-EXPORT int my_g_param_type_register_static(x64emu_t* emu, void* name, void* pspec_info)
+EXPORT size_t my_g_param_type_register_static(x64emu_t* emu, void* name, void* pspec_info)
 {
 
     return my->g_param_type_register_static(name, findFreeGParamSpecTypeInfo(pspec_info));
@@ -833,7 +737,7 @@ EXPORT void* my_g_type_class_peek_parent(x64emu_t* emu, void* object)
     return wrapCopyGTKClass(klass, type);
 }
 
-EXPORT void my_g_signal_emit_valist(x64emu_t* emu, void* inst, uint32_t id, void* quark, x64_va_list_t b)
+EXPORT void my_g_signal_emit_valist(x64emu_t* emu, void* inst, uint32_t id, uint32_t quark, x64_va_list_t b)
 {
     #ifdef CONVERT_VALIST
     CONVERT_VALIST(b);
@@ -843,7 +747,7 @@ EXPORT void my_g_signal_emit_valist(x64emu_t* emu, void* inst, uint32_t id, void
     my->g_signal_emit_valist(inst, id, quark, VARARGS);
 }
 
-EXPORT void my_g_signal_emit(x64emu_t* emu, void* inst, uint32_t id, void* quark, uintptr_t* b)
+EXPORT void my_g_signal_emit(x64emu_t* emu, void* inst, uint32_t id, uint32_t quark, uintptr_t* b)
 {
     CREATE_VALIST_FROM_VAARG(b, emu->scratch, 3);
     my->g_signal_emit_valist(inst, id, quark, VARARGS);
@@ -882,14 +786,16 @@ EXPORT void my_g_object_set(x64emu_t* emu, void* a1, void* a2, uintptr_t* b)
     my->g_object_set_valist(a1, a2, VARARGS);
 }
 
-EXPORT void my_g_object_set_qdata_full(x64emu_t* emu, void* o, void* q, void* data, void* d)
+EXPORT void my_g_object_set_qdata_full(x64emu_t* emu, void* o, uint32_t q, void* data, void* d)
 {
     my->g_object_set_qdata_full(o, q, data, findDestroyFct(d));
 }
 
 EXPORT void my_g_object_class_install_properties(x64emu_t* emu, void* klass, uint32_t n, void* specs)
 {
-    my->g_object_class_install_properties(unwrapCopyGTKClass(klass, my->g_object_get_type()), n, specs);
+    unwrapGTKClass(klass, my->g_object_get_type());
+    my->g_object_class_install_properties(klass, n, specs);
+    wrapGTKClass(klass, my->g_object_get_type());
 }
 
 EXPORT void my_g_object_weak_ref(x64emu_t* emu, void* object, void* notify, void* data)
@@ -897,7 +803,7 @@ EXPORT void my_g_object_weak_ref(x64emu_t* emu, void* object, void* notify, void
     my->g_object_weak_ref(object, findWeakNotifyFct(notify), data);
 }
 
-EXPORT void my_g_signal_override_class_handler(x64emu_t* emu, char* name, void* gtype, void* callback)
+EXPORT void my_g_signal_override_class_handler(x64emu_t* emu, char* name, size_t gtype, void* callback)
 {
     my->g_signal_override_class_handler(name, gtype, findGCallbackFct(callback));
 }
@@ -943,10 +849,6 @@ EXPORT void* my_g_type_value_table_peek(x64emu_t* emu, size_t type)
     SetGClassPeek(my->g_type_class_peek);   \
     SetGTypeParent(my->g_type_parent);
 
-#ifdef ANDROID
-#define NEEDED_LIBS "libglib-2.0.so"
-#else
 #define NEEDED_LIBS "libglib-2.0.so.0"
-#endif
 
 #include "wrappedlib_init.h"
