@@ -16,14 +16,9 @@
 #include "librarian.h"
 #include "callback.h"
 
-const char* ldaprName =
-#ifdef ANDROID
-    "libldap_r-2.4.so"
-#else
-    "libldap_r-2.4.so.2"
-#endif
-    ;
-#define ALTNAME "libldap-2.5.so.0"
+const char* ldaprName = "libldap_r-2.4.so.2";
+#define ALTNAME "libldap_r-2.4.so"
+#define ALTNAME2 "libldap-2.5.so.0"
 #define LIBNAME ldapr
 
 #define ADDED_FUNCTIONS() \
@@ -69,17 +64,9 @@ EXPORT int my_ldap_sasl_interactive_bind_s(x64emu_t* emu, void* ld, void* dn, vo
 }
 
 #ifdef ANDROID
-#define NEEDED_LIB "liblber-2.4.so"
+#define NEEDED_LIBS "liblber-2.4.so"
 #else
-#define NEEDED_LIB "liblber-2.4.so.2"
+#define NEEDED_LIBS "liblber-2.4.so.2"
 #endif
 
-#define CUSTOM_INIT \
-    getMy(lib);         \
-    setNeededLibs(lib, 1, NEEDED_LIB);
-
-#define CUSTOM_FINI \
-    freeMy();
-
 #include "wrappedlib_init.h"
-
