@@ -32,6 +32,7 @@ typedef struct dlprivate_s {
     size_t      lib_sz;
     size_t      lib_cap;
     char*       last_error;
+    char        error_msg[100];
 } dlprivate_t;
 
 dlprivate_t *NewDLPrivate() {
@@ -45,6 +46,7 @@ void FreeDLPrivate(dlprivate_t **lib) {
 
 // dead_cells consider the "2" value to be some king of issue?
 #define MIN_NLIB 3
+int GetTID();
 
 void* my_dlopen(x64emu_t* emu, void *filename, int flag) EXPORT;
 void* my_dlmopen(x64emu_t* emu, void* mlid, void *filename, int flag) EXPORT;
@@ -57,13 +59,51 @@ void* my_dlvsym(x64emu_t* emu, void *handle, void *symbol, const char *vername)
 int my_dlinfo(x64emu_t* emu, void* handle, int request, void* info) EXPORT;
 
 #define LIBNAME libdl
-#ifdef ANDROID
-    const char* libdlName = "libdl.so";
-#else
-    const char* libdlName = "libdl.so.2";
-#endif
+const char* libdlName = "libdl.so.2";
+#define ALTNAME "libdl.so"
 
 #define CLEARERR    if(dl->last_error) {box_free(dl->last_error); dl->last_error = NULL;}
+void dl_clear_error()
+{
+    dlprivate_t *dl = my_context->dlprivate;
+    CLEARERR;
+}
+
+void dl_set_error(const char* msg)
+{
+    dlprivate_t *dl = my_context->dlprivate;
+    if(!dl->last_error)
+        dl->last_error = box_calloc(1, 129);
+    snprintf(dl->last_error, 129, "%s", msg);
+}
+
+library_t* dl_get_library(void* handle)
+{
+    dlprivate_t *dl = my_context->dlprivate;
+    CLEARERR
+    size_t nlib = (size_t)handle;
+    --nlib;
+    // size_t is unsigned
+    if(nlib>=dl->lib_sz) {
+        if(!dl->last_error)
+            dl->last_error = box_calloc(1, 129);
+        snprintf(dl->last_error, 129, "Bad handle %p)\n", handle);
+        return (void*)-1LL;
+    }
+    if(!dl->dllibs[nlib].count || !dl->dllibs[nlib].full) {
+        if(!dl->last_error)
+            dl->last_error = box_calloc(1, 129);
+        snprintf(dl->last_error, 129, "Bad handle %p (already closed))\n", handle);
+        return (void*)-1LL;
+    }
+    return dl->dllibs[nlib].lib;
+}
+
+char* dl_last_error()
+{
+    dlprivate_t *dl = my_context->dlprivate;
+    return dl->last_error;
+}
 
 void RemoveDlopen(library_t** lib, size_t idx)
 {
@@ -106,7 +146,7 @@ void* my_dlopen(x64emu_t* emu, void *filename, int flag)
             if(sys)
                 return sys;
         }
-        printf_dlsym(LOG_DEBUG, "Call to dlopen(\"%s\"/%p, %X)\n", rfilename, filename, flag);
+        printf_dlsym(LOG_DEBUG, "%04d|Call to dlopen(\"%s\"/%p, %X)\n", GetTID(), rfilename, filename, flag);
         // Transform any ${...} that maight be present
         while(strstr(rfilename, "${ORIGIN}")) {
             char* origin = box_strdup(my_context->fullpath);
@@ -254,7 +294,13 @@ void* my_dlmopen(x64emu_t* emu, void* lmid, void *filename, int flag)
 char* my_dlerror(x64emu_t* emu)
 {
     dlprivate_t *dl = my_context->dlprivate;
-    return dl->last_error;
+    //printf_dlsym(LOG_INFO, "call to dlerror(): \"%s\"\n", dl->last_error?dl->last_error:"(nil)"); //too chatty
+    if(!dl->last_error)
+        return NULL;
+    strncpy(dl->error_msg, dl->last_error, sizeof(dl->error_msg)-1);
+    box_free(dl->last_error);
+    dl->last_error = NULL;
+    return dl->error_msg;
 }
 
 KHASH_SET_INIT_INT(libs);
@@ -296,7 +342,7 @@ int my_dlsym_lib(library_t* lib, const char* rsymbol, uintptr_t *start, uintptr_
 
     return ret;
 }
-int GetTID();
+
 void* my_dlsym(x64emu_t* emu, void *handle, void *symbol)
 {
     (void)emu;
@@ -305,6 +351,8 @@ void* my_dlsym(x64emu_t* emu, void *handle, void *symbol)
     dlprivate_t *dl = my_context->dlprivate;
     uintptr_t start = 0, end = 0;
     char* rsymbol = (char*)symbol;
+    if(box64_is32bits && handle==(void*)0xffffffff)
+        handle = (void*)~0LL;
     CLEARERR
     printf_dlsym(LOG_DEBUG, "%04d|Call to dlsym(%p, \"%s\")%s", GetTID(), handle, rsymbol, dlsym_error?"":"\n");
     if(handle==NULL) {
@@ -323,7 +371,14 @@ void* my_dlsym(x64emu_t* emu, void *handle, void *symbol)
     }
     if(handle==(void*)~0LL) {
         // special case, look globably but no self (RTLD_NEXT)
-        elfheader_t *elf = FindElfAddress(my_context, *(uintptr_t*)R_RSP); // use return address to guess "self"
+        uintptr_t ret_addr = 0;
+        #ifdef BOX32
+        if(box64_is32bits)
+            ret_addr = from_ptri(ptr_t, R_ESP);
+        else
+        #endif
+            ret_addr = *(uintptr_t*)R_RSP;
+        elfheader_t *elf = FindElfAddress(my_context, ret_addr); // use return address to guess "self"
         if(GetNoSelfSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, elf, 0, -1, NULL, 0, NULL)) {
             printf_dlsym(LOG_NEVER, "%p\n", (void*)start);
             pthread_mutex_unlock(&mutex);
@@ -367,9 +422,17 @@ void* my_dlsym(x64emu_t* emu, void *handle, void *symbol)
             return NULL;
         }
     } else {
-        // still usefull?
+        // for "self" dlopen
         //  => look globably
-        if(GetGlobalSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, NULL, -1, NULL, 0, NULL)) {
+        //look in main elf first
+        int found = 0;
+        if(ElfGetSymTabStartEnd(my_context->elfs[0], &start, &end, rsymbol))
+            found = 1;
+        if(!found && GetGlobalSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, NULL, -1, NULL, 0, NULL)) 
+            found = 1;
+        if(!found && GetSymTabStartEnd(my_context->maplib, rsymbol, &start, &end))
+            found = 1;
+        if(found) {
             printf_dlsym(LOG_NEVER, "%p\n", (void*)start);
             pthread_mutex_unlock(&mutex);
             return (void*)start;
@@ -456,6 +519,8 @@ void* my_dlvsym(x64emu_t* emu, void *handle, void *symbol, const char *vername)
     int version = (vername)?2:-1;
     uintptr_t start, end;
     char* rsymbol = (char*)symbol;
+    if(box64_is32bits && handle==(void*)0xffffffff)
+        handle = (void*)~0LL;
     CLEARERR
     printf_dlsym(LOG_DEBUG, "Call to dlvsym(%p, \"%s\", %s)%s", handle, rsymbol, vername?vername:"(nil)", dlsym_error?"":"\n");
     if(handle==NULL) {
@@ -472,7 +537,14 @@ void* my_dlvsym(x64emu_t* emu, void *handle, void *symbol, const char *vername)
     }
     if(handle==(void*)~0LL) {
         // special case, look globably but no self (RTLD_NEXT)
-        elfheader_t *elf = FindElfAddress(my_context, *(uintptr_t*)R_RSP); // use return address to guess "self"
+        uintptr_t ret_addr = 0;
+        #ifdef BOX32
+        if(box64_is32bits)
+            ret_addr = from_ptri(ptr_t, R_ESP);
+        else
+        #endif
+            ret_addr = *(uintptr_t*)R_RSP;
+        elfheader_t *elf = FindElfAddress(my_context, ret_addr); // use return address to guess "self"
         if(GetNoSelfSymbolStartEnd(my_context->maplib, rsymbol, &start, &end, elf, 0, version, vername, 0, NULL)) {
                 printf_dlsym(LOG_NEVER, "%p\n", (void*)start);
             return (void*)start;
@@ -613,6 +685,11 @@ void closeAllDLOpenned()
 //extern void* _dlfcn_hook;
 #endif
 
+#define PRE_INIT\
+    if(1)                                                           \
+        lib->w.lib = dlopen(NULL, RTLD_LAZY | RTLD_GLOBAL);    \
+    else
+
 #define CUSTOM_FINI \
     closeAllDLOpenned();
 
