@@ -17,11 +17,8 @@
 #include "librarian.h"
 #include "myalign.h"
 
-#ifdef ANDROID
-    const char* pulseName = "libpulse.so";
-#else
-    const char* pulseName = "libpulse.so.0";
-#endif
+const char* pulseName = "libpulse.so.0";
+#define ALTNAME "libpulse.so"
 
 #define LIBNAME pulse
 
