@@ -17,11 +17,8 @@
 #include "librarian.h"
 #include "myalign.h"
 
-#ifdef ANDROID
-    const char* pulsesimpleName = "libpulse-simple.so";
-#else
-    const char* pulsesimpleName = "libpulse-simple.so.0";
-#endif
+const char* pulsesimpleName = "libpulse-simple.so.0";
+#define ALTNAME "libpulse-simple.so"
 
 #define LIBNAME pulsesimple
 
@@ -29,10 +26,6 @@
     if(box64_nopulse)   \
         return -1;
 
-#ifdef ANDROID
-#define NEEDED_LIBS "libpulse.so"
-#else
 #define NEEDED_LIBS "libpulse.so.0"
-#endif
 
 #include "wrappedlib_init.h"
