@@ -153,6 +153,7 @@ GO(SDL_GameControllerGetStringForButton, pFi)
 GO(SDL_GameControllerGetTouchpadFinger, iFpiipppp)
 GO(SDL_GameControllerGetType, uFp)
 GO(SDL_GameControllerGetVendor, WFp)
+GO(SDL_GameControllerHasAxis, iFpu)
 GO(SDL_GameControllerHasButton, uFpi)
 GO(SDL_GameControllerHasLED, iFp)
 GO(SDL_GameControllerHasRumble, iFp)
@@ -483,7 +484,7 @@ GO(SDL_NumHaptics, iFv)
 GO(SDL_NumJoysticks, iFv)
 GO(SDL_NumSensors, iFv)
 GOM(SDL_OpenAudio, iFEpp)
-GOM(SDL_OpenAudioDevice, iFEpippi)
+GOM(SDL_OpenAudioDevice, uFEpippi)
 GO(SDL_PauseAudio, vFi)
 GO(SDL_PauseAudioDevice, vFui)
 GO(SDL_PeepEvents, iFpiuuu)
@@ -731,8 +732,8 @@ GOM(SDL_WriteU8, uFEpu)
 
 GOM(SDL_RWseek, IFEpIi)
 GOM(SDL_RWtell, IFEp)
-GOM(SDL_RWread, uFEppuu)
-GOM(SDL_RWwrite, uFEppuu)
+GOM(SDL_RWread, LFEppLL)
+GOM(SDL_RWwrite, LFEppLL)
 GOM(SDL_RWclose, iFEp)
 
 GO2(SDL_mutexP, iFp, SDL_LockMutex)
