@@ -737,6 +737,7 @@ GOM(vkDestroyCuModuleNVX, vFEpUp)
 
 // VK_NVX_image_view_handle
 GO(vkGetImageViewAddressNVX, iFpUp)
+GO(vkGetImageViewHandle64NVX, UFpp)
 GO(vkGetImageViewHandleNVX, uFpp)
 
 // VK_NV_device_generated_commands
@@ -983,3 +984,14 @@ GO(vkCmdSetRenderingInputAttachmentIndicesKHR, vFpp)
 
 // VK_KHR_line_rasterization
 GO(vkCmdSetLineStippleKHR, vFpuW)
+
+// VK_KHR_external_semaphore_win32
+GO(vkGetSemaphoreWin32HandleKHR, iFppp)
+GO(vkImportSemaphoreWin32HandleKHR, iFpp)
+
+// VK_KHR_win32_keyed_mutex
+GO(wine_vkAcquireKeyedMutex, iFpUUu)
+GO(wine_vkReleaseKeyedMutex, iFpUU)
+
+// VK_NV_cooperative_matrix2
+GO(vkGetPhysicalDeviceCooperativeMatrixFlexibleDimensionsPropertiesNV, iFppp)
\ No newline at end of file
